
#ifndef __MRF_OPTIMIZATION_H
#define __MRF_OPTIMIZATION_H


#include <iostream>
#include <assert.h>
#include <vector>
#include <map>
#include <string.h>
#include <math.h>

#ifdef INCLUDE_MRFLIB
#include "mrf.h"
#include "ICM.h"
#include "GCoptimization.h"
#include "MaxProdBP.h"
#include "TRW-S.h"
#include "BP-S.h"

MRF::CostVal NodeCost(int nodeID, int labelID);
MRF::CostVal EdgeCost(int nodeID1, int nodeID2, int labelID1, int labelID2);
#endif

class MRFGraphValues
{
	public:
		MRFGraphValues(int nNodes, int nLabels);	
		virtual ~MRFGraphValues();
		virtual double NodeCost(int nodeID, int labelID) = 0;
		virtual double EdgeCost(int nodeID1, int nodeID2, int labeID1, int labelID2) = 0;
		virtual void CheckGraph();
	
		double m_fDoubleToIntScale;
		int m_iNNodes;
		int m_iNLabels;

};

class MRFGeneralGraph : public MRFGraphValues
{
	public:
		MRFGeneralGraph(int nNodes, int nLabels, double initCost = 1);
		virtual ~MRFGeneralGraph();
		virtual double NodeCost(int nodeID, int labelID);
		virtual double EdgeCost(int nodeID1, int nodeID2, int labeID1, int labelID2);
		virtual void SetNodeCost(int nodeID, int labelID, double value);
		virtual void SetEdgeValue(int nodeID1, int nodeID2, double value);
		virtual void CheckGraph();
	
		virtual int ConnectedComponents(std::vector<std::vector<int> > & ccs);
		
		
	protected:
		double * m_aNodeData;
		std::map<int, double> * m_aEdgeData;
};

#ifdef INCLUDE_MRFLIB
class MRFOptimization
{
	public:
		friend MRF::CostVal NodeCost(int nodeID, int labelID);
		friend MRF::CostVal EdgeCost(int nodeID1, int nodeID2, int labelID1, int labelID2);
	
		MRFOptimization(MRFGraphValues * graph);
		virtual ~MRFOptimization();
		void AddNeighbors(int nodeID1, int nodeID2);
	// NOTE: inOutNodeLabels can contain initial guess
		void Cluster(std::vector<int> * inOutNodeLabels);
		void TestGraph();
	
	protected:
		static MRFOptimization * s_CurrentOptimiaztion;	// one optimization at a time
		MRFGraphValues * m_pGraph;
	
		MRF * m_pMRFSolver;
		int m_iNEdges;
	
		void InitializeMRFLibSolver();
};
#endif
#endif

