#include "ParamParser.h"
#include <assert.h>

ParamParser::ParamParser(const VKString & inFilename, const char ** argv, int argc)
{
	std::ifstream file(inFilename.c_str());

	if (!file.is_open())
	{
		if (!argc>0)	//all params must be here
		{
			std::cout<<"[ERROR] Param File does not exist and no command line arguments provided: "<<inFilename.c_str()<<std::endl;
			assert(false);
		}
	}
	
	VKString currParam = "General";
		//read file, fill values
	VKStringList commands;
	VKString tempString(file);

	commands = tempString.replace("\n", " ").replace("\t", " ").split(" ");
	for (int i=0; i<argc; i++)	//read arguments, add or refill arguments
		commands.push_back(argv[i]);
	
	for (int i=0; i<commands.count(); i++)
	{		
		if (commands[i].startsWith("-") || commands[i].endsWith(":"))
		{
			currParam = commands[i].replace("-", "").replace(":","");
		}
		else if (currParam!="Invalid")
		{
			VKString field = commands[i++];
			VKString fld = field;
			if (fld.startsWith("+"))
				fld.replace("+", "");
			else
				m_ParamModules[currParam][fld].clear();
			if (commands.count()<=i)
			{
				assert(commands.count()>i);
			}
			if (commands[i].startsWith("[") && !commands[i].endsWith("]"))				//loop over multiple param values
			{
				assert(commands.count()>i);
				if (commands[i]=="[")
					i++;
				while(commands[i]!="]")
				{
					bool terminate = commands[i].endsWith("]");
					m_ParamModules[currParam][fld].push_back(commands[i].replace("[", "").replace("]", ""));
					if (terminate)	break;
					i++;
				}
			}
			else		//set single param value
			{
				m_ParamModules[currParam][fld].push_back(commands[i].replace("[", "").replace("]", ""));
			}
		}
	}
	file.close();
}

void ParamParser::WriteParams(const VKString & outFile)
{
	std::ofstream textStream(outFile.c_str());
	assert(textStream.is_open());
	for (std::map<VKString, std::map<VKString, VKStringList> >::iterator iter = m_ParamModules.begin();
		 iter!=m_ParamModules.end(); iter++)
	{
		textStream<<iter->first.c_str()<<":\n";
		for (std::map<VKString, VKStringList>::iterator iter2 = iter->second.begin();
			 iter2!=iter->second.end(); iter2++)
		{
			textStream<<"\t"<<iter2->first.c_str()<<"\t["<<iter2->second.join(", ").c_str()<<"]\n";
		}
	}	
	textStream.close();
}

void ParamParser::PrintParams()
{
	for (std::map<VKString, std::map<VKString, VKStringList> >::iterator iter = m_ParamModules.begin();
		 iter!=m_ParamModules.end(); iter++)
	{
		std::cout<<"Module = "<<iter->first.c_str()<<std::endl;
		for (std::map<VKString, VKStringList>::iterator iter2 = iter->second.begin();
			 iter2!=iter->second.end(); iter2++)
		{
			std::cout<<"\t'"<<iter2->first.c_str()<<"' = [";
			std::cout<<iter2->second.join(", ").c_str()<<"]"<<std::endl;
		}
	}
}

const VKStringList & ParamParser::GetStrValues(const VKString & module, 
									   const VKString & valName, bool & valid)
{
	valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();

	if (valid)
		return m_ParamModules[module][valName];
	else
		return m_EmptyList;
}

VKString ParamParser::GetStrValue(const VKString & module, 
								  const VKString & valName, bool & valid)
{	
	valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();

	if (valid)
		return m_ParamModules[module][valName][0];
	else
		return VKString();
}

double ParamParser::GetDoubleValue(const VKString & module, 
								   const VKString & valName, bool & valid)
{
	valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();
	if (valid)
		return m_ParamModules[module][valName][0].toDouble(&valid);
	else
		return -1.;
}

int ParamParser::GetIntValue(const VKString & module, 
							 const VKString & valName, bool & valid)
{
	valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();
	if (valid)
		return m_ParamModules[module][valName][0].toInt(&valid);
	else
		return -1;
}

std::vector<double> ParamParser::GetDoubleValues(const VKString & module, 
												 const VKString & valName, bool & valid)
{
	valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();
	std::vector<double> newVals;
	if (valid)
	{
		VKStringList list = m_ParamModules[module][valName];

		bool currValid;
		for (int i=0; i<list.count(); i++)
		{
			newVals.push_back(list[i].toDouble(&currValid));
			valid = currValid && valid;
		}
	}
	return newVals;
}

std::vector<int> ParamParser::GetIntValues(const VKString & module, 
										   const VKString & valName, bool & valid)
{
	valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();
	std::vector<int> newVals;

	if (valid)
	{
		VKStringList list = m_ParamModules[module][valName];

		bool currValid;
		for (int i=0; i<list.count(); i++)
		{
			newVals.push_back(list[i].toInt(&currValid));
			valid = currValid && valid;
		}
	}
	return newVals;
}


const VKStringList & ParamParser::GetStrValues(const VKString & module, 
											   const VKString & valName)
{
	m_Valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();
	AssertParamValid(module, valName, "STRs", "find");
	return m_ParamModules[module][valName];
}

VKString ParamParser::GetStrValue(const VKString & module, 
								  const VKString & valName)
{	
	m_Valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();
	AssertParamValid(module, valName, "STR", "find");	
	return m_ParamModules[module][valName][0];
}

double ParamParser::GetDoubleValue(const VKString & module, 
								   const VKString & valName)
{
	m_Valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();
	AssertParamValid(module, valName, "DBL", "find");	
	double retVal = m_ParamModules[module][valName][0].toDouble(&m_Valid);
	AssertParamValid(module, valName, "DBL", "parse");
	return retVal;
}

int ParamParser::GetIntValue(const VKString & module, 
							 const VKString & valName)
{
	m_Valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();
	AssertParamValid(module, valName, "INT", "find");
	int retVal = m_ParamModules[module][valName][0].toInt(&m_Valid);
	AssertParamValid(module, valName, "INT", "parse");
	return retVal;
}

std::vector<double> ParamParser::GetDoubleValues(const VKString & module, 
												 const VKString & valName)
{
	m_Valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();
	AssertParamValid(module, valName, "DBLs", "find");
	std::vector<double> newVals;
	VKStringList list = m_ParamModules[module][valName];
	
	for (int i=0; i<list.count(); i++)
	{
		newVals.push_back(list[i].toDouble(&m_Valid));
		AssertParamValid(module, valName, "DBLs", "parse");
	}
	return newVals;
}

std::vector<int> ParamParser::GetIntValues(const VKString & module, 
										   const VKString & valName)
{
	m_Valid = m_ParamModules[module].find(valName)!=m_ParamModules[module].end();
	std::vector<int> newVals;
	AssertParamValid(module, valName, "INTs", "find");
	VKStringList list = m_ParamModules[module][valName];
	
	for (int i=0; i<list.count(); i++)
	{
		newVals.push_back(list[i].toInt(&m_Valid));
		AssertParamValid(module, valName, "INTs", "parse");
	}
	return newVals;
}

void ParamParser::AssertParamValid(const VKString & module, const VKString & paramName,
								   const char * datatype, const char * find_parse)
{
	if (!m_Valid)
	{
		std::cout<<"[ERROR] Cannot "<<find_parse<<" "<<datatype<<" param.";
		std::cout<<"Module="<<module.c_str()<<" ParamName="<<paramName.c_str()<<std::endl;
//		if (strcmp(find_parse, "parse")==0)
//			std::cout<<"ParamValue="<<m_ParamModules[module][paramName].c_str()<<std::endl;
		assert(m_Valid);
	}	
}


