/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#include "AppCreateDB.h"
#include "CollectionGroundTruth.h"

AppCreateDB::AppCreateDB(AnalysisPipeline & pipeline, const VKString & moduleName)
: PipelineStepApplication(pipeline, moduleName)
{
	if (pipeline.GetDB()->NumModels()==0)
		m_State = CREATEDB_KEYWORD;
	else
		m_State = CREATEDB_PROCESSING;
    m_bRedownloadGoogleData = false;
    m_ProcessingStateNames.push_back("Selecting State");
    m_ProcessingStateNames.push_back("Delete whole obects");
    m_ProcessingStateNames.push_back("Delete / Copy Connected Components");
    m_ProcessingStateNames.push_back("Delete / Copy Single Faces");
    m_ProcessingStateNames.push_back("Delete Faces");
	m_ProcessingStateNames.push_back("Click Ground Truth Points");
	m_ProcessingStateNames.push_back("Add Ground Truth Label");
    m_ProcessingState = PROCESS_SELECT_STATE;
	m_iNumToDownload = 0;
	m_sGroundTruthPntSetName = "FeatPnts";
    
    m_aIsTriangleInCC = NULL;
    m_iNTriangles = 0;
    m_iDrawingFlags = DRAW_CC | DRAW_MESH;
    m_TriangsForModelCanDraw = NULL;
    m_bRightMouseButton = false;
}

bool AppCreateDB::CaptureKeyPressed(DatabaseVisWindow * window, 
                                    const char key, int mouseX, int mouseY)
{
    bool keyCaptured = false;
    bool keyAlpha = ((key>='A' && key <= 'Z') || (key>='a' && key <= 'z') 
					 || key==' ' || key=='_');
    bool keyNumeric = (key >= '0' && key <= '9');
    
    if (m_State==CREATEDB_KEYWORD && (keyAlpha || keyNumeric))
    {
		char str[2];
		str[0] = key;
		str[1] = 0;
		m_sQueryKeyword += VKString(str);
        keyCaptured = true;
    }
    else if (m_State==CREATEDB_NUM_MODELS && keyNumeric)
    {
        m_iNumToDownload = m_iNumToDownload * 10 + (int)(key - '0');
        keyCaptured = true;
    }
    else if (m_State==CREATEDB_INPUTDB && keyNumeric)
    {
        m_InputDB = (CreateDBInputDB)(int)(key - '1');
        std::cout<<"InputDB: "<<InputDBName(m_InputDB).c_str()<<std::endl;
        return CaptureKeyPressed(window, '#', mouseX, mouseY);
    }
	else if (m_State==CREATEDB_PROCESSING 
			 && m_ProcessingState==PROCESS_GROUND_TRUTH_NEW_LABEL
			 && (keyAlpha || keyNumeric))
	{
		char str[2];
		str[0] = key;
		str[1] = 0;
		m_sNewGroundTruthLabel += VKString(str);
		keyCaptured = true;
	}
    
    if (keyCaptured)
    {
        window->GetRenderer()->UpdateScreen();
        return true;
    }
    
	keyCaptured = true;
	switch (key)
	{
		case '`':
			std::cout<<"DB Create App:"<<std::endl;
			std::cout<<"\t;\t - Get models / reset. Enter keyword, nModels, src";
            std::cout<<" finish each by pound: '#'. "<<std::endl;
            std::cout<<"\tw\t - Write DB "<<std::endl;
            std::cout<<"\tW\t - Write a Copy of DB (NOT IMPLEMENTED) "<<std::endl;
            std::cout<<"\t0-9\t - delete a model in corresponding subwindow "<<std::endl;
            std::cout<<"\ts\t - Change processing state "<<std::endl;
            std::cout<<"\t?\t - View CC processing options "<<std::endl;
            std::cout<<"\tz\t - undo delete (NOT IMPLEMENTED)"<<std::endl;
            std::cout<<"\tm\t - toggle drawing triangles not in CC"<<std::endl;
            std::cout<<"\tM\t - toggle drawing triangle in selected CC"<<std::endl;
			std::cout<<"\t~/!/@\t - print PSB keywords levels (1, 2, all)"<<std::endl;
			std::cout<<"\t$\t - print Corrs keywords levels (1, 2, all)"<<std::endl;
			keyCaptured = false;
			break;
		case ' ':
			if (m_State==CREATEDB_PROCESSING
				&& m_ProcessingState==PROCESS_GROUND_TRUTH)
			{
				SelectGroundTruthPoint(window, mouseX, mouseY);
			}
			else
				std::cout<<"[WARNING] Space only works when selecting ground truth"<<std::endl;
			keyCaptured = true;
			break;
		case '~':
			PrintPSBKeywords(1);
			break;
		case '!':
			PrintPSBKeywords(2);
			break;
		case '@':
			PrintPSBKeywords(-1);
			break;
		case '$':
			FillCorrsKeywordToFilesMap();
			std::cout<<m_sCorrsHelpStr.c_str()<<std::endl;
			break;
        case '#':
            if (m_State==CREATEDB_KEYWORD)
                m_State = CREATEDB_NUM_MODELS;
            else if (m_State==CREATEDB_NUM_MODELS)
                m_State = CREATEDB_INPUTDB;
            else if (m_State==CREATEDB_INPUTDB)
                m_State = CREATEDB_SEARCHING;
			else if (m_State==CREATEDB_PROCESSING 
					 && m_ProcessingState==PROCESS_GROUND_TRUTH_NEW_LABEL)
			{
				std::cout<<"Updated Ground Truth Point Labels:"<<std::endl;
				AddGroundTruthPointLabel(m_sNewGroundTruthLabel);
				m_sNewGroundTruthLabel = "";
				PrintGroundTruthPointLabels();
			}
					 
            
            std::cout<<"Updated state: "<<AppStateName(m_State).c_str()<<std::endl;
            
            if (m_State==CREATEDB_SEARCHING)
            {
                if (m_InputDB==GOOGLE_WAREHOUSE)
                    DownloadGoogleWarehouse(m_sQueryKeyword, m_iNumToDownload, true, true);
                else if (m_InputDB==PRINCETON_SHAPE_BENCHMARK)
                    GetPrincetonShapeBenchmark(m_sQueryKeyword, m_iNumToDownload);
                else if (m_InputDB==CORRESPONDENCE_BENCHMARK)
                    GetCorrespondenceBenchmark(m_sQueryKeyword, m_iNumToDownload);
                m_Pipeline.GetDB()->InitRenderingParams(window, m_Pipeline.RenderingParams());
            }            
            break;
        case 'H':
        {
            window->m_NewModelOrder.clear();
            for (int i=0; i<m_Pipeline.GetDB()->NumModels(); i++)
                window->m_NewModelOrder.push_back(i);
            window->m_iCurrModelTraversalID = 0;
            break;
        }
        case ';':
            if (m_State==CREATEDB_PROCESSING
				&& m_ProcessingState!=PROCESS_GROUND_TRUTH_NEW_LABEL)
                m_State=CREATEDB_KEYWORD;
            else
			{
                m_State=CREATEDB_PROCESSING;
				if (m_ProcessingState==PROCESS_GROUND_TRUTH_NEW_LABEL)
					m_ProcessingState=PROCESS_GROUND_TRUTH;
			}
			
            m_sQueryKeyword = "";
            m_iNumToDownload = 0;
            m_InputDB = GOOGLE_WAREHOUSE;
			m_sNewGroundTruthLabel="";
            std::cout<<"Updated state: "<<AppStateName(m_State).c_str()<<std::endl;
            break;
        case 'z':
			if (m_ProcessingState==PROCESS_DELETE_WHOLE)
				UndoDelete();
			else if (m_ProcessingState==PROCESS_GROUND_TRUTH)
				UndoGroundTruthPointLabel();
            break;
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
            if (m_ProcessingState==PROCESS_DELETE_WHOLE)
                DeleteModelInSubwindow(window, (int)(key-'0'));
            else if (m_ProcessingState==PROCESS_SELECT_CONN_COMP
                     || m_ProcessingState==PROCESS_SELECT_FACE)
                MainProcessedModel(window, (int)(key-'0'));
            else if (m_ProcessingState == PROCESS_SELECT_STATE)
            {
                int procState = (int)(key-'0');

                if (procState  >= m_ProcessingStateNames.count())
                    procState = 0;
                m_ProcessingState = (ProcessingState)procState;
                std::cout<<"Processing State: ";
                std::cout<<m_ProcessingStateNames[procState].c_str()<<std::endl;
				if (m_ProcessingState==PROCESS_GROUND_TRUTH)
				{
					if (!m_Pipeline.GetDB()->m_PointSetNames.contains(m_sGroundTruthPntSetName))
						m_Pipeline.GetDB()->m_PointSetNames.push_back(m_sGroundTruthPntSetName);
					for (int i=0; i<m_Pipeline.GetDB()->NumModels(); i++)
					{
						Model3D * model = m_Pipeline.GetDB()->GetModelByID(i);
						model->RenderingParams()->SetPointSetToRender(m_sGroundTruthPntSetName);
					}
						
				}
            }
            break;
        case '?':
        case '/':
        case 'd':
        case 'c':
            if (m_ProcessingState==PROCESS_SELECT_CONN_COMP
                || m_ProcessingState==PROCESS_SELECT_FACE)
            {
                m_bSelectCCShowHelp = false;
                if (key=='/')
                    MainProcessedModel(window, 0);
                else if (key == 'd')
                    DeleteSelectedFaces();
                else if (key=='c')
                    CreateNewModelFromSelectedFaces();
                else if (key=='?')
                    m_bSelectCCShowHelp = true;
            }
            break;
        case 's':
            m_ProcessingState = PROCESS_SELECT_STATE;
            break;
        case 'm':
            if ((m_iDrawingFlags & DRAW_MESH)==0)
                m_iDrawingFlags |= DRAW_MESH;
            else
                m_iDrawingFlags &= ~DRAW_MESH;
            std::cout<<"DrawMesh: "<<((m_iDrawingFlags & DRAW_MESH)!=0)<<std::endl;
            break;
        case 'M':
            if ((m_iDrawingFlags & DRAW_CC)==0)
                m_iDrawingFlags |= DRAW_CC;
            else
                m_iDrawingFlags &= ~DRAW_CC;
            std::cout<<"DrawCC: "<<((m_iDrawingFlags & DRAW_CC)!=0)<<std::endl;
            break;
        case 'w':
            WriteDB();
            break;
        case 'W':
            //WriteDBCopy();
			if (m_ProcessingState==PROCESS_GROUND_TRUTH)
				WriteGroundTruthPointLabels();
			WriteDB();
            break;
		case 'O':
			ReorderModels();
			break;
        case '\\':
            m_bRightMouseButton = !m_bRightMouseButton;
            break;
        case 27:
            exit(0);
            break;
        default:
            keyCaptured = false;
            break;
    }

    if (keyCaptured)
        window->GetRenderer()->UpdateScreen();
    return keyCaptured;
}

bool AppCreateDB::CaptureMouseButtonEvent(DatabaseVisWindow * window, int x, int y, 
                                          VisWindow::MouseButton mouseButtonID, 
                                          VisWindow::MouseButtonState newState)
{
    if (window->IsMouseButtonDown(VisWindow::MOUSE_RIGHT)
		|| (m_bRightMouseButton && window->IsMouseButtonDown(VisWindow::MOUSE_LEFT)))
	{
        SelectFace(window, x, y);
        return true;
    }
    return false;
}

bool AppCreateDB::CaptureMouseMotionEvent(DatabaseVisWindow * window, 
                                          int x, int y, int dx, int dy)
{
    if (window->IsMouseButtonDown(VisWindow::MOUSE_RIGHT)
		|| (m_bRightMouseButton && window->IsMouseButtonDown(VisWindow::MOUSE_LEFT)))
	{
        SelectFace(window, x, y);
        return true;
    }    
    return false;
}

void AppCreateDB::SelectFace(DatabaseVisWindow * window, int x, int y)
{
    if (m_State == CREATEDB_PROCESSING)
    {
        Model3D * model = m_Pipeline.GetDB()->GetModelInSubwindow(0);
        assert(model!=NULL);
        ModelPoint p = model->RenderingParams()->QuickIntersection(x, y, 0, 0);
        if (!p.Invalid())
        {
            int f = p.TriID();
            assert(f>=0 && f < model->GetMesh()->NFaces());
            if (m_ProcessingState==PROCESS_SELECT_CONN_COMP)
                AddSelection(f);
            else if (m_ProcessingState==PROCESS_SELECT_FACE)
            {
                UpdateTriangleCanDraw();
                if (!m_aIsTriangleInCC[f])
                {
                    m_SelectedFaces.push_back(f);
                    m_aIsTriangleInCC[f] = true;
                }
            }
            else if (m_ProcessingState==PROCESS_ERASE_FACE)
                DeleteFace(f);
        }
    }
}


VKString AppCreateDB::GetAppType()
{
    return "AppCreateDB";
}

void AppCreateDB::Execute()
{
    // try to init db
	
	ParamParser & params = m_Pipeline.m_Params;
	VKString keyword, srcDB;
	int nModels;
	VKStringList modelsToKeep;

	bool valid = true;
	if (valid)
		keyword = params.GetStrValue("CreateDBApp", "INPUT_Keyword", valid);
	valid = valid && keyword!="none";

	if (valid)
		srcDB = params.GetStrValue("CreateDBApp", "INPUT_SrcDB", valid);
	valid = valid && srcDB!="none";

	if (valid)
		nModels = params.GetIntValue("CreateDBApp", "INPUT_NModels", valid);

	if (valid)
		modelsToKeep = params.GetStrValues("CreateDBApp", "INPUT_ModelsToKeep", valid);
	valid = valid && modelsToKeep.count()>0 && modelsToKeep[0]!="none";
	
	bool keepInMem = true;
	if (params.GetStrValue("Pipeline", "Visualize")!="true")
		keepInMem = false;
	
//	int nModelsToKeep = nModels;
//	if (valid)
//		nModelsToKeep = params.GetIntValue("CreateDBApp", "INPUT_KeepNBest", valid);
		
	if (valid)
	{
		keyword = keyword.replace("_", " ");
		std::cout<<"Initializing Database: "<<keyword.c_str()<<" in "<<srcDB.c_str();
		std::cout<<". NModels="<<nModels<<" NKeep="<<modelsToKeep.count()<<std::endl;
	
		if (srcDB=="GOOGLE_WAREHOUSE")
			DownloadGoogleWarehouse(keyword, nModels, true, keepInMem);
		else if (srcDB=="PRINCETON_SHAPE_BENCHMARK")
			GetPrincetonShapeBenchmark(keyword, nModels);
		else if (srcDB=="CORRESPONDENCE_BENCHMARK")
			GetCorrespondenceBenchmark(keyword, nModels);
		else
		{
			std::cout<<"[ERROR] Unknown src DB: "<<srcDB.c_str()<<std::endl;
			assert(false);
		}
		
		assert(modelsToKeep.count()>0);
		if (modelsToKeep[0]!="all")
		{
			for (int i=0; i<m_Pipeline.GetDB()->NumModels(); i++)
			{
				VKString modelName = m_Pipeline.GetDB()->GetModelByID(i)->ModelName();
				
				bool keep = false;
				for (int j=0; j<modelsToKeep.count() && !keep; j++)
					keep = (modelName==modelsToKeep[j]);
				
				if (!keep)
				{
					m_Pipeline.GetDB()->RemoveModel(i);
					i--;
				}
			}
			std::cout<<"Kept Models: "<<m_Pipeline.GetDB()->NumModels()<<std::endl;
		}
		WriteDB();
	}
}

void AppCreateDB::Draw(DatabaseVisWindow * window)
{
	
    if (m_State==CREATEDB_KEYWORD
        || m_State==CREATEDB_NUM_MODELS
        || m_State==CREATEDB_INPUTDB)
    {
        VKStringList strs;
        
        strs.push_back(VKString("Keyword: ")+m_sQueryKeyword);        
        strs.push_back(VKString("NModels: ")+VKString::number(m_iNumToDownload));
        strs.push_back(VKString("Input DB: ")+InputDBName(m_InputDB));
                
        int selected = 0;
        if (m_State==CREATEDB_KEYWORD)
        {
            selected = 0;
            strs[0]+="_";
        }
        else if (m_State==CREATEDB_NUM_MODELS)
        {
            selected = 1;
            if (m_iNumToDownload!=0)
                strs[1]+="_";
            else
                strs[1] = VKString("NModels: ") + "_";
        }
        else if (m_State==CREATEDB_INPUTDB)
        {
            selected = 2;
            strs[2] = "Input DB: ";
            strs.push_back("          1 : Google Warehouse");
            strs.push_back("          2 : Princeton Shape Benchmark");
            strs.push_back("          3 : Correspondence Benchmark");
        }
        
        for (int i=0; i<strs.count(); i++)
        {
            double r = 1,g = 0, b = 0;
            if (selected==i || selected==2 && i>=2)
                {r=0; b=1;}
            
			if (window->m_WriteImg==DatabaseVisWindow::WRITE_SCREENSHOT_NONE)
				window->GetRenderer()->RenderText(R2Point(-.2, .6 - i *.2), strs[i],
												  r, g, b, VisWindowRenderer::TEXT_SIZE_LARGE);
        }
    }
    else if (m_State == CREATEDB_PROCESSING)
    {
		if (m_ProcessingState == PROCESS_DELETE_WHOLE)
		{
			ModelDatabase * db = m_Pipeline.GetDB();
			std::vector<int> & mToModelID = db->GetRenderParams()->m_mWindowToModelID;
			for (int i=0; i<vkMin(4, mToModelID.size()); i++)
			{
				Model3D * model = db->GetModelByID(mToModelID[i]);
				int nVertices = model->GetMesh()->NVertices();
				VKString str=VKString("[") + VKString::number(i) + "] ";
				str = str + model->ModelName() + " : " +VKString::number(nVertices);
				window->GetRenderer()->RenderText(R2Point(-.2, .8 - i *.05), str.c_str(),
												  .8, .3, .3, VisWindowRenderer::TEXT_SIZE_MEDIUM);
			}
		}
		
		if (m_ProcessingState == PROCESS_GROUND_TRUTH_NEW_LABEL
			&& window->m_WriteImg==DatabaseVisWindow::WRITE_SCREENSHOT_NONE)
		{
			VKString str = VKString("GT[")+m_GroundTruthLabels.count()+"] : ";
			str += m_sNewGroundTruthLabel;
            window->GetRenderer()->RenderText(R2Point(-.2, .8), str,
                                              .8, .3, .3, VisWindowRenderer::TEXT_SIZE_LARGE);			
		}
        else if (m_ProcessingState == PROCESS_SELECT_STATE
				 && window->m_WriteImg==DatabaseVisWindow::WRITE_SCREENSHOT_NONE)
        {
            window->GetRenderer()->RenderText(R2Point(-.2, .8), "Processing Ops:",
                                              0, .5, .5, VisWindowRenderer::TEXT_SIZE_LARGE);
            
            for (int i=1; i<m_ProcessingStateNames.count(); i++)
                window->GetRenderer()->RenderText(R2Point(0, .8 - .2*i), 
                                                  VKString::number(i) + " : " +
                                                  m_ProcessingStateNames[i],
                                                  0, .5, .5, 
                                                  VisWindowRenderer::TEXT_SIZE_LARGE);
			
        }
        else
        {
			if (window->m_WriteImg==DatabaseVisWindow::WRITE_SCREENSHOT_NONE)
			{
				VKString nModelsStr = VKString("NModels ");
				nModelsStr += VKString::number(m_Pipeline.GetDB()->NumModels());
				
				window->GetRenderer()->RenderText(R2Point(-.2, .9), nModelsStr,
												  0, .5, .5, VisWindowRenderer::TEXT_SIZE_LARGE);
				
				if (m_bSelectCCShowHelp)
				{
					window->GetRenderer()->RenderText(R2Point(0, .5 - .1*0), 
													  VKString("c : Move CC to a separate model") ,
													  0, .5, .5, 
													  VisWindowRenderer::TEXT_SIZE_MEDIUM);
					window->GetRenderer()->RenderText(R2Point(0, .5 - .1*1), 
													  VKString("d : Delete CC") ,
													  0, .5, .5, 
													  VisWindowRenderer::TEXT_SIZE_MEDIUM);
					window->GetRenderer()->RenderText(R2Point(0, .5 - .1*2), 
													  VKString("/ : Clear All") ,
													  0, .5, .5, 
													  VisWindowRenderer::TEXT_SIZE_MEDIUM);
				}
			}
            
            if (m_Pipeline.GetDB()->NumModels()>0)
            {
                UpdateTriangleCanDraw();
                DrawMesh(window, 0);
            }
        }
    }
}
void AppCreateDB::UpdateTriangleCanDraw()
{
    Model3D * model = m_Pipeline.GetDB()->GetModelInSubwindow(0);
    if (model != m_TriangsForModelCanDraw)
    {
        if (m_iNTriangles < model->GetMesh()->NFaces())
        {
            if (m_aIsTriangleInCC!=NULL)
                delete [] m_aIsTriangleInCC;
            m_iNTriangles = model->GetMesh()->NFaces();
            m_aIsTriangleInCC = new bool[m_iNTriangles];
        }
        
        for (int i=0; i<model->GetMesh()->NFaces(); i++)
            m_aIsTriangleInCC[i] = false;
        
        for (int i=0; i<(int)m_SelectedFaces.size(); i++)
            m_aIsTriangleInCC[m_SelectedFaces[i]] = true;
        
        m_TriangsForModelCanDraw = model;
    }   // update
}

void AppCreateDB::DrawMesh(DatabaseVisWindow * window, int subwindowID)
{
    Model3D * model = m_Pipeline.GetDB()->GetModelInSubwindow(subwindowID);
    if (model==NULL)
        return;
    
    model->RenderingParams()->LoadMyCamera(subwindowID);
	
	static GLfloat materialCC[4];
	static GLfloat materialNotCC[4];
    
    if ((m_iDrawingFlags & DRAW_CC)!=0)
        model->RenderingParams()->DrawIntersectInfoToFramebuffer(subwindowID);
    else
        model->RenderingParams()->DrawIntersectInfoToFramebuffer(subwindowID, 
                                                                 m_aIsTriangleInCC);
    
    glEnable(GL_LIGHTING);
    glEnable(GL_POLYGON_SMOOTH);
    glShadeModel(GL_SMOOTH);
	glDisable(GL_CULL_FACE);
	glColor3d(1.,1., 1.);
	glDisable(GL_TEXTURE);

	window->StandardSurfaceColor(materialNotCC);
    materialCC[0] = .5;
    materialCC[1] = 0;
    materialCC[2] = 0;
    materialCC[3] = materialNotCC[3];

    const R3Mesh * mesh = model->GetMesh();
    
    assert(m_iNTriangles >= mesh->NFaces());
	glBegin(GL_TRIANGLES);
	for (int i=0; i<mesh->NFaces(); i++)
	{                
        if (m_aIsTriangleInCC[i])
        {
            if ((m_iDrawingFlags & DRAW_CC)==0)
                continue;
            else
                glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, materialCC);
        }
        else
        {
            if ((m_iDrawingFlags & DRAW_MESH)==0)
                continue;
            else
                glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, materialNotCC);
        }

		const R3MeshFace * face = mesh->Face(i);
		R3Point p1 = mesh->VertexPosition(mesh->VertexOnFace(face, 0));
		R3Point p2 = mesh->VertexPosition(mesh->VertexOnFace(face, 1));
		R3Point p3 = mesh->VertexPosition(mesh->VertexOnFace(face, 2));
        R3Vector n1 = mesh->VertexNormal(mesh->VertexOnFace(face, 0));
        R3Vector n2 = mesh->VertexNormal(mesh->VertexOnFace(face, 1));
        R3Vector n3 = mesh->VertexNormal(mesh->VertexOnFace(face, 2));
        glNormal3d(n1.X(), n1.Y(), n1.Z());
        glVertex3d(p1.X(), p1.Y(), p1.Z());
        
        glNormal3d(n2.X(), n2.Y(), n2.Z());
        glVertex3d(p2.X(), p2.Y(), p2.Z());
        
        glNormal3d(n3.X(), n3.Y(), n3.Z());
        glVertex3d(p3.X(), p3.Y(), p3.Z());        
	}
	glEnd();
    model->RenderingParams()->RestoreCameraAfterLoad(subwindowID);
}
                     
void AppCreateDB::WriteDB()
{
    m_Pipeline.GetDB()->WriteToDir();
}

void AppCreateDB::WriteDBCopy()
{
    assert(false);
}

void AppCreateDB::WriteResultsChild(std::ofstream & textStream)
{
}

bool AppCreateDB::ReadResultsChild(std::ifstream & textStream)
{
	VKString labels = m_Pipeline.GetDB()->GetAppDir(m_sAppName) + "FeatPnts.labels";
    if (ModelDatabase::FileExists(labels))
	{
		std::ifstream textStream(labels.c_str());
		assert(textStream.is_open());
		int lID;
		VKString tempText;
		while (!textStream.eof())
		{
			textStream>>lID;
			if (textStream.eof())
				break;
			tempText.readToken(textStream);
			m_GroundTruthLabels.push_back(tempText);
		}
		textStream.close();
		std::cout<<"Loaded GT:"<<std::endl;
		PrintGroundTruthPointLabels();
	}
}

void AppCreateDB::DeleteSelectedFaces()
{
    std::sort(m_SelectedFaces.begin(), m_SelectedFaces.end());
    for (int i=(int)m_SelectedFaces.size()-1; i>=0; i--)
        DeleteFace(m_SelectedFaces[i]);
    m_SelectedFaces.clear();
    
    Model3D * model = m_Pipeline.GetDB()->GetModelInSubwindow(0);
    assert(model!=NULL);
    R3Vector up = model->RenderingParams()->GetCamUp(0);
    R3Vector tow = model->RenderingParams()->GetCamTowards(0);
    model->RenderingParams()->UpdateCamera(up, tow);
}

void AppCreateDB::DeleteFace(int face)
{
    m_TriangsForModelCanDraw = NULL;
    Model3D * model = m_Pipeline.GetDB()->GetModelInSubwindow(0);
    assert(model!=NULL);
    model->RemoveFaceOnMesh(face);
}

void AppCreateDB::CreateNewModelFromSelectedFaces()
{
    m_TriangsForModelCanDraw = NULL;
    assert(m_SelectedFaces.size()>0);
 
    // create model's copy
    Model3D * model = m_Pipeline.GetDB()->GetModelInSubwindow(0);
    assert(model!=NULL);
    
    Model3D * newModel = model->SplitModel(m_SelectedFaces);

    // add model to DB
    m_Pipeline.GetDB()->AddModel(newModel, false, true);

    // render model in supplemental window 1
    m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID[1] = newModel->ModelID();

    m_SelectedFaces.clear();
    
    R3Vector up = model->RenderingParams()->GetCamUp(0);
    R3Vector tow = model->RenderingParams()->GetCamTowards(0);
    model->RenderingParams()->UpdateCamera(up, tow);

    up = newModel->RenderingParams()->GetCamUp(-1);
    tow = newModel->RenderingParams()->GetCamTowards(-1);
    newModel->RenderingParams()->UpdateCamera(up, tow);
}

void AppCreateDB::AddSelection(int seedFace)
{
    Model3D * model = m_Pipeline.GetDB()->GetModelInSubwindow(0);
    assert(model!=NULL);
    UpdateTriangleCanDraw();

    std::vector<int> saveCC;
    if (!m_aIsTriangleInCC[seedFace])
        model->FillCCFaces(seedFace, saveCC);
    
    for (int i=0; i<(int)saveCC.size(); i++)
    {
        if (!m_aIsTriangleInCC[saveCC[i]])
        {
            m_SelectedFaces.push_back(saveCC[i]);
            m_aIsTriangleInCC[saveCC[i]] = true;
        }
    }
}

void AppCreateDB::MainProcessedModel(DatabaseVisWindow * window, int fromSubwindow)
{
    if (m_Pipeline.GetDB()->NumModels() == 0)
        return;
    int modelID = m_Pipeline.GetDB()->GetModelIDInSubwindow(fromSubwindow);
    int model0 = m_Pipeline.GetDB()->GetModelIDInSubwindow(0);
    m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID[0] = modelID;
    m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID[fromSubwindow] = model0;
    m_TriangsForModelCanDraw = NULL;
    m_SelectedFaces.clear();
}

void AppCreateDB::DownloadGoogleWarehouse(const VKString & keyword, int nModels, 
										  bool loadToMem, bool keepInMem)
{
    ParamParser & par = m_Pipeline.m_Params;
    VKString dataDir = par.GetStrValue(m_sModuleName, "INPUT_DataDir");
    VKString googleDir = par.GetStrValue(m_sModuleName, "INPUT_GoogleSubdir");
	DownloadGoogleWarehouse(keyword, nModels, dataDir, googleDir, 
							m_bRedownloadGoogleData, m_Pipeline.GetDB(),
							loadToMem, keepInMem);
	m_State = CREATEDB_PROCESSING;
}

VKString AppCreateDB::DownloadGoogleWarehouse(const VKString & keyword, 
											  int nModels, 
											  const VKString & dataDir, 
											  const VKString & googleDir,
											  bool redownloadGoogle,
											  ModelDatabase * db, 
											  bool loadToMem, bool keepInMem)

{
    std::cout<<"Google Warehouse looking for ";
    std::cout<<nModels<<" "<<keyword.c_str()<<"s"<<std::endl;
    
    VKString keywordDir = keyword;
    keywordDir.replace(" ", "_");
	
	VKString keywordCompatible = keyword;
	keywordCompatible.replace(" ", "%20");
    
    VKString tempDirCrawled = keywordDir + "_Crawled";
    VKString tempDirConverted = keywordDir + "_Converted";
    
    std::cout<<"===> Preparing google data dirs: "<<std::endl;
    std::cout<<"\t\t"<<tempDirCrawled.c_str()<<std::endl;
    std::cout<<"\t\t"<<tempDirConverted.c_str()<<std::endl;
    
    VKString outDir = dataDir + "/" + googleDir + "/" + tempDirCrawled + "/";
    VKString convDir = dataDir + "/" + googleDir + "/" + tempDirConverted;
    if (redownloadGoogle)
    {
        VKString cmdDelTemp = VKString("if [ -d ") + outDir + " ]; then rm -r ";
        cmdDelTemp = cmdDelTemp + outDir + "; fi";
        int retVal = system(cmdDelTemp.c_str());
        
        cmdDelTemp = VKString("if [ -d ") + convDir + " ]; then rm -r ";
        cmdDelTemp = cmdDelTemp + convDir + "; fi";
        retVal = system(cmdDelTemp.c_str());
		
        cmdDelTemp = VKString("if [ -d ") + convDir + "_TempDir ]; then rm -r ";
        cmdDelTemp = cmdDelTemp + convDir + "_TempDir; fi";
        retVal = system(cmdDelTemp.c_str());
    }

    std::cout<<"===> Running web crawler"<<std::endl;
    if (!ModelDatabase::DirExists(outDir))
    {
        VKString crawlerScript = dataDir + "/" + googleDir + "/crawler.v2.py";
        VKString cmdCrawlWeb = VKString("python ") + crawlerScript + " \"";
		cmdCrawlWeb = cmdCrawlWeb + keywordCompatible + "\"";
        cmdCrawlWeb = cmdCrawlWeb + " " + outDir + " " + VKString::number(nModels);
        std::cout<<"Crawl: "<<cmdCrawlWeb.c_str()<<std::endl;
        system(cmdCrawlWeb.c_str());
    }
    assert(ModelDatabase::DirExists(outDir));
    
    if (!ModelDatabase::DirExists(convDir))
    {
        std::cout<<"===> Unpacking, converting data"<<std::endl;
        VKString convertScript = dataDir + "/" + googleDir + "/convertAllFilesInDir.sh";
        VKString cmdConvert = convertScript + " " + outDir + " .dae " + convDir + " .dae";
        std::cout<<"Convert: "<<cmdConvert.c_str()<<std::endl;
        system(cmdConvert.c_str());
    }
    assert(ModelDatabase::DirExists(convDir));
    
	// TODO: auto cleanup data (e.g. remove ground plane, walls), prune dataset, 
	VKStringList dataDirs;          
	dataDirs.push_back(googleDir + "/" + tempDirConverted + "/");
	VKStringList dataFiles;        
	dataFiles.push_back("database.txt");
	
	if (db!=NULL)
	{
		std::cout<<"===> Reading models"<<std::endl;
		std::cout<<"\tLoad: "<<loadToMem<<" "<<keepInMem<<std::endl;
		db->InitializeDB(dataDir, dataDirs, dataFiles, keywordDir, 
						 true, loadToMem, keepInMem);
	}
	std::cout<<"===> DB is created with "<<db->NumModels()<<std::endl;	
    
	return dataDirs[0];
}

void AppCreateDB::FillPSBKeywordToFilesMap()
{
	if (m_PSBKeywordToFiles.size() > 0)
		return;

	ParamParser & par = m_Pipeline.m_Params;
    VKString dataDir = par.GetStrValue(m_sModuleName, "INPUT_DataDir");
    VKString psbDir = par.GetStrValue(m_sModuleName, "INPUT_PSBSubdir");
	
	VKString claPrefix = dataDir + "/" + psbDir + "/classification/v1/base/";
	VKStringList claFiles;
	claFiles.push_back(claPrefix + "test.cla");
	//claFiles.push_back(claPrefix + "train.cla");
	m_ClassRoot.m_NodeStr = "0";
	
	assert(claFiles.count()==1);	// cannot suport two - since always assumes linear heritage
	for (int fn = 0; fn < claFiles.count(); fn++)
	{
		std::ifstream textStream(claFiles[fn].c_str());
		assert(textStream.is_open());
		VKString tempStr;
		tempStr.readToken(textStream);		assert(tempStr=="PSB");
		
		int tempInt;
		textStream>>tempInt;	// don't know
		textStream>>tempInt;	// don't know
		textStream>>tempInt;	// don't know
		
		VKStringList keywords;
		int nModelsToFollow = 0;
		keywords.push_back("0");
		
		std::vector<PSBClassTree*> classNodes;
		classNodes.push_back(&m_ClassRoot);
		
		while (!textStream.eof())
		{
			tempStr.readLine(textStream);
			if (tempStr=="")
				continue;	// empty line
			
			VKStringList lineTokens = tempStr.split(" ");
			assert(keywords.count()==(int)classNodes.size());
			if (lineTokens.count()==3)	// keywords
			{
				assert(nModelsToFollow==0);
				
				VKString leafStr = lineTokens[0];
				VKString rootStr = lineTokens[1];
				nModelsToFollow = lineTokens[2].toInt();
				
				int rootFound=-1;
				for (int i=0; i<keywords.count() && rootFound==-1; i++)
				{
					if ((keywords[i] == rootStr))
						rootFound = i;
				}
				if (rootFound==-1)
				{
					std::cout<<"[ERROR] Parsing PSB (note I modified *cla files)"<<std::endl;
					std::cout<<leafStr.c_str()<<" < ";
					std::cout<<rootStr.c_str()<<" n="<<nModelsToFollow<<std::endl;
					std::cout<<"\tRemoving"<<rootFound<<std::endl;
					std::cout<<"Keyword list: "<<std::endl;
					for (int i=0; i<keywords.count(); i++)
						std::cout<<"\t"<<keywords[i].c_str()<<std::endl;
					std::cout<<"\t\tNModels="<<nModelsToFollow<<std::endl;
					std::cout<<"=========\n"<<std::endl;
					assert(rootFound>=0);
				}
				while (keywords.count() > rootFound+1)
					keywords.remove(rootFound+1);
				classNodes.erase(classNodes.begin()+(rootFound+1), classNodes.end());
				assert(keywords.count()==(int)classNodes.size());
				assert(keywords.count()>0);
				PSBClassTree * newNode = new PSBClassTree();
				classNodes.push_back(newNode);
				classNodes[classNodes.size()-2]->m_pChildren.push_back(newNode);
				newNode->m_NodeStr = leafStr;
				keywords.push_back(leafStr);
			}
			else if (lineTokens.count()==1)	// model
			{
				if (keywords.count()==0 || nModelsToFollow<=0)
				{
					std::cout<<"[ERROR] PSB: failed to parse "<<tempStr.c_str()<<std::endl;
					assert(keywords.count() > 0);
					assert(nModelsToFollow > 0);
				}
				nModelsToFollow--;
				
				//std::cout<<"m"<<lineTokens[0].c_str()<<".psb -> ";
				for (int i=0; i<keywords.count(); i++)
				{
					//std::cout<<keywords[i].c_str()<<" ";
					m_PSBKeywordToFiles[keywords[i]].push_back(lineTokens[0].toInt());
					classNodes[i]->m_Models.push_back(lineTokens[0].toInt());
				}
				//std::cout<<"\n"<<std::endl;

				//if (nModelsToFollow==0)
				//	keywords.remove(keywords.count()-1);
				
			}
			else
			{
				std::cout<<"[ERROR] PSB: unknown entry: "<<tempStr.c_str()<<std::endl;
				assert(false);
			}
		}
		
	}
}

void AppCreateDB::PrintPSBNode(PSBClassTree & node, int currDepth, int maxDepth)
{
	if (currDepth>maxDepth && maxDepth != -1)
		return;
	assert(m_PSBKeywordToFiles.find(node.m_NodeStr)!=m_PSBKeywordToFiles.end());
	for (int i=0; i<currDepth; i++)
		std::cout<<"\t";
	
	int nModels = (int)m_PSBKeywordToFiles[node.m_NodeStr].size();
	std::cout<<node.m_NodeStr.c_str()<<" : "<<nModels<<std::endl;
	
	for (int i=0; i<(int)node.m_pChildren.size(); i++)
		PrintPSBNode(*(node.m_pChildren[i]), currDepth+1, maxDepth);
}

void AppCreateDB::PrintPSBKeywords(int maxDepth)
{
	FillPSBKeywordToFilesMap();
	std::cout<<"PSB Keywords: "<<std::endl;

	PrintPSBNode(m_ClassRoot, 0, maxDepth);
}

void AppCreateDB::GetPrincetonShapeBenchmark(const VKString & keyword, int nModels)
{
	std::cout<<"Princeton Shape Benchmark looking for ";
    std::cout<<nModels<<" "<<keyword.c_str()<<"s"<<std::endl;

	VKString keywordDir = keyword;
	FillPSBKeywordToFilesMap();
	
	ParamParser & par = m_Pipeline.m_Params;
    VKString dataDir = par.GetStrValue(m_sModuleName, "INPUT_DataDir");
    VKString psbDir = par.GetStrValue(m_sModuleName, "INPUT_PSBSubdir");

	VKString segsPrefix = psbDir + "/SegmentationDB";
	VKString tempDBName = VKString("DB_") +keyword+ ".txt";
	
	
	VKStringList dataDirs;          dataDirs.push_back(psbDir+"/SegmentationDB/");
    VKStringList dataFiles;         dataFiles.push_back(tempDBName);
	
	std::ofstream textStream((dataDir+"/"+dataDirs[0]+"/"+tempDBName).c_str());
	assert(textStream.is_open());

	std::vector<int> & models = m_PSBKeywordToFiles[keyword];
	std::sort(models.begin(), models.end());
	int nAdded = 0;
	for (int i=0; i<(int)models.size(); i++)
	{
		if (nAdded >= nModels && nModels!=0)
			break;
		if (i>=1 && models[i-1]==models[i])
		{
			std::cout<<"[WARNING] Model "<<models[i];
			std::cout<<" repeated twice in "<<keyword.c_str()<<std::endl;
			continue;
		}
		if (ModelDatabase::FileExists((dataDir+"/"+dataDirs[0]+"/m"+models[i]+".psb")))
		{
			nAdded++;
			textStream<<VKString::number(i).c_str()<<":\n";
			textStream<<"\tEntryType PSB\n";
			textStream<<"\tFilepath [m"<<models[i]<<".psb]\n\n";
		}
	}
	textStream.close();
	m_Pipeline.GetDB()->InitializeDB(dataDir, dataDirs, dataFiles, keywordDir, true);
	
	m_State = CREATEDB_PROCESSING;
}

void AppCreateDB::FillCorrsKeywordToFilesMap()
{
	if (m_PathToModel.size() > 0)
		return;
	
	ParamParser & par = m_Pipeline.m_Params;
    VKString dataDir = par.GetStrValue(m_sModuleName, "INPUT_DataDir");
    VKString corrsDir = par.GetStrValue(m_sModuleName, "INPUT_CorrsSubdir");

	VKString scapePrefix = dataDir + "/"+ corrsDir+"/Data/SCAPE/";
	VKString toscaPrefix = dataDir + "/"+ corrsDir+"/Data/TOSCA/";
	VKString shrecPrefix = dataDir + "/"+ corrsDir+"/Data/watertight_shrec07/";
	
	// add Scape
	m_sCorrsHelpStr = "Correspondence Benchmark Data:\n";
	m_sCorrsHelpStr += "SCAPE (human models only):\n";
	for (int i=0; i<=71; i++)
	{
		// check if db file exists - if not create
		VKString nStr = VKString("0")+VKString::number(i);
		if (i < 10)
			nStr = VKString("0") + nStr;
		VKString dbFileOnly = VKString("DBFiles/mesh") + nStr + ".psb";
		VKString dbFile = scapePrefix + dbFileOnly;
		
		m_Pipeline.GetDB()->CreateDirsOnPathToFile(dbFile);
		
		bool fileExists = false;
		if (!ModelDatabase::FileExists(dbFile))
		{
			VKString offFileOnly = VKString("Meshes/mesh")+nStr+".off";
			VKString offFile = scapePrefix + offFileOnly;
			if (ModelDatabase::FileExists(offFile))
			{
				std::ofstream textStream(dbFile.c_str());
				assert(textStream.is_open());
				textStream<<"ModelPSB:\n";
				textStream<<"\tGeometryType\tUNIFORM_MANIFOLD\n";
				textStream<<"\tGeometry\t[../Meshes/mesh"<<nStr.c_str()<<".off]\n";
				textStream<<"\tCorrPntSets\t[../Corrs/1.pid AllVertices]\n";
				textStream<<"\tCorrGroups\t[human scape]\n";
				textStream<<"\tSymm\t[../Corrs/1.sym.info none]\n";
				textStream<<"\tCamera\t[../Meshes/camera.txt]\n";
				textStream<<"\tCameraLine\t[0]\n";
				textStream<<"\tQueryTriplet\t[SCAPE "<<1<<" "<<i<<"]\n";
				textStream.close();
				fileExists = true;
			}
		}
		else
			fileExists = true;
		
		if (fileExists)
		{
			int currID = (int)m_PathToModel.size();
			m_CorrKeywordToFiles["scape"].push_back(currID);
			m_CorrKeywordToFiles["human"].push_back(currID);
			m_PathToModel.push_back(VKString("SCAPE/")+dbFileOnly);
		}
	}
	int nScape = (int)m_CorrKeywordToFiles["scape"].size();
	m_sCorrsHelpStr += VKString("\tscape : ")+VKString::number(nScape)+"\n";
	
	// add TOSCA
	VKStringList toscaClasses;
	toscaClasses.push_back("cat");
	toscaClasses.push_back("centaur");
	toscaClasses.push_back("dog");
	toscaClasses.push_back("horse");
	toscaClasses.push_back("wolf");
	int firstHuman = toscaClasses.count();
	toscaClasses.push_back("gorilla");
	toscaClasses.push_back("david");
	toscaClasses.push_back("michael");
	toscaClasses.push_back("victoria");
	
	for (int i=0; i<toscaClasses.count(); i++)
	{
		VKString tc = toscaClasses[i];
		VKString humanOrAnimal = "animal";
		if (i >= firstHuman)
			humanOrAnimal = "human";
		if (tc=="centaur")
			humanOrAnimal="other";
		
		for (int j=0; j<30; j++)
		{
			VKString nStr = VKString::number(j);
			VKString dbFileOnly = VKString("DBFiles/")+ tc + nStr + ".psb";
			VKString dbFile = toscaPrefix + dbFileOnly;
			
			m_Pipeline.GetDB()->CreateDirsOnPathToFile(dbFile);
			
			bool fileExists = false;
			if (!ModelDatabase::FileExists(dbFile))
			{
				VKString offFileOnly = VKString("Meshes/")+tc+nStr+".off";
				VKString offFile = toscaPrefix + offFileOnly;
				//std::cout<<"Could not find off file: "<<offFile.c_str()<<std::endl;
				if (ModelDatabase::FileExists(offFile))
				{
					std::ofstream textStream(dbFile.c_str());
					assert(textStream.is_open());
					textStream<<"ModelPSB:\n";
					textStream<<"\tGeometryType\tUNIFORM_MANIFOLD\n";
					textStream<<"\tGeometry\t[../Meshes/"<<(tc+nStr).c_str()<<".off]\n";
					textStream<<"\tCorrPntSets\t[../Corrs/"<<tc.c_str()<<".pid AllVertices]\n";
					textStream<<"\tCorrGroups\t["<<humanOrAnimal.c_str();
					textStream<<" "<<tc.c_str()<<"_tosca]\n";
					textStream<<"\tSymm\t[../Corrs/"<<tc.c_str()<<".sym.info none]\n";
					textStream<<"\tCamera\t[../Meshes/camera_"<<tc.c_str()<<".txt]\n";
					textStream<<"\tCameraLine\t[0]\n";
					textStream<<"\tQueryTriplet\t[TOSCA "<<tc.c_str()<<" "<<j<<"]\n";
					
					textStream.close();
					fileExists = true;
				}
			}
			else
				fileExists = true;
			
			if (fileExists)
			{
				int currID = (int)m_PathToModel.size();
				m_CorrKeywordToFiles[VKString("tosca ")+humanOrAnimal].push_back(currID);
				m_CorrKeywordToFiles[VKString("tosca ")+tc].push_back(currID);
				m_CorrKeywordToFiles[humanOrAnimal].push_back(currID);
				m_PathToModel.push_back(VKString("TOSCA/")+dbFileOnly);
			}
		}
	}
	
	m_sCorrsHelpStr += "TOSCA:\n";
	int nToscaHumans = (int)m_CorrKeywordToFiles["tosca human"].size();
	int nToscaAnimals = (int)m_CorrKeywordToFiles["tosca animal"].size();
	m_sCorrsHelpStr += VKString("\ttosca human : ")+VKString::number(nToscaHumans)+"\n";;
	m_sCorrsHelpStr += VKString("\ttosca animal : ")+VKString::number(nToscaAnimals)+"\n";;
	for (int i=0; i<toscaClasses.count(); i++)
	{
		VKString keyword = VKString("tosca ")+toscaClasses[i];
		int nInToscaClass = (int)m_CorrKeywordToFiles[keyword].size();
		m_sCorrsHelpStr += VKString("\t") + keyword + "  : "+VKString::number(nInToscaClass)+"\n";
	}
	

	// add SHREC
	VKStringList shrecClasses;
	shrecClasses.push_back("human");
	shrecClasses.push_back("cup");
	shrecClasses.push_back("glasses");
	shrecClasses.push_back("airplane");
	shrecClasses.push_back("ant");
	shrecClasses.push_back("chair");
	shrecClasses.push_back("octopus");
	shrecClasses.push_back("table");
	shrecClasses.push_back("teddy");
	shrecClasses.push_back("hand");
	shrecClasses.push_back("plier");
	shrecClasses.push_back("fish");
	shrecClasses.push_back("bird");
	shrecClasses.push_back("spring");
	shrecClasses.push_back("armadillo");
	shrecClasses.push_back("bust");
	shrecClasses.push_back("mech");
	shrecClasses.push_back("bearing");
	shrecClasses.push_back("vase");
	shrecClasses.push_back("animal");

	m_sCorrsHelpStr += "SHREC:\n";
	for (int i=0; i<shrecClasses.count(); i++)
	{
		VKString sc = shrecClasses[i];
		for (int j=0; j<20; j++)
		{
			int fileID = i * 20 + j + 1;
			VKString fileIDStr = VKString::number(fileID);
			VKString dbFileOnly = VKString("DBFiles/")+ fileIDStr + ".psb";
			VKString dbFile = shrecPrefix + dbFileOnly;
			m_Pipeline.GetDB()->CreateDirsOnPathToFile(dbFile);

			if (!ModelDatabase::FileExists(dbFile))
			{
				VKString offFileOnly = VKString("Meshes/")+fileIDStr+".off";
				VKString offFile = shrecPrefix + offFileOnly;
				//std::cout<<"File Does not exist: "<<offFile.c_str()<<std::endl;
				assert (ModelDatabase::FileExists(offFile));
				std::ofstream textStream(dbFile.c_str());
				assert(textStream.is_open());
				textStream<<"ModelPSB:\n";
				textStream<<"\tGeometryType\tUNIFORM_MANIFOLD\n";
				textStream<<"\tGeometry\t[../Meshes/"<<(fileIDStr).c_str()<<".off]\n";
				textStream<<"\tCorrPntSets\t[../Corrs/"<<fileID<<".vts]\n";
				textStream<<"\tCorrGroups\t["<<sc.c_str()<<"]\n";
				textStream<<"\tSymm\t[../Corrs/"<<i<<".sym.info]\n";
				textStream<<"\tCamera\t[../Meshes/camera.txt]\n";
				textStream<<"\tCameraLine\t["<<(fileID-1)<<"]\n";
				textStream<<"\tQueryTriplet\t[SHREC "<<(i+1)<<" "<<(j+1)<<"]\n";				
				
				textStream.close();
			}

			int currID = (int)m_PathToModel.size();
			m_CorrKeywordToFiles[VKString("shrec ")+sc].push_back(currID);
			m_CorrKeywordToFiles[sc].push_back(currID);
			m_PathToModel.push_back(VKString("watertight_shrec07/")+dbFileOnly);
		}
		VKString keyword = VKString("shrec ")+sc;
		int nShrecClass = (int)m_CorrKeywordToFiles[keyword].size();
		m_sCorrsHelpStr += VKString("\t")+keyword+" : "+VKString::number(nShrecClass)+"\n";
	}
	
	m_sCorrsHelpStr += "Combinations:\n";
	int nHumans = (int)m_CorrKeywordToFiles["human"].size();
	int nAnimals = (int)m_CorrKeywordToFiles["animal"].size();
	m_sCorrsHelpStr += VKString("\thuman : ")+VKString::number(nHumans)+"\n";
	m_sCorrsHelpStr += VKString("\tanimal : ")+VKString::number(nAnimals)+"\n";
}

void AppCreateDB::GetCorrespondenceBenchmark(const VKString & keyword, int nModels)
{
	FillCorrsKeywordToFilesMap();
    m_State = CREATEDB_PROCESSING;

	ParamParser & par = m_Pipeline.m_Params;
    VKString dataDir = par.GetStrValue(m_sModuleName, "INPUT_DataDir");
    VKString corrsDir = par.GetStrValue(m_sModuleName, "INPUT_CorrsSubdir");
	VKString keywordDir = keyword;
	keywordDir.replace(" ", "_");
	
	VKString dbPrefix = dataDir + "/"+ corrsDir+"/Data/";
	VKString tempDBName = VKString("DB_") +keywordDir+ ".txt";
	
	VKStringList dataDirs;          dataDirs.push_back(corrsDir+"/Data/");
    VKStringList dataFiles;         dataFiles.push_back(tempDBName);

	if (m_CorrKeywordToFiles.find(keyword)==m_CorrKeywordToFiles.end())
	{
		std::cout<<"[WARNING] Could not find keyword: "<<keyword.c_str()<<std::endl;
		std::cout<<"Valid Keywords:"<<std::endl;
		std::cout<<m_sCorrsHelpStr.c_str()<<std::endl;
		return;
	}
	
	std::vector<int> & models = m_CorrKeywordToFiles[keyword];
	
	int nAdded = 0;
	
	std::ofstream textStream((dbPrefix + tempDBName).c_str());
	for (int i=0; i<(int)models.size(); i++)
	{
		if (nAdded>=nModels && nModels>0)
			break;
		int modelID = models[i];
		assert(modelID < (int)m_PathToModel.size());
		VKString filePath = m_PathToModel[modelID];
		VKString fullPath = dbPrefix + filePath;
		if (ModelDatabase::FileExists(fullPath))
		{
			nAdded++;
			VKString nStr = VKString::number(i);
			while (nStr.length() < 4)
				nStr = VKString("0") + nStr;
			textStream<<nStr.c_str()<<":\n";
			textStream<<"\tEntryType PSB\n";
			textStream<<"\tFilepath ["<<filePath.c_str()<<"]\n\n";
		}
		else
		{
			std::cout<<"[WARNING] File does not exist: "<<fullPath.c_str()<<std::endl;
		}
	}
	textStream.close();
	assert(ModelDatabase::FileExists(dbPrefix + tempDBName));
	
	m_Pipeline.GetDB()->InitializeDB(dataDir, dataDirs, dataFiles, keywordDir, true);

}

void AppCreateDB::DeleteModelInSubwindow(DatabaseVisWindow * window, int subwindowID)
{
    std::vector<int> & models = m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID;
    int nSubwindows = (int)models.size();
    if (subwindowID<0 || subwindowID>=nSubwindows)
    {
        std::cout<<"[WARNING] Could not delete model in subwindow: ";
        std::cout<<subwindowID<<" Max="<<nSubwindows<<std::endl;
        return;
    }
    
    int modelID = models[subwindowID];
    DeleteModel(modelID);
    
    for (int i=0; i<(int)window->m_NewModelOrder.size(); i++)
    {
        int idInOrder = window->m_NewModelOrder[i];
        
        if (idInOrder > modelID)
            window->m_NewModelOrder[i]--;
        else if (idInOrder == modelID)
        {
            window->m_NewModelOrder.erase(window->m_NewModelOrder.begin() + i);
            i--;
        }
    }

    if (window->m_iCurrModelTraversalID + nSubwindows >= (int)window->m_NewModelOrder.size())
        window->m_iCurrModelTraversalID--;
    
    if (window->m_NewModelOrder.size()==0)
    {
        for (int i=0; i<(int)models.size(); i++)
        {
            if (models[i] >= modelID)
                models[i]--;
            if (models[i] < 0)
                models[i] = 0;
        }
    }
    else
    {
        window->FillMostSimilarModels(window->m_iCurrModelTraversalID, window->m_NewModelOrder);
    }

}

void AppCreateDB::DeleteModel(int modelID)
{
    m_Pipeline.GetDB()->RemoveModel(modelID);
    
}

void AppCreateDB::UndoDelete()
{
    assert(false);
}

VKString AppCreateDB::InputDBName(CreateDBInputDB db)
{
    if (db==GOOGLE_WAREHOUSE)
        return "Google 3d Warehouse";
    else if (db==PRINCETON_SHAPE_BENCHMARK)
        return "Princeton Shape Benchmark";
    else if (db==CORRESPONDENCE_BENCHMARK)
        return "Correspondence Benchmark";
    return "Unknown";
}


VKString AppCreateDB::AppStateName(CreateDBState state)
{
    if (state==CREATEDB_KEYWORD)
        return "Enter Search Keyword";
    else if (state==CREATEDB_NUM_MODELS)
        return "Enter Number of Models";
    else if (state==CREATEDB_INPUTDB)
        return "Enter input DB id";
    else if (state==CREATEDB_SEARCHING)
        return "Searching for models with given query";
    else if (state==CREATEDB_PROCESSING)
        return "Processng DB";
    return "Unknown";
}

/// GROUND TRUTH LABELING
void AppCreateDB::SelectGroundTruthPoint(DatabaseVisWindow * window, int x, int y)
{
	if (m_State == CREATEDB_PROCESSING
		&& m_ProcessingState == PROCESS_GROUND_TRUTH)
    {
		int i=0;
		//for (int i=0; i<4; i++)
		{
			Model3D * model = m_Pipeline.GetDB()->GetModelInSubwindow(i);
			assert(model!=NULL);
			//ModelPoint p = model->RenderingParams()->QuickIntersection(x, y, 0, 0);
//			if (!p.Invalid())
//			{
			int vtxID = model->RenderingParams()->SelectVertex(x, y);
			if (vtxID<0)
				return;
			
			PointSet * pnts = model->GetPointSet(m_sGroundTruthPntSetName);
			if (pnts==NULL)
			{
				pnts = new PointSet(m_sGroundTruthPntSetName);
				model->AddPointSet(m_sGroundTruthPntSetName, pnts);
				pnts->SetModel(model);
			}
			pnts = model->GetPointSet(m_sGroundTruthPntSetName);
			assert(pnts!=NULL);
			pnts->AddPoint(ModelPoint(vtxID, model));
			std::cout<<"Adding Vertex: "<<vtxID<<" to "<<pnts->NumPoints()<<std::endl;
			
			m_GTLabelsHistory.push_back(model->ModelID());
			if (m_GTLabelsHistory.size()>1000)
				m_GTLabelsHistory.erase(m_GTLabelsHistory.begin());
//			}
		}
    }
}

void AppCreateDB::AddGroundTruthPointLabel(const VKString & strLabel)
{
	m_GroundTruthLabels.push_back(strLabel);
}

void AppCreateDB::PrintGroundTruthPointLabels()
{
	std::cout<<"GT Labels:"<<std::endl;
	for (int i=0; i<m_GroundTruthLabels.count(); i++)
		std::cout<<"\t"<<i<<": "<<m_GroundTruthLabels[i].c_str()<<std::endl;
}

void AppCreateDB::WriteGroundTruthPointLabels()
{
	// write point labels
	VKString appDir = m_Pipeline.GetDB()->GetAppDir(m_sAppName);
	VKString pntsFile = appDir + "/"+m_sGroundTruthPntSetName+".labels";
	m_Pipeline.GetDB()->CreateDirsOnPathToFile(pntsFile);
	std::ofstream textStream(pntsFile.c_str());
	assert(textStream.is_open());
	for (int i=0; i<m_GroundTruthLabels.count(); i++)
		textStream<<i<<" "<<m_GroundTruthLabels[i].c_str()<<"\n";
	textStream.close();
	
	CollectionGroundTruth * gtMap = NULL;
	int nExpected = m_GroundTruthLabels.count();
	// write per model files
	for (int i=0; i<m_Pipeline.GetDB()->NumModels(); i++)
	{
		Model3D * model = m_Pipeline.GetDB()->GetModelByID(i);
		VKString pntsDir = m_Pipeline.GetDB()->GetPointSetDir(model->ModelName(), 
															  m_sGroundTruthPntSetName);
		PointSet * pnts = model->GetPointSet(m_sGroundTruthPntSetName);
		if (pnts!=NULL)
		{
			if (pnts->NumPoints() != nExpected)
			{
				std::cout<<"[WARNING] Did not write: "<<model->ModelName().c_str();
				std::cout<<" nPnts="<<pnts->NumPoints()<<" Expected="<<nExpected<<std::endl;
				continue;
			}
			pnts->WritePointSet(m_Pipeline.GetDB());
			model->CameWithPointSet(m_sGroundTruthPntSetName);
			
			// add / create ground truth map				
			VKString gtTrueName = CollectionGroundTruth::DefaultGroundTruthMapName();
			model->CameWithCollection(gtTrueName);

			// create or find ground truth map
			if (gtMap==NULL)
			{
				CorrCollection * coll = m_Pipeline.GetDB()->GetCorrCollection(gtTrueName);
					
				if (coll==NULL)
				{
					gtMap = new CollectionGroundTruth(gtTrueName, m_Pipeline.GetDB());
					m_Pipeline.GetDB()->AddCorrCollection(gtTrueName, gtMap);
				}
				else
				{
					assert(coll->GetCollectionType()=="CollectionGroundTruth");
					gtMap = (CollectionGroundTruth*)coll;
				}
				assert(gtMap!=NULL);
			}

			// add point set / class name to collection
			gtMap->AddModelFeaturePntsGT(model, m_sGroundTruthPntSetName, "manual");
		}
		else
			std::cout<<"[WARNING] No points: "<<model->ModelName().c_str()<<std::endl;
	}
	
	// write global ground truth file
	gtMap->WriteCollection();
}

void AppCreateDB::UndoGroundTruthPointLabel()
{
	if (m_GTLabelsHistory.size()>0)
	{
		int lID = (int)m_GTLabelsHistory.size()-1;
		int lastModel = m_GTLabelsHistory[lID];
		m_GTLabelsHistory.erase(m_GTLabelsHistory.begin()+lID);
		Model3D * model = m_Pipeline.GetDB()->GetModelByID(lastModel);
		assert(model!=NULL);
		PointSet * pnts = model->GetPointSet(m_sGroundTruthPntSetName);
		assert(pnts->NumPoints()>0);
		pnts->RemovePoint(pnts->NumPoints()-1);
	}
	else
		std::cout<<"[WARNING] Reached end of undo history"<<std::endl;
}


void AppCreateDB::ReorderModels()
{
	std::cout<<"Re-ordering models"<<std::endl;
	std::vector<VKString> modelNames;
	
	for (int i=0; i<m_Pipeline.GetDB()->NumModels(); i++)
		modelNames.push_back(m_Pipeline.GetDB()->GetModelByID(i)->ModelName());
	
	std::sort(modelNames.begin(), modelNames.end());
	
	for (int i=0; i<m_Pipeline.GetDB()->NumModels(); i++)
		m_Pipeline.GetDB()->SwapModels(modelNames[i], i);
}

