/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __AppPartSelection_H
#define __AppPartSelection_H

#include "PipelineStepApplication.h"	

class AppPartSelection : public PipelineStepApplication
{
	public: 
		enum InteractionMode
		{
			APP_PART_SELECTION,
			APP_INTERACTIVE_LABELING,
            APP_DRAW_DECAL,
			APP_JOINT_DEFORMATION,
		};
	
        enum DecalTransferState
        {
            DECAL_TRANS_SELECT_REGION,
            DECAL_TRANS_DRAW_DECAL,
            DECAL_TRANS_DECAL_TRANSFERED
        };
    
        enum DecalTransferDrawFlags
        {
            DECAL_DRAW_NONE         = 0,
            DECAL_DRAW_DECAL        = 1,
            DECAL_DRAW_MESH         = 2,
            DECAL_DRAW_MESH_REGION  = 4,
            DECAL_DRAW_TRANSFERED   = 8
        };
    
		enum SemanticGlueState
		{
			GLUE_NONE,
			GLUE_SELECT_LABEL,
			GLUE_INTERACTIVE_ALIGNMENT,
			GLUE_PROPAGATED_TRANSFORMS,
			GLUE_HIDE_SECOND_MODEL
		};
	
		enum SemanticGlueAlignmentFlags
		{
			GLUE_SCALE_MODEL_DIAMETER		= 1,
			GLUE_SCALE_FROM_ALIGNMENT		= 2,
			GLUE_ROTATION_OPPOSE_NORMALS	= 4,
			GLUE_ROTATION_FROM_ALIGNMENT	= 8
		};
	
		enum ModelOrientation
		{
			ORIENT_TOP_AND_UP, 
			ORIENT_ALIGN_AREA
		};
	
	
		AppPartSelection(AnalysisPipeline & pipeline, const VKString & moduleName);
	
		virtual void SetInteractionMode(InteractionMode mode);
		
		virtual bool CaptureMouseMotionEvent(DatabaseVisWindow * window, 
											 int x, int y, int dx, int dy);
	
		virtual bool CaptureKeyPressed(DatabaseVisWindow * window, 
									   const char key, int mouseX, int mouseY);
	
		virtual void FillPtrsFromParams();
		virtual void FillPtrs(const VKString & collection, const VKString & corrValue,
							  const VKString & outputModelValue);
		virtual void InteractAppEnabled(DatabaseVisWindow * window);
		virtual void InteractAppDisabled(DatabaseVisWindow * window);
	
		virtual VKString GetAppType();
		virtual void Execute();
		virtual void Draw(DatabaseVisWindow * window);
		
		//virtual int GetNPoints();

		virtual void WriteResultsChild(std::ofstream & textStream);
		virtual bool ReadResultsChild(std::ifstream & textStream);
		virtual void WriteInteractionResults();
		virtual void ReadInteractionResults();
	
		virtual void OrientAllModels(ModelOrientation orient);
		
		virtual void SemanticGlueAutoAttachToAll();
        virtual void ChangeInteractionMode(InteractionMode newInteractionMode);
	
	protected:	
		virtual void DrawAlignmentEdges(DatabaseVisWindow * window);
	
		virtual void DrawMesh(DatabaseVisWindow * window, Model3D * model, 
							  int subwindowID, R3Affine * xform=NULL, bool autoHandleCameras=true);
		virtual void DrawMeshVertex(int subwindow, ModelPerVertexValues * perVertexVals,
									GLfloat surfaceMaterial1[4], GLfloat surfaceMaterial2[4], 
									 Model3D * mesh, const R3MeshVertex * vtx, R3Affine * xform);
        virtual bool DrawGluingApp(DatabaseVisWindow * window);
        virtual void DrawFineDetailPainting(DatabaseVisWindow * window, 
                                           Model3D * model, int subwindowID);
        virtual void UpdateSimilaritiesToPainted(DatabaseVisWindow * window);
	
		InteractionMode m_InteractionMode;
	
		int m_iCurrLabelID;
		VKString m_sCurrLabel;
		VKStringList m_CreatedLabels;
		bool m_bWritingNewLabel;
	
		//int m_iNPoints;
	
		int m_iShowNumParts;
		double m_fShowPartThreshold;
		int m_iSavedRenderingFlags;
		int m_iDefaultRenderingFlags;
	
		std::map<Model3D*, ModelPerVertexValues *> m_pModelIDToPaintedOn;
		ModelPerVertexValues * GetPaintedOnRendering(Model3D * model);
		void FillPaintedOnRendering(Model3D* model, std::vector<int> & selectedPntCld);
		bool PaintedOn(Model3D * model, int vertexID);
		void ClearPaint();
	
		virtual void SaveCurrRegionLabel();
		virtual void FillCumulativeSimilarities();
		virtual void FillModelValuesMostSimilarParts(std::vector<int> & modelIDs, int NBestModels);
		virtual void FillModelValuesForModel(int modelID);
	
		virtual void CreateFinalLabelsAssignMax();
		virtual void CreateFinalLabelsMRFOptimization(bool refineFaces);
		virtual void CreateFinalLabelsGlobalMRFOptimization();
	
		virtual void CreateAlternativeConservation(double * values);
		virtual void CreateModelValue(const VKString & name, double * values);
		//virtual void DeformModels();
	
		virtual void ClearFinalLabels();
		virtual void ClearAllCreatedModelValues();
		virtual void ClearCreatedModelValue(const VKString & name);	
	
		int m_iCurrIDBestPartCorrModels;
		
		int GetSelectedModel();
		void SelectCurrentModel(int modelID);
		std::vector<int> & GetRefSelectedPnts();
	
		std::map<ModelDatabase*, std::vector<int> > m_DBToSelectedPntIDs;
		std::map<ModelDatabase*, int > m_DBToSelectedModelID;
	//	std::vector<int> m_aiSelectedIDs;
	//	int m_iSelectedModel;
		std::vector<int> m_aiSimilarModels;
		std::map<int, ModelPerVertexValues *> m_mModelIDToPerVertexValues;
		std::vector<bool> m_bModelPendingUpdate;
	
		bool m_bNeedsUpdate;
		
		double * m_afCumulativeSimilarities;		// SLOW_TAG
		VKString m_sDefaultModelValue;
		
		CorrCollection * m_pInputCollection;
		CorrValue * m_pInputCorrValue;
	
		std::map<int, std::map<int, std::vector<int> > > m_mModelPointLabel;
	
		std::vector<int> m_ModelValueOrder;
		std::vector<double> m_PerModelValue;
		std::vector<double>	m_PerModelMax;
		double m_fGlobalMax;
		
		bool m_bRightMouseButton;

		double m_fSmoothnessScale;
	
		int m_iPointDescriptorID;
	
		int m_InteractiveSelectionFlags;
		
	// semantic glue
		SemanticGlueState m_SemanticGlueState;		
		VKString m_sGlueLabelName;
	
		int m_iSemanticGlueAlignmentFlags;
	
		void FillGluedCorrs(Model3D * m1, Model3D * m2, std::vector<int> * savePairs, 
							R3Affine * saveT1 = NULL, R3Affine * saveT2 = NULL, 
							PointWithID ** savePntsArray1 = NULL, int * saveNSelected1 = NULL,
							PointWithID ** savePntsArray2 = NULL, int * saveNSelected2 = NULL);
	
		void FindRigidAlignment(Model3D * m1, Model3D * m2, 
								std::vector<int> & corrPairs,
								double * weights);
        void FillCorrespondenceValues(Model3D * m1, std::vector<int> & pointsOnM1,
                                      std::map<Model3D*, std::vector<int> > * savePairs,
                                      std::map<Model3D*, std::vector<double> >* saveWeights);
	
		bool m_bDrawMeshes;
		bool m_bApplyTransformToBoth;
		
		void ScaleAffine(Model3D * m1, Model3D * m2, double dy);
		void RotateAffine(Model3D * m1, Model3D * m2, double dx, double dy);
		void TranslateAffine(Model3D * m1, Model3D * m2, double dx, double dy);

	// m_GlueDataModelPairToTransform[m2][m1] = Map: m2 -> m1
		std::map<Model3D *, std::map<Model3D *, R3Affine> > m_GlueDataModelPairToTransform;
		R3CoordSystem m_MainPairCoords;
		double m_fMainPairScale;
		
		ModelDatabase * m_pDBWithToBeGluedModel;
		Model3D * m_pModelToBeGlued;
		
		PointWithID * m_aTempArrayPnts1;
		PointWithID * m_aTempArrayPnts2;
		double * m_aTempArrayWeights;
		int m_iTempArraySize;
	
		double m_fM1ToM2DiameterScale;
    
    // fine-detail painting
        void TransformPaintedToOther();
        void FillCoarsePaintingFromFinePainting();
        void InitializeFineDetailState();
        void AdvanceFineDetailState();
        
        struct FineDetailBrushStrokes 
        {
            ModelPoint m_Pnt;
            double m_fBrushSize;
            RNRgb m_BrushColor;
        };
        double m_fCurrBrushSize;
        RNRgb m_CurrBrushColor;
        
        std::vector<FineDetailBrushStrokes> m_BrushStrokes;
        std::vector<Model3D*> m_EnabledQuickIntersectionBuffers;
    
        int m_iDEFAULT_DECALDRAW_EVERYTHING;
        int m_iDEFAULT_DECALDRAW_ONLY_DECAL;
        int m_iDEFAULT_DECALDRAW_CLEAN_DECAL;
        int m_iDecalTransferFlags;
        DecalTransferState m_DecalTransferState;
	
	// faceted search
		void AddFacet(double weight);
		void ClearFacets(bool all);
		std::vector<std::vector<double> > m_FacetsPerModelValues;
		std::vector<double *> m_FacetsPerPointValues;
		std::vector<int> m_FacetsNPoints;
	
		void GlueOtherToCurrent();
		void ShapeDescriptorSimilarityTerm(std::vector<int> & pnts, 
										   int modelFrom, 
										   std::vector<double> & similarities);
		std::vector<double> m_PerModelShapeDescriptorScaling;
		
};


#endif

