/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/


#ifndef __CollectionDiffuse_H
#define __CollectionDiffuse_H

#include "CorrCollection.h"
#include "DiffusionMap.h"

class CollectionDiffuse : public CorrCollection
{
	public:
		enum CorrMtxType
		{
			CORR_MTX_PAIRWISE_CORRS,
			CORR_MTX_POINT_DESCRIPTORS,
			CORR_MTX_EXTERNAL_SCRIPT,
			CORR_MTX_EXTERNAL_FILE
		};
		struct CorrMtxParams
		{
			CorrMtxType m_Type;
			
			VKString m_sInputCollection;
			VKString m_sLocalCorrValue;
			VKString m_sGlobalCorrValue;
			double m_fMinOverlap;
			double m_fMaxOverlap;	
			double m_fValueThreshold;
			
			VKString m_sPointDescriptor;
			VKString m_sExternalScriptName;
		};
		
		struct DiffusionParams
		{
			double m_fDiffTime;
			double m_fDistPercentile;
			DiffusionMap::DiffusedSearchType m_ANNSearchType;
			DiffusionMap::EigenDecompositionLibrary m_EigsLib;
			DiffusionMap::DistToSimNormType m_Dist2SimNorm;
		};	
	
		CollectionDiffuse(const VKString & collectionName, ModelDatabase * db);
		CollectionDiffuse(const VKString & collectionName, ModelDatabase * db, 
						  const CorrMtxParams & corrMtxParams, 
						  const DiffusionParams & diffusionParams);
		
		virtual double DiffusedCorrValue(const ModelPoint & pntFrom, 
										 const ModelPoint & pntTo) = 0;
		
		//static int NPoints(ModelDatabase * db);	
		static DiffusionMap::DiffusedSearchType StrToDiffSearch(const VKString & str);
		static DiffusionMap::EigenDecompositionLibrary StrToEigsLib(const VKString & str);
		static DiffusionMap::DistToSimNormType StrToDist2Sim(const VKString & str);
	
		virtual void CallAddEntryForAll();
		virtual void CallAddEntryForAllPairwiseCollections();
		virtual void CallAddEntryForAllPointDescriptors();
		virtual void CallAddEntryForAllExternalScript();
		virtual void CallAddEntryForAllExternalFile();
		virtual void AddEntry(int p1, int p2, double val) = 0;
	
		virtual void ReadMtxParams(std::ifstream &textStream);
		virtual void ReadDiffParams(std::ifstream &textStream);
		virtual void WriteMtxParams(std::ofstream &textStream);
		virtual void WriteDiffParams(std::ofstream &textStream);
		
	protected:
		DiffusionParams m_DiffusionParams;
		CorrMtxParams m_CorrMtxParams;
};


#endif


