/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __CollectionDiffuseInterpolator_H
#define __CollectionDiffuseInterpolator_H

#include <vector>
#include <map>
#include "CollectionDiffuse.h"
#include "CorrValueDiffuse.h"
#include "CollectionIdentityPointsets.h"

class CollectionDiffuseInterpolator : public CollectionDiffuse
{
	public:
		CollectionDiffuseInterpolator(const VKString & collectionName, ModelDatabase * db);
		CollectionDiffuseInterpolator(const VKString & collectionName, ModelDatabase * db,
									  const VKString & inputDiffuse, 
									  const VKString & inputDiffVal,
									  const VKString & inputPntSet,
									  const VKString & outputPntSet);
	
		virtual void GetCorrs(const ModelPoint & pnt,	
							  std::vector<ModelPoint> * corrPnts);			

		virtual double DiffusedCorrValue(const ModelPoint & pntFrom, const ModelPoint & pntTo);
		virtual VKString GetCollectionType();
		virtual void AddEntry(int p1, int p2, double val);
	
		virtual ModelPoint GetBestCorr(const ModelPoint & pnt, 
									   Model3D * toModel, 
									   const VKString & modelValue, 
									   double * value);

	
	protected:
		VKString m_sInputDiffCollection;
		VKString m_sInputCorrValue;
		VKString m_sInputPntSet;
		VKString m_sOutputPntSet;
	
		CorrCollection * m_pDiffuseCollection;
		CorrValue * m_pDiffuseCorrValue;
	
		virtual void FillPtrs();
		virtual CollectionIdentityPointsets * GetInterpolator(Model3D * model);
	
		std::map<Model3D*, CollectionIdentityPointsets *> m_apInterpolators;
	
		virtual bool ReadCollectionChild(std::ifstream & textStream);
		virtual void WriteCollectionChild(std::ofstream & textStream);
	
		std::map<ModelPoint, double> & GetCachedCorrs(const ModelPoint & pnt);
		int m_iCacheSize;
		
		std::map<ModelPoint, std::map<ModelPoint, double> > m_CachedCorrs;
		std::vector<ModelPoint> m_AddedPntsInOrder;
};

#endif

