/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __CollectionGroundTruth_H
#define __CollectionGroundTruth_H

#include "CorrCollection.h"
#include "CorrValue.h"
class CollectionGroundTruth : public CorrCollection
{
	public: 
		enum GTType
		{
			GTTYPE_UNKNOWN,
			GTTYPE_FEAT_PNTS,
			GTTYPE_RIGID_ALIGN
		};
	
		enum PntsIDType
		{
			PNTS_VERTEXID,
			PNTS_FACEID,
			PNTS_POINTCLDID,
			PNTS_GENERAL
		};
	
		struct GTCorrInfo
		{
			GTCorrInfo();
			
			VKStringList m_ModelInClass;
			VKStringList m_FeaturePntSetName;
			std::vector<PntsIDType> m_PntsTypes;
			std::vector<std::map<ModelPoint, int> > m_ModelPointToID;
			
			bool m_bAffineDefined;
			R3Affine m_AffineToCanonical;
		};
	
		CollectionGroundTruth(const VKString & collectionName,
							  ModelDatabase * database);
		virtual ~CollectionGroundTruth();

		virtual void RemoveModel(int modelID);
	
		virtual void GetCorrs(const ModelPoint & pnt, std::vector<ModelPoint> * corrPnts);
		virtual VKString GetCollectionType();
	
		virtual void AddModelFeaturePntsGT(Model3D* model, const VKString & featPntsSetName,
										   const VKString & className);
		virtual void AddModelRigidToCanonicalGT(Model3D* model, R3Affine & gtXForm);
	
		virtual bool ReadCollectionChild(std::ifstream & textStream);
		virtual void WriteCollectionChild(std::ofstream & textStream);
	
		virtual int GetPntID(GTCorrInfo & info, int classID, const ModelPoint & pnt);
	
		virtual void SetPreferedGTType(GTType tp);
		static VKString DefaultGroundTruthMapName();
		static VKString DefaultGroundTruthCorrValue();
	
	protected:
		GTType m_PreferedGTType;
		std::map<Model3D *, GTCorrInfo> m_ModelInfo;
		std::map<VKString, std::vector<Model3D*> > m_ModelsInClass;
		int m_iConstantCorrValue;
		int m_iRigidAlignCorrValue;
};


#endif

