/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/


#ifndef __CollectionIdentityPointsets_H
#define __CollectionIdentityPointsets_H

#include "CorrCollection.h"
#include "CollectionSinglePair.h"
#include "CollectionPairwise.h"

class CollectionIdentityPointsets : public CollectionSinglePair
{
	public:
		enum PntSetType
		{
			PNTS_CLD,
			PNTS_VTX,
			PNTS_FACE,
			PNTS_GEN_FROM,
			PNTS_GEN_TO
		};
		CollectionIdentityPointsets(const VKString & collectionName,
									ModelDatabase * database, int corrID,
									const VKString & modelName, 
									const VKString & pointSetFrom, 
									const VKString & pointSetTo, 
									int nTakeKNeighbors, bool fillValues);
	
		virtual ~CollectionIdentityPointsets();
	
		virtual void Initialize(bool fillValues);
	
		virtual void GetCorrs(const ModelPoint & pnt, std::vector<ModelPoint> * corrPnts);
		virtual void QuickCorrsWithWeights(const ModelPoint & pnt, 
										   int ** corrIDs, double ** weights,
										   int * nCorrs, PntSetType * idTypes);
	
		virtual VKString GetCollectionType();

		static VKString DefaultVerticesToPointCloudMap();
		static void CreateVerticesToPointCloudMap(ModelDatabase * db, const VKString & modelName, 
												  bool createInverse, bool addLabels = false);
		
		static VKString DefaultFacesToPointCloudMap();
		static void CreateFacesToPointCloudMap(ModelDatabase * db, const VKString & modelName,
											   bool createInverse, bool addLabels = false);
		static void AddMapToCollection(const VKString & collectionName, ModelDatabase * db,
									   int modelID, CollectionIdentityPointsets * identityMap);
		static void AddLabelsAllFaces(const VKString & collectionName, ModelDatabase * db,
									  int modelID, CollectionIdentityPointsets * identityMap);
		
		static CollectionIdentityPointsets * CreateInverse(CollectionIdentityPointsets * corrs);
	
		PntSetType m_PntsFromType;
		PntSetType m_PntsToType;
		int GetCorrID(const ModelPoint & pnt, PntSetType setType);
	

		VKString m_sModelName;
		VKString m_sPntsFrom;
		VKString m_sPntsTo;
		
		int m_iNNeighbors;		
	
		int m_NFromPnts;
		int m_NToPnts;

		PointSet * m_PntsTo;
		int ** m_aCorrs;
		double ** m_CorrWeights;
		int * m_aNumCorrs;
	
		std::map<const ModelPoint, int> m_mNonIndexableSetFrom;
		std::map<const ModelPoint, int> m_mNonIndexableSetTo;
	
		
};



#endif
