/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/


#ifndef __CollectionPairwise_H
#define __CollectionPairwise_H

#include "CorrCollection.h"

class CollectionSinglePair;

class CollectionPairwise : public CorrCollection
{
	public:	
		CollectionPairwise(const VKString & collectionName, ModelDatabase * database, 
						   bool writeToSingleFile);
		virtual ~CollectionPairwise();
	
		virtual CollectionPairwise * SafeCastToPairwise();
	
		virtual void GetCorrs(const ModelPoint & pnt, std::vector<ModelPoint> * corrPnts);
		
		virtual int GetFromModelCorrGlobalID(int modelFromID, int corrID);
		virtual int GetToModelCorrGlobalID(int modelToID, int corrID);
		virtual int GetModelCorrGlobalID(int modelFromID, int modelToID, int corrID);
		virtual CollectionSinglePair * GetModelCorr(int corrID);
		virtual int NumCorrsFromModel(int modelFrom);
		virtual int NumCorrsToModel(int modelTo);
		virtual int NumModelCorrs(int modelFrom, int modelTo);
		virtual int NumModelCorrs();
	
		virtual ModelPoint GetBestCorr(const ModelPoint & pnt, 
									   Model3D * toModel, 
									   const VKString & modelValue, 
									   double * value);
		

	
		virtual void ReplaceModelCorr(int mapID, CollectionSinglePair * modelCorr);
		virtual void AddModelCorr(CollectionSinglePair * modelCorr);
		
		virtual VKString GetCollectionType();
		virtual bool ReadCollectionChild(std::ifstream & textStream);
		virtual void WriteCollectionChild(std::ofstream & textStream);
	
		virtual bool ReadCorrValues(const VKString & corrValueName);
		virtual void WriteCorrValues(const VKString & corrValueName);
	
	protected:
		bool m_bWriteToSingleFile;
	
		std::vector<CollectionSinglePair*> m_ModelCorrs;
		std::vector<int> m_mCorrIDToModelFromID;
		std::vector<int> m_mCorrIDToModelToID;
		std::map<int, std::vector<int> > m_mModelFromIDToCorrIDs;
		std::map<int, std::vector<int> > m_mModelToIDToCorrIDs;
		std::map<int, std::map<int, std::vector<int> > > m_mModelPairToCorrIDs;
};
#endif
