/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#include <iostream>
#include <fstream>

#include "CorrValue.h"
#include "CollectionSinglePair.h"
#include "PointSet.h"
#include "Model3D.h"
#include "ModelDatabase.h"
#include "SinglePairPnt2Pnt.h"
#include "SinglePairRigidAlign.h"
#include "CorrCollection.h"

CollectionSinglePair::CollectionSinglePair(const VKString & collectionName, ModelDatabase * db, int corrID)
: CorrCollection(collectionName, db, corrID)
{
	m_pModelFrom = NULL;
	m_pModelTo = NULL;

	m_sModelFromName = "LoadFromFile";
	m_sModelToName = "LoadFromFile";
	
}

CollectionSinglePair::CollectionSinglePair(const VKString & modelFrom, const VKString & modelTo,
										   const VKString & collectionName, ModelDatabase * db, int corrID)
: CorrCollection(collectionName, db, corrID)
{
	m_pModelFrom = NULL;
	m_pModelTo = NULL;
	m_sModelFromName = modelFrom;
	m_sModelToName = modelTo;
}

CollectionSinglePair::~CollectionSinglePair()
{
}

VKString CollectionSinglePair::GetCollectionType()
{
	return "CollectionSinglePair";
}

Model3D * CollectionSinglePair::ModelFrom()
{
	if (m_pModelFrom==NULL)
	{
		m_pModelFrom = m_pDB->GetModelByName(m_sModelFromName);
		assert(m_pModelFrom!=NULL);
	}
	return m_pModelFrom;
}

Model3D * CollectionSinglePair::ModelTo()
{
	if (m_pModelTo==NULL)
		m_pModelTo = m_pDB->GetModelByName(m_sModelToName);
	return m_pModelTo;
}

//void CollectionSinglePair::DrawCorrEdges(const VKString & pickFromSet)
//{
//	assert(ModelFrom()!=NULL);
//	assert(ModelTo()!=NULL);
//	//ModelFrom()->RenderingParams()->LoadMyCamera()
//	PointSet * pntSet = ModelFrom()->GetPointSet(pickFromSet);
//	assert(pntSet!=NULL);
//	int nPnts = pntSet->NumPoints();
//	for (int i=0; i<pntSet->NumPoints(); i++)
//	{
//		ModelPoint pnt = pntSet->GetPoint(i);
//		DrawCorrEdge(pnt, nPnts);
//	}
//}
//
//void CollectionSinglePair::DrawCorrEdge(const ModelPoint & pnt, int numPnts)
//{
//	//std::cout<<"From: "<<ModelFrom()->ModelName().c_str()<<" To: "<<ModelTo()->ModelName().c_str()<<std::endl;	
//	glDisable(GL_LIGHTING);
//	glColor3d(0, 1, 0);
//	if (numPnts==1)
//		glLineWidth(2.);
//	else
//		glLineWidth(1.);
//	std::vector<ModelPoint> corrs;
//	
//	R3Point pFrom = ModelFrom()->RenderingParams()->TransformMyPntToCurrCamera(pnt.GetPosition());
//	GetCorrs(pnt, &corrs);
//	
//	for (int j=0; j<(int)corrs.size(); j++)
//	{
//		assert(!corrs[j].Invalid());
//		R3Point pTo = ModelTo()->RenderingParams()->TransformMyPntToCurrCamera(corrs[j].GetPosition());
//		glBegin(GL_LINES);
//		glVertex3d(pFrom.X(), pFrom.Y(), pFrom.Z());
//		glVertex3d(pTo.X(), pTo.Y(), pTo.Z());
//		glEnd();
//	}
//}
//
//void CollectionSinglePair::DrawByValueTransfer(const VKString & modelValue)
//{
//	assert(false);
//}

bool CollectionSinglePair::ReadModelPair(std::ifstream & textStream)
{
	VKString text;
	text.readToken(textStream);
	assert(text=="ModelPair");
	m_sModelFromName.readToken(textStream);
	m_sModelToName.readToken(textStream);
	return true;
}

void CollectionSinglePair::WriteModelPair(std::ofstream & textStream)
{
	textStream<<"ModelPair "<<m_sModelFromName.c_str()<<" "<<m_sModelToName.c_str()<<"\n";
}




