/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#include <stdio.h>
#include <time.h>

#include "PipelineStepApplication.h"
#include "DatabaseVisWindow.h"
#include "ModelDatabase.h"
#include "Model3D.h"
#include "CorrCollection.h"
#include "CollectionDiffuseFullMtx.h"
#include "CollectionDiffuseDynamicGraph.h"
#include "CorrValueDiffuse.h"

DatabaseVisWindow::DatabaseVisWindow(ParamParser & visParams, 
									 AnalysisPipeline & pipeline)
: VisWindow(visParams), m_Pipeline(pipeline)
{	
	m_WriteImg = WRITE_SCREENSHOT_NONE;
	m_iRenderDesign = RENDER_DESIGN_WND | RENDER_DESIGN_CTRL_WND;
	m_iPointDescriptorToRender = -1;
	m_bPendingCorrValueRenderUpdate = true;
	m_iCorrRenderFlags = RENDER_CORRS_NONE;
	m_sSelectedCollectionToRender = "";
	m_iCollectionCorrIDToRender=-1;
	m_sCorrValueToRender="";
	
	m_iCurrCamera = -1;
	m_UIMode = UIMode_NONE;
	m_iSelectedWindow = 0;
	m_iCurrentSelection = 0;
	
	m_iSelectedList = 0;
	m_sLists.push_back("Mdl");
	m_sLists.push_back("Pts");
	m_sLists.push_back("Dst");
	m_sLists.push_back("Val");
	m_sLists.push_back("Cor");
	m_sLists.push_back("  <->");
	m_sLists.push_back("CVal");
	
	if (pipeline.m_sAppNames.count()==0)
		m_sActiveApp = "none";
	else
	{
		m_sActiveApp = pipeline.m_sAppNames[0];
		m_Pipeline.GetPipelineApp(m_sActiveApp)->InteractAppEnabled(this);
	}	
}

void DatabaseVisWindow::KeyPressed(const char key, int mouseX, int mouseY)
{
	double minPercent = 0.05;
	
	if (m_Pipeline.GetPipelineApp(m_sActiveApp)!=NULL
		&& m_Pipeline.GetPipelineApp(m_sActiveApp)->CaptureKeyPressed(this, key, mouseX, mouseY))
		return;
	
    if (m_Pipeline.GetDB()->NumModels()==0)
        return;
    
	SetPendingUpdateToTrue();
	switch(key)
	{
		case '`':
			std::cout<<"DataVis Help:"<<std::endl;
			std::cout<<"---- RENDERING FLAGS:\t"<<std::endl;
			std::cout<<"\tf\t Faces (F - all models)"<<std::endl;
			std::cout<<"\te\t Edges (E - all models)"<<std::endl;
			std::cout<<"\tp\t Point sets (P - all models)"<<std::endl;
			std::cout<<"\t!\t Point descriptors "<<std::endl;
			std::cout<<"\tn\t Normals (N - all models)"<<std::endl;
//			std::cout<<"\tc\t Corrs (C - all models)"<<std::endl;
			std::cout<<"\tv\t Model Values (V - all models)"<<std::endl;
			//std::cout<<"\td\t Distances on a model (D - all models)"<<std::endl;
			std::cout<<"\tr\t Corrs as edges"<<std::endl;
			std::cout<<"\tt\t Corrs as spheres"<<std::endl;
			std::cout<<"\ty\t Transfer model value by a corr"<<std::endl;
			std::cout<<"\tu\t Corr value, confidence"<<std::endl;
			std::cout<<"\ta\t Current App"<<std::endl;
			std::cout<<"\t\tValue Rendering:"<<std::endl;
			std::cout<<"\tg\t Global values"<<std::endl;
			std::cout<<"\tG\t Isocontours (not implemented)"<<std::endl;
			std::cout<<"\t=\t Decrease Top Percentile"<<std::endl;
			std::cout<<"\t+\t Increase Top Percentile"<<std::endl;
			std::cout<<"\t-\t Decrease Bottom Percentile"<<std::endl;
			std::cout<<"\t_\t Increase Bottom Percentile"<<std::endl;
			std::cout<<"---- SELECT:\t"<<std::endl;
			std::cout<<"\tSPACE\t point on surface"<<std::endl;
			std::cout<<"\tq\t Change NN search in embedded diffused space"<<std::endl;
			//std::cout<<"\tw\t point by number ('w' + enter number + '#')"<<std::endl;
			std::cout<<"\tm\t model by number ('m' + enter number + '#')"<<std::endl;
			std::cout<<"\tb\t Change DB - next, previos"<<std::endl;
			std::cout<<"---- INTERACTION:\t"<<std::endl;
			std::cout<<"\tz\t t+=1 in diffusion distance (Z - t-=1)"<<std::endl;
			std::cout<<"\tx\t t+=4 in diffusion distance (X - t-=4)"<<std::endl;
			std::cout<<"\ts\t Diff. distance to similarity sigma (S -= .05)"<<std::endl;
			std::cout<<"\ts\t Diff. distance to similarity sigma (S -= .01)"<<std::endl;
			std::cout<<"\tESC\t Terminate App"<<std::endl;
			std::cout<<"---- OTHER:\t"<<std::endl;
			std::cout<<"\th\t Show the most similar shapes w.r.t shape descriptor"<<std::endl;
			std::cout<<"\t?\t Show the most similar shapes w.r.t. correspondence value"<<std::endl;
			std::cout<<"\t^\t Print all images (currently on screen)"<<std::endl;
			std::cout<<"\t&\t Print all images (in the database)"<<std::endl;
			std::cout<<"\t*\t Print surface values (currently on screen)"<<std::endl;
			std::cout<<"\tw/W\t Write/(w)read cameras"<<std::endl;
			std::cout<<"\t[/]\tDecrease/Increase sphere radius"<<std::endl;
			break;
		case 'n':
		case 'N':
			m_Pipeline.GetDB()->ToggleModelRenderFlag(Model3DRenderingParams::RENDER_PNT_NORMALS, 
													  (key < 'a') ? (-1) : (m_iSelectedWindow));
			break;
		case 'f':
		case 'F':
			m_Pipeline.GetDB()->ToggleModelRenderFlag(Model3DRenderingParams::RENDER_FACES, 
													  (key < 'a') ? (-1) : (m_iSelectedWindow));
			break;
		case 'e':
		case 'E':
			m_Pipeline.GetDB()->ToggleModelRenderFlag(Model3DRenderingParams::RENDER_EDGES, 
													  (key < 'a') ? (-1) : (m_iSelectedWindow));
			break;
		case 'p':
		case 'P':
			m_Pipeline.GetDB()->ToggleModelRenderFlag(Model3DRenderingParams::RENDER_POINT_SET,
													  (key < 'a') ? (-1) : (m_iSelectedWindow));
			break;
//		case 'c':
//		case 'C':
//			ToggleCorrFlag(RENDER_CORRS_SET);
//			break;
		case 'v':
		case 'V':
			m_Pipeline.GetDB()->ToggleModelRenderFlag(Model3DRenderingParams::RENDER_VALUE,
													  (key < 'a') ? (-1) : (m_iSelectedWindow));
			break;
//		case 'd':
//		case 'D':
//			m_Pipeline.GetDB()->ToggleModelRenderFlag(Model3DRenderingParams::RENDER_DISTANCE, 
//													  (key < 'a') ? (-1) : (m_iSelectedWindow));
//			break;
		case 'r':
			ToggleCorrFlag(RENDER_CORRS_EDGES);
			break;
		case 't':
			ToggleCorrFlag(RENDER_CORRS_PNTS);
			break;
		case 'y':
			ToggleCorrFlag(RENDER_CORRS_MVALUE);
			break;
		case 'u':
			ToggleCorrFlag(RENDER_CORRS_CVALUE);
			break;
		case 'R':
			ToggleCorrFlag(RENDER_CORRS_SPECIAL_1);
			break;
		case 'T':
			ToggleCorrFlag(RENDER_CORRS_SPECIAL_2);
			break;
		case 'Y':
			ToggleCorrFlag(RENDER_CORRS_SPECIAL_3);
			break;
		case 'U':
			ToggleCorrFlag(RENDER_CORRS_SPECIAL_4);
			break;
		case '[':
		case ']':
			for (int i=0; i<m_Pipeline.GetDB()->NumModels(); i++)
			{
				Model3DRenderingParams * r = m_Pipeline.GetDB()->GetModelByID(i)->RenderingParams();
				if (key=='[')
					r->ChangeRenderingRadius(-0.01);
				else
					r->ChangeRenderingRadius(+0.01);
			}
			break;
		case 'a':
		case 'A':
			if (m_Pipeline.GetPipelineApp(m_sActiveApp)!=NULL)
				m_Pipeline.GetPipelineApp(m_sActiveApp)->InteractAppDisabled(this);

			if (m_sActiveApp=="none")
			{
				if (m_Pipeline.m_sAppNames.count()>=1)
				{
					if (key=='a')
						m_sActiveApp = m_Pipeline.m_sAppNames[0];
					else
						m_sActiveApp = m_Pipeline.m_sAppNames[m_Pipeline.m_sAppNames.count()-1];
				}
			}
			else 
			{
				for (int i=0; i<m_Pipeline.m_sAppNames.count(); i++)
				{
					if (m_Pipeline.m_sAppNames[i]==m_sActiveApp)
					{
						int newID = (key=='a') ? (i+1) : (i-1);
						if (newID >= m_Pipeline.m_sAppNames.count() || newID<0)
							m_sActiveApp = "none";
						else
							m_sActiveApp = m_Pipeline.m_sAppNames[newID];
					}
				}
			}
			std::cout<<"SwitchedApp: "<<m_sActiveApp.c_str()<<std::endl;
			
			if (m_Pipeline.GetPipelineApp(m_sActiveApp)!=NULL)
				m_Pipeline.GetPipelineApp(m_sActiveApp)->InteractAppEnabled(this);
			break;
		case 'z':
		case 'Z':
		case 'X':
		case 'x':
		case 's':
		case 'S':
		case 'q':
		case 'Q':
		case 'd':
		case 'D':	
		{
			CorrCollection * coll = m_Pipeline.GetDB()->GetCorrCollection(m_sSelectedCollectionToRender);
			if (coll!=NULL && coll->GetCollectionType()=="CollectionDiffuseFullMtx")
			{
				CollectionDiffuseFullMtx * corrDiffused = (CollectionDiffuseFullMtx*)coll;
				
				if (key=='z')
					corrDiffused->InteractivelyChangeDepth(1);
				else if (key=='Z')
					corrDiffused->InteractivelyChangeDepth(-1);
				else if (key=='x')
					corrDiffused->InteractivelyChangeDepth(4);
				else if (key=='X')
					corrDiffused->InteractivelyChangeDepth(-4);
				else if (key=='s')
					corrDiffused->InteractivelyChangeEmbeddedSpaceDistance(.01);
				else if (key=='S')
					corrDiffused->InteractivelyChangeEmbeddedSpaceDistance(-.01);
				else if (key=='d')
					corrDiffused->InteractivelyChangeEmbeddedSpaceDistance(.05);
				else if (key=='D')
					corrDiffused->InteractivelyChangeEmbeddedSpaceDistance(-.05);
				else if (key=='q')
					corrDiffused->InteractivelyChangeSearchType();
			}
			else if (coll!=NULL && coll->GetCollectionType()=="CollectionDiffuseDynamicGraph")
			{
				CollectionDiffuseDynamicGraph * corrDiffused;
				corrDiffused = (CollectionDiffuseDynamicGraph*)coll;

				if (key=='z')
					corrDiffused->InteractiveChangeDiffusionTime(1);
				else if (key=='Z')
					corrDiffused->InteractiveChangeDiffusionTime(-1);
				else if (key=='x')
					corrDiffused->InteractiveChangeDiffusionTime(4);
				else if (key=='X')
					corrDiffused->InteractiveChangeDiffusionTime(-4);
				else if (key=='s')
					corrDiffused->InteractiveChangeDiffusionSigma(.01);
				else if (key=='S')
					corrDiffused->InteractiveChangeDiffusionSigma(-.01);
				else if (key=='d')
					corrDiffused->InteractiveChangeDiffusionSigma(.05);
				else if (key=='D')
					corrDiffused->InteractiveChangeDiffusionSigma(-.05);
				else if (key=='q')
				{
					corrDiffused->Step3FullIteration(1);
					corrDiffused->DrawLog(this);
				}
				else if (key=='Q')
				{
					int mID = m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID[0];
					if (corrDiffused->m_iEmbeddBasedOnSingleModel != mID)
						corrDiffused->m_iEmbeddBasedOnSingleModel = mID;
					else
						corrDiffused->m_iEmbeddBasedOnSingleModel = -1;
				}
			}
			else
				std::cout<<"[WARNING] Collection not found or not diffused"<<std::endl;
				break;
		}
		case 'g':
			m_bGlobalColors = !m_bGlobalColors;
			std::cout<<"Global Normalization of Values: "<<m_bGlobalColors<<std::endl;
			break;
		case 'G':
			m_bDefaultDrawIsocontours = !m_bDefaultDrawIsocontours;
			std::cout<<"Isocontours: "<<m_bDefaultDrawIsocontours<<std::endl;
			break;
		case '+':
		case '=':
			m_fMaxPercentile += (key=='+') ? minPercent: -minPercent;
			if (m_fMaxPercentile > 1.)
				m_fMaxPercentile = 1.;
			else if (m_fMaxPercentile <= m_fMinPercentile)
				m_fMaxPercentile = m_fMinPercentile + minPercent;
			std::cout<<"Percentile: ["<<m_fMinPercentile<<", "<<m_fMaxPercentile<<"]"<<std::endl;
			break;
		case '-':
		case '_':
			m_fMinPercentile += (key=='-') ? -minPercent: minPercent;
			if (m_fMinPercentile < 0)
				m_fMinPercentile = 0;
			else if (m_fMinPercentile >= m_fMaxPercentile)
				m_fMinPercentile = m_fMaxPercentile - minPercent;
			std::cout<<"Percentile: ["<<m_fMinPercentile<<", "<<m_fMaxPercentile<<"]"<<std::endl;
			break;
		case 'W':
			ReadCameras();
			break;
		case 'w':
		{
			WriteCameras();
			//m_UIMode = UIMode_SELECT_POINT;
			//m_iCurrentSelection = 0;
			Model3DRenderingParams * mRend = NULL;
			mRend = m_Pipeline.GetDB()->GetModelInSubwindow(0)->RenderingParams();
			for (int i=-1; i<=0; i++)
			{
				R3Point orig = mRend->GetCamPosition(i);
				R3Vector tow = mRend->GetCamTowards(i);
				R3Vector up = mRend->GetCamUp(i);

				if (i==-1)
					std::cout<<"CameraInit = ";
				else
					std::cout<<"CameraXFormed = ";
				std::cout<<orig.X()<<" "<<orig.Y()<<" "<<" "<<orig.Z()<<" ";
				std::cout<<tow.X()<<" "<<tow.Y()<<" "<<" "<<tow.Z()<<" ";
				std::cout<<up.X()<<" "<<up.Y()<<" "<<" "<<up.Z()<<" .4"<<std::endl;;
			}
			break;
		}
		case 'm':
			m_UIMode = UIMode_SELECT_MODEL;
			m_iCurrentSelection = 0;
			break;
		case ' ':
		{
			static ModelPoint pPrev;
			MouseButtonEvent(mouseX, mouseY, MOUSE_LEFT, MOUSE_BT_DOWN);
			ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
			int modelID = renderer->m_mWindowToModelID[m_iSelectedWindow];
			Model3D * model = m_Pipeline.GetDB()->GetModelByID(modelID);
			Model3DRenderingParams * rend = model->RenderingParams();
			rend->SelectPoint(mouseX, mouseY, 0);
			if (rend->GetDistanceToRender()!="" 
				&& !pPrev.Invalid() && !rend->GetSelectedPoint().Invalid())
			{
				VKString distName = rend->GetDistanceToRender();
				ModelDistance * dist = model->GetModelDistance(distName);
				if (dist==NULL)
					std::cout<<"[WARNING] Cannot find metric: "<<distName.c_str()<<std::endl;
				else
				{
					double dv = dist->Distance(pPrev, rend->GetSelectedPoint());
					std::cout<<distName.c_str()<<" : "<<dv<<std::endl;
				}
			}
			pPrev = rend->GetSelectedPoint();
			break;
		}
		case '\'':
		case '"':
		{
			std::vector<int> & w2m = m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID;
			int nModelsInOrd = (int)m_NewModelOrder.size();
			int nSteps = (int)w2m.size()-1;
			if (nModelsInOrd>0)
			{
				m_iCurrModelTraversalID += (key=='\'') ? nSteps : (-nSteps);
				if (m_iCurrModelTraversalID>=(int)m_NewModelOrder.size()-nSteps)
					m_iCurrModelTraversalID = (int)m_NewModelOrder.size()-nSteps;
				if (m_iCurrModelTraversalID<=0)
					m_iCurrModelTraversalID = 0;
				m_iCurrModelTraversalID = m_iCurrModelTraversalID % (int)m_NewModelOrder.size();
				FillMostSimilarModels(m_iCurrModelTraversalID, m_NewModelOrder);
			}
			break;
		}
		case '?':	// create set of the most similar shapes (w.r.t. to selected correspondence)
		{
			VKString selStr = m_sSelectedCollectionToRender;
			CorrCollection * coll = m_Pipeline.GetDB()->GetCorrCollection(selStr);
			if (coll==NULL)
			{
				std::cout<<"[WARNING] Collection not found"<<std::endl;
				break;
			}
			CorrValue * corrVal = coll->GetCorrValueByName(m_sCorrValueToRender);
			if (corrVal==NULL)
			{
				std::cout<<"[WARNING] Correspondence value not found"<<std::endl;
				break;				
			}
			
			ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
			int modelID = renderer->m_mWindowToModelID[0];
			Model3D * model = m_Pipeline.GetDB()->GetModelByID(modelID);
			assert(model!=NULL);
			
			std::vector<Sortable> modelValues;
			std::vector<ModelPoint> corrs;
			ModelPoint p = ((Model3D*)model)->RenderingParams()->GetSelectedPoint();
			for (int i=0; i<m_Pipeline.GetDB()->NumModels(); i++)
				modelValues.push_back(Sortable(0, m_Pipeline.GetDB()->GetModelByID(i), i));
			
			if (p.Invalid())
			{
				std::cout<<"[WARNING] No point selecting based on all corrs"<<std::endl;
				for (int i=0; i<model->PointCloud()->NumPoints(); i++)
				{
					ModelPoint pn = model->PointCloud()->GetPoint(i);
					coll->GetCorrs(pn, &corrs);
					std::vector<double> perMdlVals;
					for (int mID=0; mID<m_Pipeline.GetDB()->NumModels(); mID++)
						perMdlVals.push_back(0);
					
					for (int c=0; c<(int)corrs.size(); c++)
					{
						double value = corrVal->Value(pn, corrs[c]);
						Model3D * corrModel = (Model3D*)corrs[c].GetModel();
						if (corrModel == model)
							continue;
						int mID = corrModel->ModelID();
						
						//perMdlVals[mID] = perMdlVals[mID]+value;
						perMdlVals[mID] = vkMax(perMdlVals[mID], value);
					}
					for (int mID=0; mID<m_Pipeline.GetDB()->NumModels(); mID++)
						modelValues[mID].value += (double)perMdlVals[mID];
				}
			}
			else
			{
				coll->GetCorrs(p, &corrs);
				
				for (int i=0; i<(int)corrs.size(); i++)
				{
					double value = corrVal->Value(p, corrs[i]);
					Model3D * corrModel = (Model3D*)corrs[i].GetModel();
					if (corrModel == model)
						continue;
					double & currVal = modelValues[corrModel->ModelID()].value;
					currVal = vkMax(currVal, value);
				}
			}			
			std::sort(modelValues.begin(), modelValues.end());
			
			m_iCurrModelTraversalID = 0;
			m_NewModelOrder.clear();

			for (int i=(int)modelValues.size()-1; i>=0; i--)
			{
				m_NewModelOrder.push_back(modelValues[i].id);
				Model3D * model = m_Pipeline.GetDB()->GetModelByID(modelValues[i].id);
				std::cout<<model->ModelName().c_str()<<" : "<<modelValues[i].value<<std::endl;
			}
			
			FillMostSimilarModels(m_iCurrModelTraversalID, m_NewModelOrder);
			break;
		}
		case 'H':
		{
			m_iCurrModelTraversalID = 0;
			m_NewModelOrder.clear();
			for (int i=0; i < m_Pipeline.GetDB()->NumModels(); i++)
				m_NewModelOrder.push_back(i);
			//std::cout<<"FillMostSimilar"<<std::endl;
			FillMostSimilarModels(m_iCurrModelTraversalID, m_NewModelOrder);
			break;
		}
		case 'h':	// create set of the most similar shapes (w.r.t. shape descriptor)
		{						
			int nMostSimilarModels = m_Pipeline.GetDB()->NumModels();
			int nModelsToPrint = 7;
			
			ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
			int modelID = renderer->m_mWindowToModelID[0];
			Model3D * model = m_Pipeline.GetDB()->GetModelByID(modelID);
			assert(model!=NULL);
			VKStringList & descrNames = m_Pipeline.GetDB()->m_ShapeDescriptorNames;
			std::cout<<"Models most similar to "<<model->ModelName().c_str()<<std::endl;
				 
			for (int i=0; i<descrNames.count(); i++)
			{
				std::vector<ShapeDescriptor *> compareTo;
				for (int om=0; om<m_Pipeline.GetDB()->NumModels(); om++)
				{
					if (om!=modelID)
					{
						Model3D * otherModel = m_Pipeline.GetDB()->GetModelByID(om);
						ShapeDescriptor * descr = otherModel->GetShapeDescriptor(descrNames[i]);
						if (descr!=NULL)
							compareTo.push_back(descr);
					}
				}
				
				std::vector<ShapeDescriptor*> neighbors;
				ShapeDescriptor * currDescr = model->GetShapeDescriptor(descrNames[i]);
				if (currDescr!=NULL)
				{
					std::cout<<"\t"<<descrNames[i].c_str()<<": ";
					currDescr->FindMyNearestNeighbors(compareTo, nMostSimilarModels, 
													  neighbors, NULL);
										
					int nPrint = vkMin(nModelsToPrint, (int)neighbors.size());
					for (int nhdID = 0; nhdID < nPrint; nhdID++)
						std::cout<<neighbors[nhdID]->GetModel()->ModelName().c_str()<<" ";
					
					std::cout<<std::endl;
					
					if (i==0)	// for the first descriptor - select most similar models in subwindows
					{
						m_iCurrModelTraversalID = 0;
						m_NewModelOrder.clear();
						for (int nID=0; nID<(int)neighbors.size(); nID++)
							m_NewModelOrder.push_back(neighbors[nID]->GetModel()->ModelID());
						//std::cout<<"FillMostSimilar"<<std::endl;
						FillMostSimilarModels(m_iCurrModelTraversalID, m_NewModelOrder);
					}
				}
			}
			break;
		}
		case 'b':
		case 'B':
			m_Pipeline.ChangeDB(key=='b');
			break;
		case '#':
		{			
			ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
			int nWnd = (int) renderer->m_mWindowToModelID.size();
			if (m_iSelectedWindow<0 || m_iSelectedWindow >= nWnd)
			{
				std::cout<<"[WARNING] Selected invalid window id:"<<m_iSelectedWindow<<std::endl;
				m_iSelectedWindow = 0;
			}
			else if (m_iCurrentSelection >= m_Pipeline.GetDB()->NumModels())
			{
				std::cout<<"[WARNING] Selected invalid model id:"<<m_iCurrentSelection<<std::endl;
				m_iCurrentSelection = 0;
			}
			else if (m_UIMode==UIMode_SELECT_MODEL)
			{
				renderer->m_mWindowToModelID[m_iSelectedWindow] = m_iCurrentSelection;
			}
			else if (m_UIMode==UIMode_SELECT_POINT)
			{
				int modelID = renderer->m_mWindowToModelID[m_iSelectedWindow];
				m_Pipeline.GetDB()->GetModelByID(modelID)->RenderingParams()->SelectPoint(m_iCurrentSelection);
			}
			
			m_UIMode = UIMode_NONE;
			break;
		}
		case '!':
		{	
			m_iPointDescriptorToRender++;
			if (m_iPointDescriptorToRender >= m_Pipeline.GetDB()->m_PointDescriptorNames.count())
				m_iPointDescriptorToRender = -1;
			std::cout<<"Pnt Descriptor: ";
			if (m_iPointDescriptorToRender>=0)
				std::cout<<m_Pipeline.GetDB()->m_PointDescriptorNames[m_iPointDescriptorToRender].c_str();
			else
				std::cout<<"none";
			std::cout<<std::endl;
			break;
		}			
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
		case '0':
		{
			int digit = (int)(key - '0');
			if (m_UIMode==UIMode_SELECT_WINDOW)
				m_iSelectedWindow = m_iSelectedWindow * 10 + digit;
			else if (m_UIMode!=UIMode_NONE)
				m_iCurrentSelection = m_iCurrentSelection * 10 + digit;			
			break;
		}
		case '^':
			WriteImages("Screenshots", WRITE_SCREENSHOT, -1, false, false);
			break;
		case '&':
			WriteImages("Screenshots", WRITE_SCREENSHOT_EACH_AS_MAIN);
			break;
		case '*':
			WriteSurfaceValues("Screenshots");
			break;
		case 27:
			m_pRenderer->TerminateWindow();
			exit(0);
			break;
		default:
			std::cout<<"[WARNING] No action defined for '"<<key<<"'"<<std::endl;
			
	}
	
	// user-controlled events
	m_pRenderer->UpdateScreen();
}

void DatabaseVisWindow::FillMostSimilarModels(int firstID, std::vector<int> & ids)
{						
	std::vector<int> & w2m = m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID;

	for (int wID = 1; wID < (int)w2m.size(); wID++)
	{
		if (firstID + wID - 1 >= (int)ids.size())
			break;
		w2m[wID] = ids[firstID + wID-1];
	}
}


void DatabaseVisWindow::MouseButtonEvent(int x, int y, MouseButton mouseButtonID, 
										 MouseButtonState newState)
{
	UpdateMouseButtons(mouseButtonID, newState);
	
	if (m_Pipeline.GetPipelineApp(m_sActiveApp)!=NULL
		&& m_Pipeline.GetPipelineApp(m_sActiveApp)->CaptureMouseButtonEvent(this, x, y, 
																			mouseButtonID, newState))
		return;	
	if (mouseButtonID==MOUSE_RIGHT && newState==MOUSE_BT_DOWN)
	{		
		bool inWnd;
		double xWnd, yWnd;
		VKString subwindowName = InSubWindow(x, y, &inWnd, &xWnd, &yWnd);
		if (!inWnd)
		{
			int wndID = ClickInWindow(x, y);
			if (wndID>=0 && m_iSelectedWindow != wndID
				&& m_iSelectedWindow==0)
			{
				ModelDatabaseRenderingParams * renderer;
				renderer = m_Pipeline.GetDB()->GetRenderParams();
				assert(m_iSelectedWindow>=0 
					   && wndID < (int)renderer->m_mWindowToModelID.size()
					   && m_iSelectedWindow < (int)renderer->m_mWindowToModelID.size());
				
				int m1 = renderer->m_mWindowToModelID[m_iSelectedWindow];
				int m2 = renderer->m_mWindowToModelID[wndID];
				renderer->m_mWindowToModelID[wndID] = m1;
				renderer->m_mWindowToModelID[m_iSelectedWindow] = m2;
				VKString mName = m_Pipeline.GetDB()->GetModelByID(m2)->ModelName();
				std::cout<<"Switched To: "<<mName.c_str()<<std::endl;
			}
		}
	}
	else if (mouseButtonID==MOUSE_LEFT && newState==MOUSE_BT_DOWN)
	{
		bool inWnd;
		double xWnd, yWnd;
		VKString subwindowName = InSubWindow(x, y, &inWnd, &xWnd, &yWnd);
		if (!inWnd)
		{
			int wndID = ClickInWindow(x, y);
			if (wndID!=-1)
				m_iSelectedWindow = wndID;
		}
		else 
			ProcessClickInSubwindow(subwindowName, xWnd, yWnd, true);
	}
		
	m_pRenderer->UpdateScreen();
}

void DatabaseVisWindow::MouseMotionEvent(int x, int y, int dx, int dy)
{
	assert(m_iSelectedWindow!=-1);
	
	if (m_Pipeline.GetPipelineApp(m_sActiveApp)!=NULL
		&& m_Pipeline.GetPipelineApp(m_sActiveApp)->CaptureMouseMotionEvent(this, x, y, dx, dy))
	{
		m_pRenderer->UpdateScreen();
		return;		
	}
	
	bool inWnd;
	double xWnd, yWnd;
	VKString subwindowName = InSubWindow(x, y, &inWnd, &xWnd, &yWnd);

	if (!inWnd)
	{
        if (m_Pipeline.GetDB()->NumModels()==0)
            return;
        
		// camera events
		ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
		assert(m_iSelectedWindow>=0 
			   && m_iSelectedWindow < renderer->m_mWindowToModelID.size());
		int modelID = renderer->m_mWindowToModelID[m_iSelectedWindow];
		Model3D * model = m_Pipeline.GetDB()->GetModelByID(modelID);
		assert(model!=NULL);
		
		if (!Ctrl() && !Alt() && !Shift())	// rotate
			model->RenderingParams()->Rotate(m_iSelectedWindow, x, y, dx, dy);
		else if (Ctrl())
			model->RenderingParams()->Scale(m_iSelectedWindow, x, y, dx, dy);
		else if (Shift())
			model->RenderingParams()->Translate(m_iSelectedWindow, x, y, dx, dy);
	}
	else
		ProcessClickInSubwindow(subwindowName, xWnd, yWnd, false);
	
	m_pRenderer->UpdateScreen();
}

void DatabaseVisWindow::Resize(int w, int h)
{
	m_iWidth = w;
	m_iHeight = h;
	for (int i=0; i<(int)m_pModelsRegisteredForEvents.size(); i++)
		m_pModelsRegisteredForEvents[i]->WindowResize(w, h);
}

ModelPoint DatabaseVisWindow::SelectedPoint()
{
	int modelID = m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID[0];
	if (modelID != -1)
	{
		Model3D * model = m_Pipeline.GetDB()->GetModelByID(modelID);
		assert(model!=NULL);
		return model->RenderingParams()->GetSelectedPoint();
	}
	return ModelPoint();
}

VKString DatabaseVisWindow::CorrValueToRender()
{
	return m_sCorrValueToRender;
}

VKString DatabaseVisWindow::CollectionToRnder(int * saveCorrID)
{
	*saveCorrID = m_iCollectionCorrIDToRender;
	return m_sSelectedCollectionToRender;
}

void DatabaseVisWindow::SetCorrToRender(const VKString & corrCollName, 
										int corrID, 
										const VKString & corrValueName)
{
	m_iCollectionCorrIDToRender = corrID;
	m_sSelectedCollectionToRender = corrCollName;
	m_sCorrValueToRender = corrValueName;
}

VKString DatabaseVisWindow::ModelValueToRender(int subwindowID)
{
	ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
	assert(subwindowID>=0 && subwindowID < (int)renderer->m_mWindowToModelID.size());
	int modelID = renderer->m_mWindowToModelID[subwindowID];
	Model3D * model = m_Pipeline.GetDB()->GetModelByID(modelID);
	assert(model!=NULL);
	return model->RenderingParams()->GetModelValueToRender();
}

void DatabaseVisWindow::SetDesignRender(bool on)
{
	if (on)
		m_iRenderDesign = RENDER_DESIGN_WND | RENDER_DESIGN_CTRL_WND;
	else
		m_iRenderDesign = 0;
}

VKString DatabaseVisWindow::ImageName(int imgID)
{
	VKString nStr = VKString::number(imgID);
	while (nStr.length() < 4)
		nStr = VKString("0") + nStr;
	return VKString("IMG_")+nStr;
}

void DatabaseVisWindow::WriteImages(const VKString & prefix, WriteImgType tp,
									int imgID0, bool createTimeDir, bool avoidDuplicates,
									VKString * saveDir, VKStringList * saveImgNames)
{
	m_WriteImg = tp;
	
	int currDesign = m_iRenderDesign;
	
	VKString writeDir;

	m_iRenderDesign = 0;

	if (tp==WRITE_SCREENSHOT)
	{	
		writeDir = GetWriteToDir(prefix, createTimeDir, avoidDuplicates);
		StartDrawToBuffer();
		Draw();
		BufferID & bufferID = EndDrawToBuffer();	
			
		VKString filename = VKString("IMG_")+GetTime();
		if (imgID0>=0)
			filename = ImageName(imgID0);
			
		WriteCurrentImage(bufferID, writeDir+filename, 
						  true, 0, 0, Width(), Height());
		if (saveImgNames!=NULL)
			saveImgNames->push_back(filename);
	}
	else if (tp==WRITE_SCREENSHOT_ALL)
	{
		writeDir = GetWriteToDir(prefix, true, true);
		VKString htmlFile = (writeDir+"/index.html");
		bool couldGen = m_Pipeline.GetDB()->CreateDirsOnPathToFile(htmlFile);
		assert(couldGen);
		std::ofstream textStream(htmlFile.c_str());
		assert(textStream.is_open());
		textStream<<"<html><head><title>"<<prefix.c_str()<<": "<<GetTime().c_str();
		textStream<<"</title>"<<"</head><body>\n<table border=1>";
		int nWindows = (int)m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID.size()-1;
		int nScreens = (int)m_NewModelOrder.size();
		
		int currTraversalID = m_iCurrModelTraversalID;
		
		int imgID = 0;
		for (int travID=0; travID < nScreens; travID += nWindows)
		{
			SetPendingUpdateToTrue();
			//std::cout<<"Writing image: "<<travID<<" / "<<nScreens<<" INC="<<nWindows<<std::endl;
			FillMostSimilarModels(travID, m_NewModelOrder);
			StartDrawToBuffer();
			Draw();
			BufferID & bufferID = EndDrawToBuffer();	
			VKString filename = VKString("IMG_")+VKString::number(imgID++);
			WriteCurrentImage(bufferID, writeDir+filename, 
							  false, 0, 0, Width(), Height());
			textStream<<"\t<tr><td><img src=\""<<(filename+".png").c_str()<<"\"></td></tr>\n";
			if (saveImgNames!=NULL)
				saveImgNames->push_back(filename);
		}
		m_iCurrModelTraversalID = currTraversalID;
		textStream<<"</table></body></html>\n";
	}
	else if (tp==WRITE_SCREENSHOT_EACH_AS_MAIN)
	{
		int currTraversalID = m_iCurrModelTraversalID;
		int cols = 4;
		writeDir = GetWriteToDir(prefix, true, true);
		VKString htmlFile = (writeDir+"/index.html");
		bool couldGen = m_Pipeline.GetDB()->CreateDirsOnPathToFile(htmlFile);
		assert(couldGen);
		std::ofstream textStream(htmlFile.c_str());
		assert(textStream.is_open());
		textStream<<"<html><head><title>"<<prefix.c_str()<<": "<<GetTime().c_str();
		textStream<<"</title>"<<"</head><body>\n<table border=1>";
		
		int nWindows = (int)m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID.size()-1;
		int mainWnd = m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID[0];
		int imgID = 0;
		double mainCX, mainCY, mainScale, mainDX, mainDY;
		GetWindowParams(0, &mainCX, &mainCY, &mainScale, &mainDX, &mainDY);
		
		int x0 = (int)((mainCX + Width())/2 - mainDX/4);
		int y0 = (int)((mainCY + Height())/2 - mainDY/4);
		int x1 = (int)((mainCX + Width())/2 + mainDX/4);
		int y1 = (int)((mainCY + Height())/2 + mainDY/4);
		
		int nScreens = (int)m_NewModelOrder.size();
		bool smartUpdates = true;
		int nextUpdate = 0;
		for (int i=-1; i<nScreens; i++)
		{
			int modelID;
			if (i==-1)
				modelID = mainWnd;
			else
				modelID = m_NewModelOrder[i];
						
			if (smartUpdates && i==nextUpdate)
			{
				m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID[0] = mainWnd;
				m_iCurrModelTraversalID = i;
				FillMostSimilarModels(m_iCurrModelTraversalID, m_NewModelOrder);
				SetPendingUpdateToTrue();
				StartDrawToBuffer();
				Draw();
				BufferID & bufferID = EndDrawToBuffer();
				nextUpdate += nWindows;
			}
			
			if (i!=-1 && modelID==mainWnd)
				continue;			
			
			m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID[0] = modelID;			
			StartDrawToBuffer();
			Draw();
			BufferID & bufferID = EndDrawToBuffer();	
			VKString filename = ImageName(imgID++);
			WriteCurrentImage(bufferID, writeDir+filename, 
							  true, x0, y0, x1, y1);
			

			
			if ((imgID-1)%cols == 0)
				textStream<<"\t<tr>\n";
			textStream<<"\t\t<td><img width=150 src=\""<<(filename+".png").c_str()<<"\">";
			
			textStream<<"<br>"<<m_Pipeline.GetDB()->GetModelByID(modelID)->ModelName().c_str();
			textStream<<"</td>\n";
			
			if ((imgID)%cols == 0 || i>=m_Pipeline.GetDB()->NumModels()-1)
				textStream<<"\t</tr>\n";

			if (saveImgNames!=NULL)
				saveImgNames->push_back(filename);
		}
		textStream<<"</table></body></html>\n";
		m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID[0] = mainWnd;
		m_iCurrModelTraversalID = currTraversalID;
	}
	else
		assert(false);

	if (saveDir!=NULL)
		*saveDir = writeDir;
	m_WriteImg = WRITE_SCREENSHOT_NONE;
	m_iRenderDesign = currDesign;
}

void DatabaseVisWindow::WriteCurrentImage(BufferID & bufferID, 
										  const VKString & filename, 
										  bool cropToWhite,
										  int x0, int y0, int x1, int y1)
{
	assert(x0>=0 && x0 <= Width());
	assert(x1>=0 && x1 <= Width());
	assert(y0>=0 && y0 <= Height());
	assert(y1>=0 && y1 <= Height());
	assert(x1 > x0 && y1 > y0);
	R2Image img(x1-x0, y1-y0);
	assert(Width()==bufferID.m_iWidth && Height()==bufferID.m_iHeight);
	double minContX = Width()+1;
	double maxContX = -1;
	double minContY = Height()+1;
	double maxContY = -1;
	for (int i=x0; i<x1; i++)
	{
		for (int j=y0; j<y1; j++)	
		{
			int id = (j * Width() + i) * 4;
			
			unsigned char r = bufferID.m_aColorBuffer[id+0];
			unsigned char g = bufferID.m_aColorBuffer[id+1];
			unsigned char b = bufferID.m_aColorBuffer[id+2];
			RNRgb rgb((double)r / 255., (double)g / 255., (double)b / 255.);
			bool zero = (r==255 && g==255 && b==255);
			zero = zero || (r==0 && g==0 && b==0);
			if (!zero)
			{
				minContX = vkMin(minContX, i-x0);
				maxContX = vkMax(maxContX, i-x0);
				minContY = vkMin(minContY, j-y0);
				maxContY = vkMax(maxContY, j-y0);
			}
			img.SetPixelRGB(i, j, rgb);
		}
	}

	//std::cout<<"filename: "<<filename.c_str()<<std::endl;
	bool couldCreate = m_Pipeline.GetDB()->CreateDirsOnPathToFile(filename);
	assert(couldCreate);
	img.WriteBMP((filename+".bmp").c_str());
	VKString cmdConvert = VKString("convert ") + filename + ".bmp ";
	
	if (cropToWhite)
	{
		minContX = vkMax(minContX - 5, 0);
		maxContX = vkMin(maxContX + 5, img.Width()-1);
		minContY = vkMax(minContY - 5, 0);
		maxContY = vkMin(maxContY + 5, img.Height()-1);
		
		int wd = maxContX-minContX;
		int ht = maxContY-minContY;
		minContY = img.Height() - maxContY;

		cmdConvert += VKString("-crop ") + VKString::number(wd);
		cmdConvert += VKString("x")+VKString::number(ht);
		cmdConvert += VKString("+")+VKString::number(minContX);
		cmdConvert += VKString("+")+VKString::number(minContY);
		cmdConvert += " ";
//		cmdConvert += VKString("+repage");
	}
	
	cmdConvert += filename + ".png";
//	std::cout<<cmdConvert.c_str()<<std::endl;
	system(cmdConvert.c_str());
	system((VKString("rm ") + filename + ".bmp ").c_str());
	
	std::cout<<"Printed File: "<<(filename + ".png").c_str()<<std::endl;

}

VKString DatabaseVisWindow::GetWriteToDir(const VKString & prefix, 
										  bool timeSubdir, bool uniqueDir)
{
	VKString imgDir = m_Pipeline.GetDB()->GetMainDir() + "/Images/" + prefix;
	if (timeSubdir)
		imgDir += VKString("/At")+GetTime();

	if (uniqueDir && m_Pipeline.GetDB()->DirExists(imgDir))
	{
		int currID = 1;
		VKString uniqueDirname = imgDir + "_ID"+VKString::number(currID);
		while (m_Pipeline.GetDB()->DirExists(uniqueDirname))
			uniqueDirname = imgDir + "_ID"+VKString::number(++currID);
		imgDir = uniqueDirname;
	}
	return imgDir+"/";
}

VKString DatabaseVisWindow::GetTime()
{
	time_t rawtime;
	struct tm * timeinfo;
	time ( &rawtime );
	timeinfo = localtime ( &rawtime );
	VKString tmStr(asctime (timeinfo));
	tmStr.replace(" ", "__");
	tmStr.replace(":", "_");
	tmStr.replace("\n", "");
	return tmStr;
}

void DatabaseVisWindow::Draw()
{
	// draw log?
	static bool drawLogs = true;
	if (drawLogs)
	{
		drawLogs = false;
		ModelDatabase * db = m_Pipeline.GetDB();
		for (int i=0; i<db->m_CorrCollectionNames.count(); i++)
		{
			CorrCollection * coll = db->GetCorrCollection(db->m_CorrCollectionNames[i]);
			if (coll!=NULL && coll->GetCollectionType()=="CollectionDiffuseDynamicGraph")
			{
				CollectionDiffuseDynamicGraph * diff = (CollectionDiffuseDynamicGraph*)coll;
				diff->DrawLog(this);
			}
		}
	}
	
	//glClearColor(0, 0, 0, 1.0);
	glClearColor(220./255., 222./255, 218./255, 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_ACCUM_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	// draw app
	PipelineStepApplication * app = m_Pipeline.GetPipelineApp(m_sActiveApp);
	if (app!=NULL)
		app->Draw(this);
	
	// draw correspondences
	if (m_iCollectionCorrIDToRender==-1)
	{
		CorrCollection * coll = m_Pipeline.GetDB()->GetCorrCollection(m_sSelectedCollectionToRender);
		if (coll!=NULL)
			coll->Draw(this);
	}
	else
	{
		CollectionSinglePair * coll = NULL;
        coll = m_Pipeline.GetDB()->GetCorrSinglePair(m_sSelectedCollectionToRender,
                                                     m_iCollectionCorrIDToRender);
		if (coll!=NULL)
			coll->Draw(this);
	}
	
	// models
	ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
	int nWnd = (int)renderer->m_mWindowToModelID.size(); 
	
	for (int i=0; i<nWnd; i++)
	{
		int modelID = renderer->m_mWindowToModelID[i];
		if (modelID != -1)
		{
			Model3D * model = m_Pipeline.GetDB()->GetModelByID(modelID);
			if (model!=NULL)
                model->Draw(i);
		}
		
		if (m_iRenderDesign & RENDER_DESIGN_WND)
			DrawWindow(i);
	}
	
	// Draw Subwindows: model list
	for (int i=0; i<(int)m_AvailiableSubWindows.size(); i++)
	{
		bool exists;
		VKStringList corner = m_VisParams.GetStrValues("Visualization", 
													   m_AvailiableSubWindows[i], exists);
		if (!exists)
			continue;
				
		if (m_iRenderDesign & RENDER_DESIGN_CTRL_WND)
		{
			if (corner.count()==3)
			{
				int top, left;
				SubwindowPosition(i, &top, &left);

				if (corner[0]=="ModelList")
				{
					StartRenderSubwindow(top, left);
					DrawModelListSubwindow();
				}
				else if (corner[0]=="DiffusedEmbedding"
						 || corner[0]=="DiffusedAlignmentGraph")
				{
					VKString rndStr = m_sSelectedCollectionToRender;
					CorrCollection * coll = m_Pipeline.GetDB()->GetCorrCollection(rndStr);
					if (coll!=NULL 
						&& coll->GetCollectionType()=="CollectionDiffuseDynamicGraph")
					{
						StartRenderSubwindow(top, left);
						CollectionDiffuseDynamicGraph * corrDiffused;
						corrDiffused = (CollectionDiffuseDynamicGraph*)coll;
						if (corner[0]=="DiffusedEmbedding")
							corrDiffused->DrawAllEmbeddedPoints(this);
						else if (corner[0]=="DiffusedAlignmentGraph")
							corrDiffused->DrawAlignmentGraph(this);
						else
							assert(false);
					}
				}
			}
		}
	}

	if (m_UIMode==UIMode_SELECT_WINDOW)
		m_pRenderer->RenderText(R2Point(-.2, .6), VKString("Selecting Window: ")+VKString::number(m_iSelectedWindow), 
								1, 0, 0, VisWindowRenderer::TEXT_SIZE_LARGE);
	else if (m_UIMode!=UIMode_NONE)
	{
		VKString prefix("Selection:");
		if (m_UIMode==UIMode_SELECT_MODEL)
			prefix = VKString("Selecting Model: ");
		
		m_pRenderer->RenderText(R2Point(-.2, .6), prefix+VKString::number(m_iCurrentSelection), 
								1, 0, 0, VisWindowRenderer::TEXT_SIZE_LARGE);
	}
	
	UpdatedCorrValueRendering();
}

void DatabaseVisWindow::DrawModelListSubwindow()
{
	
	
	VKStringList * renderList=NULL;
	int nStart = -1;
	int nEnd = -1;
	int selectedID = -1;
	assert(m_iSelectedList >=0 && m_iSelectedList < m_sLists.count());
	
	int activeWnd = m_iSelectedWindow;
	if (m_sLists[m_iSelectedList]=="Cor" || m_iSelectedList==CORRS_SELECTION_ARROWS )
		activeWnd=0;
	
	int modelID = m_Pipeline.GetDB()->GetRenderParams()->m_mWindowToModelID[activeWnd];
	Model3D * model = m_Pipeline.GetDB()->GetModelByID(modelID);
	
	int selectedElements = -1;			// for a selected item in the list - render:
	int selectedOutOfElements = -1;		//			(selectedElements / selectedOutOfElements)
	std::vector<int> otherOutOfElements;	// for all other items render (otherOutOfElements)
	
	if (m_sLists[m_iSelectedList]=="Mdl")
	{
		selectedID = modelID;
		nStart = m_Pipeline.GetDB()->GetRenderParams()->m_iModelListStart;
		nEnd = m_Pipeline.GetDB()->GetRenderParams()->m_iModelListEnd;
		renderList = &(m_Pipeline.GetDB()->GetRenderParams()->m_ViewableModelNames);
	}
	else if (m_sLists[m_iSelectedList]=="Pts")
	{
		renderList = &(m_Pipeline.GetDB()->m_PointSetNames);
		selectedID = renderList->containsGetID(model->RenderingParams()->GetPointSetToRender());
	}
	else if (m_sLists[m_iSelectedList]=="Dst")
	{
		renderList = &(m_Pipeline.GetDB()->m_DistanceNames);
		selectedID = renderList->containsGetID(model->RenderingParams()->GetDistanceToRender());
	}
	else if (m_sLists[m_iSelectedList]=="Val")
	{
		renderList = &(m_Pipeline.GetDB()->m_ModelValueNames);
		selectedID = renderList->containsGetID(model->RenderingParams()->GetModelValueToRender());
	}
	else if (m_sLists[m_iSelectedList]=="Cor" || m_iSelectedList == CORRS_SELECTION_ARROWS )
	{
		renderList = &(m_Pipeline.GetDB()->m_CorrCollectionNames);		
		selectedID = renderList->containsGetID(m_sSelectedCollectionToRender);
		CorrCollection * corrCollect = m_Pipeline.GetDB()->GetCorrCollection(m_sSelectedCollectionToRender);
		if (corrCollect!=NULL && corrCollect->SafeCastToPairwise()!=NULL)
		{
			selectedOutOfElements = corrCollect->SafeCastToPairwise()->NumModelCorrs();
			selectedElements = m_iCollectionCorrIDToRender;
		}
		for (int i=0; i<renderList->count(); i++)
		{
			corrCollect = m_Pipeline.GetDB()->GetCorrCollection((*renderList)[i]);
			if (corrCollect!=NULL && corrCollect->SafeCastToPairwise()!=NULL)
				otherOutOfElements.push_back(corrCollect->SafeCastToPairwise()->NumModelCorrs());
		}
	}
	else if (m_sLists[m_iSelectedList]=="CVal")
	{
		renderList = &(m_Pipeline.GetDB()->m_CorrValueNames);
		selectedID = renderList->containsGetID(m_sCorrValueToRender);
	}
	else 
		assert(false);
	
	assert(renderList!=NULL);
	if (nStart==-1)
		nStart = 0;
	if (nEnd==-1)
		nEnd = renderList->count();
	
	for (int i=nStart; i<vkMin(nEnd, renderList->count()); i++)
	{
		double r=1., g=0, b=0;
		bool selected = false;
		VKString text = (*renderList)[i];
		if (selectedID==i)
		{
			r=0.; g=0; b=1.;
			if (selectedElements!=-1 && selectedOutOfElements!=-1)
			{
				text += VKString(" (")+VKString::number(selectedElements);
				text += VKString(" / ")+VKString::number(selectedOutOfElements)+")";
			}
		}
		else if (i < (int)otherOutOfElements.size())
			text += VKString(" (") + VKString::number(otherOutOfElements[i])+")";
				
		m_pRenderer->RenderText(R3Point(.05, .9 - .85*(double)(i-nStart) / (double)(nEnd - nStart), 0), text, 
								r, g, b, VisWindowRenderer::TEXT_SIZE_SMALL);
	}
	
	double bottomY = 0.01;
	double rightX = .8;
	
	// Render bottom (page) selection
	VKString idsToRender = VKString::number(nStart)+" - " + VKString::number(nEnd-1);
	if (nEnd==0)
		idsToRender = "Empty List";
	m_pRenderer->RenderText(R3Point(.25, bottomY, 0), idsToRender, 
							.4, .4, .4, VisWindowRenderer::TEXT_SIZE_MEDIUM);
	if (m_sLists[m_iSelectedList]=="Mdl")
	{
		m_pRenderer->RenderText(R3Point(.05, bottomY, 0), VKString("<"), 
								.4, .4, .4, VisWindowRenderer::TEXT_SIZE_LARGE);
		m_pRenderer->RenderText(R3Point(.9, bottomY, 0), VKString(">"), 
								.4, .4, .4, VisWindowRenderer::TEXT_SIZE_LARGE);
	}

	// Render right (list) selection
	for (int i=0; i<m_sLists.count(); i++)
	{
		double r =.4, g=.4, b = .4;
		if (m_iSelectedList==i || (m_iSelectedList==CORRS_SELECTION_ARROWS && m_sLists[i]=="Cor"))
		{		r = .1; g = .1; b = .8;			}
		m_pRenderer->RenderText(R3Point(rightX, .9 - i * .1, 0), m_sLists[i], 
								r, g, b, VisWindowRenderer::TEXT_SIZE_MEDIUM);
	}
	glColor3d(0, 0, 1.);
	glLineWidth(1.);
	glBegin(GL_LINE_STRIP);
	glVertex2d(rightX-.01, .89 - m_iSelectedList * .1);
	glVertex2d(.95, .89 - m_iSelectedList * .1);
	glEnd();
}

void DatabaseVisWindow::ProcessClickInSubwindow(const VKString & wndName,
												double xWnd, double yWnd, 
												bool justPressed)
{
	if (wndName=="ModelList")
		ClickInModelList(xWnd, yWnd);
	else if (wndName=="DiffusedEmbedding"
			 || wndName=="DiffusedAlignmentGraph")
	{
		VKString selStr = m_sSelectedCollectionToRender;
		CorrCollection * coll = m_Pipeline.GetDB()->GetCorrCollection(selStr);
		CollectionDiffuseDynamicGraph * corrDiffused;
		if (coll!=NULL && coll->GetCollectionType()=="CollectionDiffuseDynamicGraph")
		{
			corrDiffused = (CollectionDiffuseDynamicGraph*)coll;
			if (wndName=="DiffusedEmbedding")
				corrDiffused->SelectEmbeddedPoint(this, xWnd, yWnd);
			else if (wndName=="DiffusedAlignmentGraph")
				corrDiffused->SelectNodeInAlignmentGraph(this, xWnd, yWnd, 
														 justPressed);
			else
				assert(false);
		}
	}
}

void DatabaseVisWindow::ClickInModelList(double xWnd, double yWnd)
{
	SetPendingUpdateToTrue();
	yWnd = 1. - yWnd;
	//int oldSelection = m_iSelectedList;
	if (xWnd > .8)			// right area
	{
		int selection = (yWnd-.05) / .1;
		
		if (selection < m_sLists.count() && selection>=0)
			m_iSelectedList = selection;
	}
	
	int mask = Model3DRenderingParams::RENDER_MAIN_FLAGS_MASK;
	int nStart = 0;
	int nEnd = 0;
	int nMax = 0;
	int dn = 0;

	ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
	VKStringList * renderList = NULL;
	bool resetFlags = true;
	int newFlag = 0;
	if (m_sLists[m_iSelectedList]=="Mdl")
	{
		nStart = renderer->m_iModelListStart;
		nEnd = renderer->m_iModelListEnd;
		nMax = renderer->m_ViewableModelNames.count();
		newFlag = Model3DRenderingParams::RENDER_FACES;
	}
	else
	{
		if (m_sLists[m_iSelectedList]=="Pts")
		{
			renderList = &(m_Pipeline.GetDB()->m_PointSetNames);
			newFlag = Model3DRenderingParams::RENDER_POINT_SET;
		}
		else if (m_sLists[m_iSelectedList]=="Dst")
		{
			renderList = &(m_Pipeline.GetDB()->m_DistanceNames);
			newFlag = Model3DRenderingParams::RENDER_DISTANCE;
		}
		else if (m_sLists[m_iSelectedList]=="Val")
		{
			renderList = &(m_Pipeline.GetDB()->m_ModelValueNames);
			newFlag = Model3DRenderingParams::RENDER_VALUE;
		}
		else if (m_sLists[m_iSelectedList]=="Cor" || m_iSelectedList==CORRS_SELECTION_ARROWS )
		{
			SetCorrFlag(RENDER_CORRS_SET, true);
			renderList = &(m_Pipeline.GetDB()->m_CorrCollectionNames);
			if (m_iSelectedList==CORRS_SELECTION_ARROWS)
			{
				m_iSelectedList--;
				assert(m_sLists[m_iSelectedList]=="Cor");
				newFlag = -1;
			}
			else 
			{
				//mask = Model3DRenderingParams::RENDER_ALL_FLAGS; 
				//newFlag = Model3DRenderingParams::RENDER_CORRS 
				//			| Model3DRenderingParams::RENDER_FACES
				//			| Model3DRenderingParams::RENDER_CORRS_EDGES;
				newFlag = -1;
			}

		}
		else if (m_sLists[m_iSelectedList]=="CVal")
		{
			renderList = &(m_Pipeline.GetDB()->m_CorrValueNames);
			//resetFlags = false;
			//mask = Model3DRenderingParams::RENDER_ALL_FLAGS; 
			//newFlag = Model3DRenderingParams::RENDER_CORRS_CVALUE;
			newFlag = -1;
		}
		assert(renderList!=NULL);
		nEnd = renderList->count();
		nMax = renderList->count();		
	}
	dn = nEnd-nStart;

	if (xWnd > .8 && yWnd <= .9)			// right area
	{
		if (resetFlags)
		{
			for (int i=0; i<m_Pipeline.GetDB()->NumModels(); i++)
			{
				if (newFlag!=-1)
					m_Pipeline.GetDB()->GetModelByID(i)->RenderingParams()->SetFlags(newFlag, mask);
			}
		}
		int selection = (yWnd-.05) / .1;

		if (selection < m_sLists.count() && selection>=0
			&& selection==CORRS_SELECTION_ARROWS)
		{
			int modelID = renderer->m_mWindowToModelID[0];
			double margin = .1;
			if (xWnd < .8 + margin)
				m_iCollectionCorrIDToRender--;
			else if (xWnd>1.-margin)
				m_iCollectionCorrIDToRender++;
			
			CorrCollection * coll = m_Pipeline.GetDB()->GetCorrCollection(m_sSelectedCollectionToRender);
			if (coll==NULL)
				m_iCollectionCorrIDToRender = -1;
			else
			{
				CollectionPairwise * collPairwise = coll->SafeCastToPairwise();
				if (collPairwise==NULL)
					m_iCollectionCorrIDToRender = -1;
				else
				{
					int nCorrs = collPairwise->NumModelCorrs();
					if (m_iCollectionCorrIDToRender<=-2)
						m_iCollectionCorrIDToRender = nCorrs-1;
					else if (m_iCollectionCorrIDToRender>=nCorrs)
						m_iCollectionCorrIDToRender = 0;
				}
			}
			
		}
	}
	else if (yWnd > .9)		// bottom choices
	{
		if (xWnd <= .5)
			nStart -= dn;
		else if (xWnd > .5)
			nStart += dn;
		
		if (nStart >= nMax)
			nStart = 0;
		else if (nStart < 0)
			nStart = vkMax(nMax - dn, 0);
		nEnd = nStart + dn;
		renderer->m_iModelListStart = nStart;
		renderer->m_iModelListEnd = nEnd;
	}
	else	// main rendering list
	{
		int selectedID = nStart + (yWnd-.05) / .85 * dn;

		if (selectedID>=0 && selectedID < nMax)
		{
			if (m_sLists[m_iSelectedList]=="Mdl")
				renderer->m_mWindowToModelID[m_iSelectedWindow] = selectedID;
			else
			{
				assert(renderList!=NULL);
				assert(selectedID < renderList->count());
				VKString nameToRender = (*renderList)[selectedID];
				
				for (int i=0; i<m_Pipeline.GetDB()->NumModels(); i++)
				{
					Model3D * model = m_Pipeline.GetDB()->GetModelByID(i);
					if (m_sLists[m_iSelectedList]=="Pts")
						model->RenderingParams()->SetPointSetToRender(nameToRender);
					else if (m_sLists[m_iSelectedList]=="Dst")
						model->RenderingParams()->SetDistanceToRender(nameToRender);
					else if (m_sLists[m_iSelectedList]=="Val")
						model->RenderingParams()->SetModelValueToRender(nameToRender);
					else if (m_sLists[m_iSelectedList]=="Cor" || m_iSelectedList==CORRS_SELECTION_ARROWS )
					{
						m_sSelectedCollectionToRender = nameToRender;
						if (m_iSelectedList!=CORRS_SELECTION_ARROWS)
							m_iCollectionCorrIDToRender = -1;
					}
					else if (m_sLists[m_iSelectedList]=="CVal")
						m_sCorrValueToRender = nameToRender;
					else 
						assert(false);
				}
			}		
		}
	}
}


void DatabaseVisWindow::DrawWindow(int wndID)
{
	double r=1., g=0., b=0.;
	if (wndID == m_iSelectedWindow)
	{
		r = 0.;	g = 0.;	b = 1.;
	}
	double textW=.03;
	double textH=.04;

	double cx, cy, scale, dx, dy;
	GetWindowParams(wndID, &cx, &cy, &scale, &dx, &dy);
	double textX = (cx-dx/2.) / Width() + textW;
	double textY = (cy) / Height() - textH;
	m_pRenderer->RenderText(R2Point(textX, textY), VKString::number(wndID), 
							r, g, b, VisWindowRenderer::TEXT_SIZE_MEDIUM);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glDisable(GL_LIGHTING);
	glColor3d(r, g, b);
	glLineWidth(1.);
	glBegin(GL_LINE_STRIP);
	glVertex2d(textX - textW*.15, textY - textH*.15);
	glVertex2d(textX - textW*.15, textY + textH);
	glVertex2d(textX + textW, textY + textH);
	glVertex2d(textX + textW, textY  - textH*.15);
	glVertex2d(textX - textW*.15, textY - textH*.15);
	glEnd();
	
	double minX = (cx - dx/2.) / Width();
	double maxX = (cx + dx/2.) / Width();
	double minY = (cy - dy/2.) / Height();
	double maxY = (cy + dy/2.) / Height();
	
	double off = 0.01;
	glBegin(GL_LINE_STRIP);
	glVertex2d(minX + off, minY + off);
	glVertex2d(minX + off, maxY - off);
	glVertex2d(maxX - off, maxY - off);
	glVertex2d(maxX - off, minY + off);
	glVertex2d(minX + off, minY + off);
	glEnd();
	
	// draw model / corr value range
	
	ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
	
	int modelID = renderer->m_mWindowToModelID[wndID];
    Model3D * model = NULL;
	if (modelID != -1)
        model = m_Pipeline.GetDB()->GetModelByID(modelID);
    
    if (model!=NULL)
	{        
		VKString vlStr=model->RenderingParams()->m_sValueType;
		if (vlStr!="")
		{
			double mnv = model->RenderingParams()->m_fMinValue;
			double mxv = model->RenderingParams()->m_fMaxValue;
			double mean = model->RenderingParams()->m_fMeanValue;
			double ave = model->RenderingParams()->m_fAveValue;
			double xPos = minX + (maxX-minX)*.4;
			GetRenderer()->RenderText(R2Point(xPos, maxY - textH), vlStr, 1, 0, 0, 
									  VisWindowRenderer::TEXT_SIZE_SMALL);
			VKString str = VKString(" min=") + VKString::number(mnv) + ", max=" + VKString::number(mxv);
			GetRenderer()->RenderText(R2Point(xPos, maxY - 2*textH), str, 1, 0, 0, 
									  VisWindowRenderer::TEXT_SIZE_SMALL);			
			VKString str2 = VKString(" ave=") + VKString::number(ave) + ", mean=" + VKString::number(mean);
			GetRenderer()->RenderText(R2Point(xPos, maxY - 3*textH), str2, 1, 0, 0, 
									  VisWindowRenderer::TEXT_SIZE_SMALL);

		}
	}		
}

int DatabaseVisWindow::ClickInWindow(int x, int y)
{
	ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
	int nWnd = (int) renderer->m_mWindowToModelID.size();
	double fx = (double) x / (double)Width();
	double fy = (double) y / (double)Height();
	
	fx = fx * 2. - 1.;
	fy = fy * 2. - 1.;
	for (int i=0; i < nWnd; i++)
	{
		double cx, cy, scale, dx, dy;
		GetWindowParams(i, &cx, &cy, &scale, &dx, &dy);
		double minX = (cx - dx/2.) / Width();
		double maxX = (cx + dx/2.) / Width();
		double minY = (cy - dy/2.) / Height();
		double maxY = (cy + dy/2.) / Height();
//		double textX = (cx-dx/2.) / Width() + textW;
//		double textY = (cy) / Height() - textH;
		if (fx >= minX && fx <= maxX && fy>=minY && fy<=maxY)
			return i;
	}
	return -1;
}

void DatabaseVisWindow::InitializeGL()
{
	// Initialize background color 
	//glClearColor(200.0/255.0, 200.0/255.0, 200.0/255.0, 1.0);
	
	// Initialize lights
	static GLfloat lmodel_ambient[] = { 0.2, 0.2, 0.2, 1.0 };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
	static GLfloat light0_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
	glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
	glEnable(GL_LIGHT0);
	static GLfloat light1_diffuse[] = { 0.5, 0.5, 0.5, 1.0 };
	glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_diffuse);
	glEnable(GL_LIGHT1);
	glEnable(GL_NORMALIZE);
	glEnable(GL_LIGHTING); 
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	
	// Initialize graphics modes  
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);	
	
	// initialize alpha-blending
	glEnable (GL_BLEND); 
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	
	glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
}

void DatabaseVisWindow::GetWindowParams(int wndID, double * cx, double * cy, double * scale,
										double * dx, double * dy)
{
	std::vector<int> nMainWnd = m_VisParams.GetIntValues("Visualization", "NMainWnd");
	std::vector<int> nAdditionalWnd = m_VisParams.GetIntValues("Visualization", "NAddWnd");
	assert(nMainWnd.size()==2 && nAdditionalWnd.size()==2);

	double mainFrac = m_VisParams.GetDoubleValue("Visualization", "MainWndSize");
	double minX = (double)-Width();
	double maxX = (double)Width();
	double minY = (double)-Height();
	double maxY = (double)Height();
	
	double mainWndSizeX = (maxX - minX) * mainFrac;
	double mainWndSizeY = (maxY - minY);

	if (wndID < nMainWnd[0] * nMainWnd[1])	// main window
	{
		double wndInMainDX = mainWndSizeX / nMainWnd[1];
		double wndInMainDY = mainWndSizeY / nMainWnd[0];
		int r = wndID / nMainWnd[1];
		int c = wndID % nMainWnd[1];
				
		*cy = minY + wndInMainDY/2. + r*wndInMainDY;
		*cx = minX + wndInMainDX/2. + c*wndInMainDX;
		
		*scale = mainFrac * 1. / vkMax(nMainWnd[0], nMainWnd[1]);
		
		if (dx!=NULL)
			*dx = wndInMainDX;
		if (dy!=NULL)
			*dy = wndInMainDY;
	}
	else 
	{
		wndID -= nMainWnd[0] * nMainWnd[1];
		assert(wndID>=0);
		double addWndSizeX = (maxX - minX) * (1.-mainFrac);
		double addWndSizeY = (maxY - minY);
		
		double wndInAddDX = addWndSizeX / nAdditionalWnd[1];
		double wndInAddDY = addWndSizeY / nAdditionalWnd[0];
		
		int r = wndID / nAdditionalWnd[1];
		int c = wndID % nAdditionalWnd[1];
		*cy = minY + wndInAddDY/2. + r*wndInAddDY  ;
		*cx = minX + mainWndSizeX + wndInAddDX/2. + c*wndInAddDX;
		
		*scale = (1.-mainFrac) * 1. / vkMax(nAdditionalWnd[0], nAdditionalWnd[1]);
		*scale *= 3;
		
		if (dx!=NULL)
			*dx = wndInAddDX;
		if (dy!=NULL)
			*dy = wndInAddDY;
		
	}
}

void DatabaseVisWindow::WriteSurfaceValues(const VKString & dirName)
{
	std::cout<<"[WARNING] WriteSurfaceValues - not implemented"<<std::endl;
}

bool DatabaseVisWindow::IsCorrFlagSet(CorrRenderFlags flag)
{
	return (m_iCorrRenderFlags & flag) != 0;
}

void DatabaseVisWindow::ToggleCorrFlag(CorrRenderFlags flag)
{
	if (IsCorrFlagSet(flag))
		m_iCorrRenderFlags = m_iCorrRenderFlags & (~flag);
	else
		m_iCorrRenderFlags = m_iCorrRenderFlags | flag;
}

void DatabaseVisWindow::SetCorrFlag(CorrRenderFlags flag, bool on)
{
	if (IsCorrFlagSet(flag)!=on)
		ToggleCorrFlag(flag);
}

void DatabaseVisWindow::SetPendingUpdateToTrue()
{
	m_bPendingCorrValueRenderUpdate = true;
	ModelDatabaseRenderingParams * renderer = m_Pipeline.GetDB()->GetRenderParams();
	int nWnd = (int)renderer->m_mWindowToModelID.size(); 
	
	for (int i=0; i<nWnd; i++)
	{
		int modelID = renderer->m_mWindowToModelID[i];
		if (modelID != -1)
		{
			Model3D * model = m_Pipeline.GetDB()->GetModelByID(modelID);
			assert(model!=NULL);
			model->RenderingParams()->FreePrecomputedColors();
		}
	}
	
}

bool DatabaseVisWindow::PendingCorrValueRenderingUpdate()
{
	return m_bPendingCorrValueRenderUpdate;
}

void DatabaseVisWindow::UpdatedCorrValueRendering()
{
	m_bPendingCorrValueRenderUpdate = false;
}

int DatabaseVisWindow::RegisterModel(Model3DRenderingParams * modelRenderParams)
{
	m_pModelsRegisteredForEvents.push_back(modelRenderParams);
	return (int)m_pModelsRegisteredForEvents.size()-1;
}

void DatabaseVisWindow::SetCurrCameraModel(int modelID)
{
	m_iCurrCamera = modelID;
}

Model3DRenderingParams * DatabaseVisWindow::GetCurrCameraModel()
{
	assert(m_iCurrCamera >= 0 && m_iCurrCamera < (int)m_pModelsRegisteredForEvents.size());
	return m_pModelsRegisteredForEvents[m_iCurrCamera];
}

int DatabaseVisWindow::SelectedWindowID()
{
	return m_iSelectedWindow;
}

void DatabaseVisWindow::WriteCameras()
{
	// write all camera transformations
	ModelDatabase * db = m_Pipeline.GetDB();
	VKString appDir = db->GetMainDir();
	
	std::ofstream textStream((appDir+"CameraTransformations.txt").c_str());
	textStream<<"DB "<<db->DatabaseName().c_str()<<" "<<db->NumModels()<<"\n";
	for (int modelID = 0; modelID < db->NumModels(); modelID++)
		db->GetModelByID(modelID)->RenderingParams()->WriteCameras(textStream);
}

void DatabaseVisWindow::ReadCameras()
{
	VKString tempStr;
	ModelDatabase * db = m_Pipeline.GetDB();
	VKString appDir = db->GetMainDir();

	std::ifstream textStream((appDir+"CameraTransformations.txt").c_str());
	
	bool noVis = false;
	if (textStream.is_open())
	{
		tempStr.readToken(textStream);		assert(tempStr=="DB");
		tempStr.readToken(textStream);		assert(tempStr==db->DatabaseName());
		int nModels;
		textStream>>nModels;
		assert(nModels==db->NumModels());
		
		for (int modelID = 0; modelID < db->NumModels(); modelID++)
		{
			if (db->GetModelByID(modelID)->RenderingParams()==NULL)
			{
				noVis = true;
				break;
			}
			db->GetModelByID(modelID)->RenderingParams()->ReadCameras(textStream);
		}
	}
}

void DatabaseVisWindow::DisableApp()
{
	m_sActiveApp = "none";
}

void DatabaseVisWindow::EnableApp()
{
	if (m_Pipeline.m_sAppNames.count() > 0)
		m_sActiveApp = m_Pipeline.m_sAppNames[0];	
}

