/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#include "gaps.h"

#include "VkFunctions.h"

#include "ModelPerVertexValues.h"
#include "Model3D.h"
#include "VisWindow.h"
#include "CollectionIdentityPointsets.h"
#include "DistanceEuclidean.h"

/// interpolator descriptor
//ModelValuesInterpolatorDescriptor::ModelValuesInterpolatorDescriptor()
//{
//	m_Interpolate = MODEL_INTERPOLATE_NEAREST_NEIGHBOR;
//	m_sDistanceMetricName = "auto";
//	m_sInterpolateFromSet = "auto";
//	m_sInterpolateToSet = "auto";
//	
//}
//
//ModelValuesInterpolatorDescriptor::ModelValuesInterpolatorDescriptor(const VKString & distanceMetric, 
//																	 double sigma)
//{
//	assert(false);
//}



/// model values
ModelPerVertexValues::ModelPerVertexValues(Model3D * model, bool pointCloudOnly, bool allocateAsNeededOnly)
{
	m_pRenderR = NULL;
	m_pRenderG = NULL;
	m_pRenderB = NULL;
	m_bPointCloudOnly = pointCloudOnly;
	
	m_pModel = model;
	
	m_bAllVertexValuesAllocated = !allocateAsNeededOnly;
	m_PerVertexValues = NULL;
	m_PerVertexValuesExist = NULL;
	if (m_bAllVertexValuesAllocated)
		AllocatePerVertexStorage();
	
	m_iNumValuesNotSet = NEntries();
	
	m_fMinValue = FLT_MAX;
	m_fMaxValue = -FLT_MAX;	
}

ModelPerVertexValues::~ModelPerVertexValues()
{
	if (m_PerVertexValues!=NULL)
		delete [] m_PerVertexValues;
	
	if (m_PerVertexValuesExist!=NULL)
		delete [] m_PerVertexValuesExist;	
}

void ModelPerVertexValues::ClearValues()
{
	m_iNumValuesNotSet = NEntries();
	if (m_bAllVertexValuesAllocated)
		for (int i=0; i<NEntries(); i++)
			m_PerVertexValuesExist[i] = false;
	else
		m_VertexToValueMap.clear();
	
}

void ModelPerVertexValues::CopyTo(ModelPerVertexValues * copy) const
{
	assert(m_pModel==copy->m_pModel);
	if (m_pRenderR!=NULL)
	{
		if (copy->m_pRenderR==NULL)
			copy->m_pRenderR = m_pRenderR->CreateCopy();
		else
			m_pRenderR->CopyTo(copy->m_pRenderR);
	}

	if (m_pRenderG!=NULL)
	{
		if (copy->m_pRenderG==NULL)
			copy->m_pRenderG = m_pRenderG->CreateCopy();
		else
			m_pRenderG->CopyTo(copy->m_pRenderG);		
	}

	if (m_pRenderB!=NULL)
	{
		if (copy->m_pRenderB==NULL)
			copy->m_pRenderB = m_pRenderB->CreateCopy();
		else
			m_pRenderB->CopyTo(copy->m_pRenderB);		
	}
	
	for (int i=0; i<(int)m_EdgesIsoContours.size(); i++)
		copy->m_EdgesIsoContours.push_back(m_EdgesIsoContours[i]);
	
	copy->m_fMinValue = m_fMinValue;
	copy->m_fMaxValue = m_fMaxValue;
	
	if (m_PerVertexValues!=NULL)
	{
		assert(m_PerVertexValuesExist!=NULL);
		if (copy->m_PerVertexValues==NULL || copy->m_PerVertexValuesExist==NULL)
			copy->AllocatePerVertexStorage();
		assert(copy->m_PerVertexValues!=NULL && copy->m_PerVertexValuesExist!=NULL);
		
		for (int i=0; i<NEntries(); i++)
		{
			copy->m_PerVertexValuesExist[i] = m_PerVertexValuesExist[i];
			copy->m_PerVertexValues[i] = m_PerVertexValues[i];
		}
	}
	copy->m_VertexToValueMap = m_VertexToValueMap;
	copy->m_iNumValuesNotSet = m_iNumValuesNotSet;
}

void ModelPerVertexValues::TakeMaxValues(ModelPerVertexValues * anotherVals)
{
	if (m_pRenderR!=NULL)
		delete m_pRenderR;
	if (m_pRenderG!=NULL)
		delete m_pRenderG;
	if (m_pRenderB!=NULL)
		delete m_pRenderB;
	
	m_pRenderR = NULL;
	m_pRenderG = NULL;
	m_pRenderB = NULL;
	
	m_fMinValue = FLT_MAX;
	m_fMaxValue = FLT_MIN;
		
	m_iNumValuesNotSet = NEntries();
	assert(anotherVals->m_PerVertexValues!=NULL && m_PerVertexValues!=NULL);
	
	for (int i=0; i<NEntries(); i++)
	{
		bool existMe = m_PerVertexValuesExist[i];
		bool existOther = anotherVals->m_PerVertexValuesExist[i];
		if (existMe || existOther)
		{
			if (existMe && existOther)
				m_PerVertexValues[i] = vkMax(m_PerVertexValues[i], anotherVals->m_PerVertexValues[i]);
			else if (existOther)
				m_PerVertexValues[i] = anotherVals->m_PerVertexValues[i];
			m_fMinValue = vkMin(m_fMinValue, m_PerVertexValues[i]);
			m_fMaxValue = vkMax(m_fMaxValue, m_PerVertexValues[i]);
			m_iNumValuesNotSet--;
		}
	}
}

ModelPerVertexValues * ModelPerVertexValues::CreateCopy() const
{
	ModelPerVertexValues * copy = new ModelPerVertexValues(m_pModel, m_bPointCloudOnly, 
														   !m_bAllVertexValuesAllocated);
	CopyTo(copy);	
	return copy;
}


double ModelPerVertexValues::GetValueAtVertex(int vertexID)
{
	if (m_bAllVertexValuesAllocated)
	{
		assert(m_PerVertexValuesExist[vertexID]);
		return m_PerVertexValues[vertexID];
	}
	else
	{
		std::map<int, double>::iterator iter = m_VertexToValueMap.find(vertexID);
		assert(iter != m_VertexToValueMap.end());
		return iter->second;
	}
}

void ModelPerVertexValues::SetVertexValue(int vertexID, double value)
{
	m_fMinValue = vkMin(value, m_fMinValue);
	m_fMaxValue = vkMax(value, m_fMaxValue);	
	
	if (m_bAllVertexValuesAllocated)
	{
		m_PerVertexValues[vertexID] = value;
		if (!m_PerVertexValuesExist[vertexID])
			m_iNumValuesNotSet--;
		m_PerVertexValuesExist[vertexID] = true;
	}
	else
	{
		std::map<int, double>::iterator iter = m_VertexToValueMap.find(vertexID);
		
		if (iter == m_VertexToValueMap.end())
		{
			m_iNumValuesNotSet--;
			m_VertexToValueMap[vertexID] = value;
		}
		else
			iter->second = value;
	}
}

void ModelPerVertexValues::SetVertexValue(const ModelPoint & atVertexPnt, double value)
{
	bool atVertex;
	int vertexID = atVertexPnt.NearestVertex(&atVertex);
	assert(atVertex);
	SetVertexValue(vertexID, value);
}

double ModelPerVertexValues::GetInterpolatedValue(const ModelPoint & pnt)
{
	if (m_bPointCloudOnly)		// connected point cloud
	{
		int pointCloudID = pnt.PntCloudID();
		assert(pointCloudID!=-1);
		if (m_bAllVertexValuesAllocated)
		{
			assert(m_PerVertexValuesExist[pointCloudID]);
			return m_PerVertexValues[pointCloudID];
		}
		else 
		{
			std::map<int, double>::iterator iter = m_VertexToValueMap.find(pointCloudID);
			assert(iter!=m_VertexToValueMap.end());
			return iter->second;
		}
	}
	else		// vertex id
	{
		bool atVertex;
		int vertexID = pnt.NearestVertex(&atVertex);
		if (atVertex)
		{
			if (m_bAllVertexValuesAllocated)
			{
				assert(m_PerVertexValuesExist[vertexID]);
				return m_PerVertexValues[vertexID];
			}
			else
			{
				std::map<int, double>::iterator iter = m_VertexToValueMap.find(vertexID);
				assert(iter!=m_VertexToValueMap.end());
				return iter->second;
			}
		}
		else
		{
			if (m_bAllVertexValuesAllocated)
			{
				assert(m_PerVertexValuesExist[pnt.VertID(0)]);
				assert(m_PerVertexValuesExist[pnt.VertID(1)]);
				assert(m_PerVertexValuesExist[pnt.VertID(2)]);
				return pnt.Interpolate(m_PerVertexValues[pnt.VertID(0)], 
									   m_PerVertexValues[pnt.VertID(1)], 
									   m_PerVertexValues[pnt.VertID(2)]);
			}
			else
			{
				std::map<int, double>::iterator iter1 = m_VertexToValueMap.find(pnt.VertID(0));
				assert(iter1 != m_VertexToValueMap.end());
				std::map<int, double>::iterator iter2 = m_VertexToValueMap.find(pnt.VertID(1));
				assert(iter2 != m_VertexToValueMap.end());			
				std::map<int, double>::iterator iter3 = m_VertexToValueMap.find(pnt.VertID(2));
				assert(iter3 != m_VertexToValueMap.end());			
				
				return pnt.Interpolate(iter1->second, iter2->second, iter3->second);
			}
		}	
	}
}

ModelPoint ModelPerVertexValues::NextAtVertexSampleRequiredForInterpolation(const ModelPoint & pnt)
{
	if (m_bPointCloudOnly)
	{
		assert(pnt.PntCloudID()!=-1);
		return pnt;
	}
	else 
	{
		int vertexID = NextVertexRequiredForInterpolation(pnt);
		if (vertexID == -1)
			return ModelPoint();
		else 
			return ModelPoint(vertexID, m_pModel, pnt.PntCloudID());
	}
}

int ModelPerVertexValues::NextVertexRequiredForInterpolation(const ModelPoint & pnt)
{
	bool atVertex;
	int vertexID = pnt.NearestVertex(&atVertex);
	if (m_bPointCloudOnly)
	{
		assert(atVertex);
		int pointCloudID = pnt.PntCloudID();
		if (m_bAllVertexValuesAllocated)
			return m_PerVertexValuesExist[pointCloudID] ? -1 : vertexID;
		else
			return (m_VertexToValueMap.find(pointCloudID)!=m_VertexToValueMap.end()) ? -1 : vertexID;
	}
	else	
	{
		if (atVertex)
		{
			if (m_bAllVertexValuesAllocated)
				return m_PerVertexValuesExist[vertexID] ? -1 : vertexID;
			else
				return (m_VertexToValueMap.find(vertexID)!=m_VertexToValueMap.end()) ? -1 : vertexID;
		}
		else
		{
			for (int i=0; i<3; i++)
			{
				if ((m_bAllVertexValuesAllocated && !m_PerVertexValuesExist[pnt.VertID(i)])
					|| (!m_bAllVertexValuesAllocated 
						&& m_VertexToValueMap.find(pnt.VertID(i))==m_VertexToValueMap.end()))
					return pnt.VertID(i);
			}
		}
	}
	return -1;
}

void ModelPerVertexValues::AllocatePerVertexStorage()
{
	m_bAllVertexValuesAllocated = true;
	
	if (m_PerVertexValues==NULL)
		m_PerVertexValues = new double[NEntries()];
	
	if (m_PerVertexValuesExist==NULL)
		m_PerVertexValuesExist = new bool[NEntries()];	
	
	for (int i=0; i<NEntries(); i++)
	{
		m_PerVertexValues[i] = 0;
		m_PerVertexValuesExist[i] = false;
	}	
}

void ModelPerVertexValues::AllocatePerVertexStorageFillFromKnown()
{
	if (m_bAllVertexValuesAllocated)
		return;
	
	m_bAllVertexValuesAllocated = true;
	AllocatePerVertexStorage();
	m_iNumValuesNotSet = NEntries();
	m_fMaxValue = -FLT_MAX;
	m_fMinValue = FLT_MAX;	
	for (std::map<int, double>::iterator iter = m_VertexToValueMap.begin(); 
		 iter != m_VertexToValueMap.end(); iter++)
		SetVertexValue(iter->first, iter->second);
	m_VertexToValueMap.clear();
}

int ModelPerVertexValues::NEntries() const 
{
	if (m_bPointCloudOnly)
		return m_pModel->PointCloud()->NumPoints();
	else 
		return m_pModel->GetMesh()->NVertices();
}

///////// RENDERING

//bool ModelPerVertexValues::InitRenderingColors(ModelValuesInterpolatorDescriptor & interpolate, 
//											   const VKString & collectionName, int corrID, 
//											   const VKString & corrValueName, const ModelPoint & selectedFrom,
//											   double bottomPercentile, double topPercentile, 
//											   bool isocontours, VisWindow * window)
//{
//	assert(false);	// TODO: implement in ModelCorr - as draw
//	assert(!selectedFrom.Invalid());
//	Model3D * modelFrom = (Model3D*)selectedFrom.GetModel();
//	Model3D * modelTo = m_pModel;
//	assert(modelTo!=NULL && modelFrom!=NULL);
//	// find CorrValue
//	ModelCorr * corr = modelFrom->GetCorr(collectionName, corrID);
//	if (corr==NULL)
//		return false;
//	
//	assert(modelTo==corr->ModelTo());
//	assert(modelFrom==corr->ModelFrom());
//	
//	CorrValue * corrValue = corr->GetCorrValueByName(corrValueName);
//	if (corrValue==NULL)
//		return false;
//	
//	// to set?
//	std::vector<ModelPoint> toPoints;
//	corr->GetCorrespondingPoints(selectedFrom, &toPoints);
//	
//	for (int i=0; i<(int)toPoints.size(); i++)
//	{
//		ModelPoint trgPnt = toPoints[i];
//		// TODO: if value already is set - replace
//		SetVertexValue(trgPnt, corrValue->Value(selectedFrom, trgPnt));
//	}
//	
//	Interpolate(interpolate);
//	
//	if (!InitRenderingColors(bottomPercentile, topPercentile, isocontours, window))
//		return false;
//	return true;
//}

bool ModelPerVertexValues::InitColors(VisWindow * window)
{
	return InitColorsPercentileRange(window->m_fMinPercentile, window->m_fMaxPercentile, 
									 window->m_bDefaultDrawIsocontours, window);
}

bool ModelPerVertexValues::InitColorsPercentileRange(double bottomPercentile, double topPercentile, 
											   bool isocontours, VisWindow * window)
{
	std::vector<double> vals;
	assert(m_pModel!=NULL);
	const R3Mesh * mesh = m_pModel->GetMesh();
	GetAllValues(vals);
		
	if (vals.size() == 0)
	{
		std::cout<<"[WARNING] ModelPerVertexValues: No known values!"<<std::endl;
		return false;
	}
	
	std::sort(vals.begin(), vals.end());
	double bottomVal = vals[vkMin(vals.size() * bottomPercentile, vals.size()-1)];
	double topVal = vals[vkMin(vals.size() * topPercentile, vals.size()-1)];
	
	return InitColorsValueRange(bottomVal, topVal, isocontours, window);
}

bool ModelPerVertexValues::InitColorsValueRange(double bottomVal, double topVal, bool isocontours,
												VisWindow * window)
{
	assert(m_pModel!=NULL);
	const R3Mesh * mesh = m_pModel->GetMesh();

	// clamp values to minimal / maximal
	if (m_bAllVertexValuesAllocated)
	{
		assert(m_PerVertexValues!=NULL);
		int nPoints = (m_bPointCloudOnly) ? m_pModel->PointCloud()->NumPoints() : mesh->NVertices();
		
		for (int i=0; i<nPoints; i++)
			if (m_PerVertexValuesExist[i])
			{
				if (m_PerVertexValues[i]<bottomVal)
					m_PerVertexValues[i] = bottomVal;
				else if (m_PerVertexValues[i]>topVal)
					m_PerVertexValues[i] = topVal;
			}
	}
	else 
	{
		for (std::map<int, double>::iterator iter = m_VertexToValueMap.begin();
			 iter != m_VertexToValueMap.end(); iter++)
		{
			if (iter->second < bottomVal)
				iter->second = bottomVal;
			else if (iter->second > topVal)
				iter->second = topVal;
		}
	}
	
	
	// initialize colors
	if (m_bAllVertexValuesAllocated)	// per vertex colors
	{
		m_pRenderR = new ModelPerVertexValues(m_pModel, m_bPointCloudOnly, false);
		m_pRenderG = new ModelPerVertexValues(m_pModel, m_bPointCloudOnly, false);
		m_pRenderB = new ModelPerVertexValues(m_pModel, m_bPointCloudOnly, false);
		PointSet * pntCloud = m_pModel->PointCloud();
		int nPoints = m_bPointCloudOnly ? pntCloud->NumPoints() : mesh->NVertices();
		for (int i=0; i<nPoints; i++)
		{
			double r, g, b;
			assert(m_PerVertexValuesExist[i]);			
			window->MapScalarToColor(m_PerVertexValues[i], bottomVal, topVal, r, g, b);
			m_pRenderR->SetVertexValue(i, r);
			m_pRenderG->SetVertexValue(i, g);
			m_pRenderB->SetVertexValue(i, b);
		}
	}
	else 
		assert(false);
	
	// TODO: isocontours
	return true;
}

void ModelPerVertexValues::GetAllValues(std::vector<double> & vals)
{
	assert(m_pModel!=NULL);
	const R3Mesh * mesh = m_pModel->GetMesh();

	// find minimal / maximal values
	if (m_bAllVertexValuesAllocated)
	{
		assert(m_PerVertexValues!=NULL);
		int nPoints = (m_bPointCloudOnly) ? m_pModel->PointCloud()->NumPoints() : mesh->NVertices();
		
		for (int i=0; i<nPoints; i++)
			if (m_PerVertexValuesExist[i])
				vals.push_back(m_PerVertexValues[i]);
	}
	else 
	{
		for (std::map<int, double>::iterator iter = m_VertexToValueMap.begin();
			 iter != m_VertexToValueMap.end(); iter++)
			vals.push_back(iter->second);
	}	
}

void ModelPerVertexValues::SetValues(const VKString & pointSet, double val )
{
	PointSet * pntSet = m_pModel->GetPointSet(pointSet);
	if (pntSet==NULL)
		pntSet = m_pModel->PointCloud();
	for (int i=0; i<pntSet->NumPoints(); i++)
		SetVertexValue(pntSet->GetPoint(i), val);
}

void ModelPerVertexValues::Interpolate(CollectionIdentityPointsets * interpolator)
{
	assert(interpolator!=NULL);
	
	// min/max shouldn't change, right?
	//	m_fMinValue = FLT_MAX;
	//	m_fMaxValue = -FLT_MAX;
	int nentries = NEntries();
	assert(interpolator->m_NFromPnts==nentries);
	for (int i=0; i<nentries; i++)
	{
		assert(interpolator->m_aNumCorrs[i]==1);
		int corrID = interpolator->m_aCorrs[i][0];
		m_PerVertexValuesExist[i] = true;
		bool ok;
		int corrVtxID = interpolator->m_PntsTo->GetPoint(corrID).NearestVertex(&ok);
		assert(ok);
		m_PerVertexValues[i] = m_PerVertexValues[corrVtxID];
	}
	m_iNumValuesNotSet = 0;	
}

void ModelPerVertexValues::Interpolate(const VKString & interpolatorCollection)
{
	assert(m_bAllVertexValuesAllocated);
	CollectionIdentityPointsets * interpolator = NULL;
//	CorrValueIdentityPointsets * interpValue = NULL;
	
	if (interpolatorCollection=="auto")
		interpolator = m_pModel->VerticesFromPointsInterpolator();
	else
		interpolator = m_pModel->InterpolatorMap(interpolatorCollection);
	
	Interpolate(interpolator);
}

//void ModelPerVertexValues::Interpolate(const ModelValuesInterpolatorDescriptor & interpolate)
//{
//	assert(m_bAllVertexValuesAllocated);
//	if (interpolate.m_Interpolate == ModelValuesInterpolatorDescriptor::MODEL_INTERPOLATE_SETTO0)
//	{
//		int nPoints;
//		if (m_bPointCloudOnly)
//			nPoints = m_pModel->PointCloud()->NumPoints();
//		else
//			nPoints = m_pModel->GetMesh()->NVertices();
//		
//		for (int i=0; i<nPoints; i++)
//		{
//			if (!m_PerVertexValuesExist[i])
//			{
//				m_PerVertexValuesExist[i] = true;
//				m_PerVertexValues[i] = 0;
//			}
//		}
//	}
//	else if (interpolate.m_Interpolate == ModelValuesInterpolatorDescriptor::MODEL_INTERPOLATE_NEAREST_NEIGHBOR)
//	{
//		ModelDistance * distance = m_pModel->GetModelDistance(interpolate.m_sDistanceMetricName);
//		if (distance==NULL)
//			distance = m_pModel->DefaultEuclideanDistance();
//
//		PointSet * fromSet = m_pModel->GetPointSet(interpolate.m_sInterpolateFromSet);
//		if (fromSet==NULL)
//			fromSet = m_pModel->PointCloud();
//
//		PointSet * toSet = m_pModel->GetPointSet(interpolate.m_sInterpolateToSet);
//		if (toSet==NULL)
//			toSet = m_pModel->GetPointSet("AllVertices");
//		assert(distance!=NULL && fromSet!=NULL && toSet!=NULL);
//
//		for (int i=0; i<toSet->NumPoints(); i++)
//		{
//			ModelPoint pTo = toSet->GetPoint(i);
//			ModelPoint bestPnt = distance->GetClosestPoint(pTo, interpolate.m_sInterpolateFromSet);
//			assert(!bestPnt.Invalid());
//			SetVertexValue(pTo, GetInterpolatedValue(bestPnt));
//		}
//	}
//	else 
//		assert(false);
//}

void ModelPerVertexValues::FillColor(int vertexID, GLfloat *material)
{
	if (m_pRenderR!=NULL && m_pRenderG!=NULL && m_pRenderB!=NULL)
	{
		material[0] = m_pRenderR->m_PerVertexValues[vertexID];
		material[1] = m_pRenderG->m_PerVertexValues[vertexID];
		material[2] = m_pRenderB->m_PerVertexValues[vertexID];
	}
}

void ModelPerVertexValues::Draw(VisWindow * window)
{
	if (m_pRenderR!=NULL && m_pRenderG!=NULL && m_pRenderB!=NULL)
	{
	//	m_pModel->RenderingParams()->LoadMyCamera(subwindow);
		glDisable(GL_LIGHTING);
		if (!m_bPointCloudOnly)		// render vertices
		{
			static GLfloat material[4];
			window->StandardSurfaceColor(material);

		    glEnable(GL_LIGHTING);
			glEnable(GL_POLYGON_SMOOTH);
			glShadeModel(GL_SMOOTH);
			glDisable(GL_CULL_FACE);
			glColor3d(1.,1., 1.);
			glDisable(GL_TEXTURE);

			glBegin(GL_TRIANGLES);
			const R3Mesh * mesh = m_pModel->GetMesh(); 
			for (int i=0; i<mesh->NFaces(); i++)
			{
				for (int v=0; v<3; v++)
				{
					R3MeshVertex * vtx = mesh->VertexOnFace(mesh->Face(i), v);
					R3Point vtxPos = mesh->VertexPosition(vtx);
					int vID = mesh->VertexID(vtx);
					material[0] = m_pRenderR->m_PerVertexValues[vID];
					material[1] = m_pRenderG->m_PerVertexValues[vID];
					material[2] = m_pRenderB->m_PerVertexValues[vID];
					//glColor3d(m_pRenderR->m_PerVertexValues[vID], 
					//		  m_pRenderG->m_PerVertexValues[vID],
					//		  m_pRenderB->m_PerVertexValues[vID]);
					glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, material);
					R3Vector norm = mesh->VertexNormal(vtx);
					glNormal3d(norm.X(), norm.Y(), norm.Z());
					glVertex3d(vtxPos.X(), vtxPos.Y(), vtxPos.Z());
				}
			}
			glEnd();
		}
		else
		{
			glBegin(GL_POINTS);
			PointSet * pntCld = m_pModel->PointCloud();
			for (int i=0; i<pntCld->NumPoints(); i++)
			{
				R3Point vtxPos = pntCld->GetPoint(i).GetPosition();
				glColor3d(m_pRenderR->m_PerVertexValues[i], 
						  m_pRenderG->m_PerVertexValues[i],
						  m_pRenderB->m_PerVertexValues[i]);
				glVertex3d(vtxPos.X(), vtxPos.Y(), vtxPos.Z());
			}
			glEnd();
		}
	//	m_pModel->RenderingParams()->RestoreCameraAfterLoad(subwindow);
	}
}

void ModelPerVertexValues::ReadValues(std::ifstream & textStream)
{	
	assert(textStream.is_open());
	VKString text;
	text.readToken(textStream);
	assert(text=="PerVertexValuesFlags");
	bool bPntCld, bAllVtxAlloc;
	textStream>>bPntCld>>bAllVtxAlloc;
	assert(m_bPointCloudOnly==bPntCld);
	assert(m_bAllVertexValuesAllocated==bAllVtxAlloc);
	
	while (true)
	{
		assert(!textStream.eof());
		int id;
		textStream>>id;
		if (id==-1)
			break;
		double val;
		textStream>>val;
		SetVertexValue(id, val);
//		if (val==-1)
//			std::cout<<"Reading VAlue: "<<id<<" "<<val<<std::endl;
	}
	textStream>>m_fMinValue>>m_fMaxValue;	
}

void ModelPerVertexValues::WriteValues(std::ofstream & textStream)
{
	assert(textStream.is_open());
	textStream<<"PerVertexValuesFlags "<<m_bPointCloudOnly<<" "<<m_bAllVertexValuesAllocated<<"\n";
	if (m_bAllVertexValuesAllocated)
	{
		int nPoints = -1;
		if (m_bPointCloudOnly)
			nPoints = m_pModel->PointCloud()->NumPoints();
		else
			nPoints = m_pModel->GetMesh()->NVertices();
		for (int i=0; i<nPoints; i++)
			if (m_PerVertexValuesExist[i])
				textStream<<i<<" "<<m_PerVertexValues[i]<<" ";
		textStream<<"-1\n";
	}
	else
	{
		for (std::map<int, double>::iterator iter = m_VertexToValueMap.begin(); 
			 iter != m_VertexToValueMap.end(); iter++)
			textStream<<iter->first<<" "<<iter->second<<" ";
		textStream<<"-1\n";
	}
	textStream<<m_fMinValue<<" "<<m_fMaxValue<<"\n";	
}

double ModelPerVertexValues::MinValue()
{
	return m_fMinValue;
}

double ModelPerVertexValues::MaxValue()
{
	return m_fMaxValue;
}

