/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#include "VKString.h"
#include "VKStringList.h"
#include "ModelPoint.h"
#include "ModelPerVertexValues.h"

class Model3D; 
class ModelDatabase;

#ifndef __ModelValue_H
#define __ModelValue_H

class ModelValue
{
	public:
		ModelValue(Model3D * model, const VKString & name, 
				   bool pointCloudOnly, bool allocateAsNeeded);
	
	// what if mesh changed
		virtual ModelValue * CopyModelValue(R3Mesh * oldMesh, R3Mesh * newMesh, 
											std::vector<int> & newMeshVtxToOldMeshVtx,
											std::vector<int> & newMeshVtxToOldMeshFace);

		virtual ModelPerVertexValues * CreateCopyOfPerVertexValues() const;
		virtual void CopyPerVertexValuesTo(ModelPerVertexValues * trg) const;

		virtual ModelValue * CreateCopyOfModelValue(const VKString & newName) const;
		virtual void CopyValuesTo(ModelValue * trg) const;
	
		virtual void TakeMaxValues(ModelValue * modelValues);
	
	// for colors / textures / etc...
		virtual void AddRelatedValue(const VKString & relatedValue);
		virtual VKString GetRelatedValue(int i);
		virtual int NumRelatedValues();
	
		virtual void DrawAsMesh(VisWindow * window);
		virtual void DrawAsPointCloud(const VKString & pntSet);
	
		virtual ~ModelValue();
		
		virtual double Value(const ModelPoint & pnt);
		virtual void SetValue(const ModelPoint & pnt, double value);
		
		//virtual void Extremas(SurfaceSampleSet * minSet, 
		//						SurfaceSampleSet * maxSet, double minDistance);
		
		virtual double MaxValue();
		virtual double MinValue();
	//	virtual void PrecomputeVertexValues();
		
	//	virtual void WriteARFFPropertyFile(const VKString & filename);
	//	virtual void WriteFeature(const VKString & featureName);
	//	virtual bool LoadFeature(const VKString & featureName);
		
	//	virtual VKString GetFeatureName() = 0;
	
	//	virtual void SaveModelValue(ModelDatabase * db);
	
	//	static bool LoadModelValue(const VKString & modelname, 
	//							   const VKString & modelValuename,
	//							   ModelDatabase * db);
	
		virtual VKString GetModelValueType() const;
	
		virtual void WriteModelValue(ModelDatabase * db);
		virtual void WriteModelValueChild(std::ofstream & textStream);
		virtual void WriteModelValuePrecomputed(std::ofstream & textStream);
		static bool ReadModelValue(const VKString & modelName, 
								   const VKString & valueName, 
								   ModelDatabase * db);
		virtual void ReadModelPrecomputed(std::ifstream & textStream);
		virtual void ReadModelValueChild(std::ifstream & textStream);		
	
	protected:
		VKStringList m_RelatedValues;
	
		virtual double CalculateValue(const ModelPoint & pnt) const;
		
		ModelPerVertexValues * m_pRenderableValues;
		bool m_bNeedToUpdateColors;
	
		ModelPerVertexValues * m_pPerVertexValues;
		bool m_bPntCldOnly;
		bool m_bAllocateAsNeeded;
		
		Model3D * m_pModel;
		
		VKString m_sModelValueName;
};

#endif


