/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#include "ModelValueLabel.h"
#include "Model3D.h"
#include "VisWindow.h"
#include "CollectionIdentityPointsets.h"

ModelValueLabel::ModelValueLabel(Model3D * model, const VKString & name, bool pointCloudOnly)
: ModelValue(model, name, pointCloudOnly, false)
{
	m_iLabelingDepth = -1;
	for (int i=0; i<model->GetMesh()->NFaces(); i++)
		m_FaceToSegmentID.push_back(-1);
}

ModelValueLabel::~ModelValueLabel()
{
}

ModelValue * ModelValueLabel::CopyModelValue(R3Mesh * oldMesh, R3Mesh * newMesh, 
											 std::vector<int> & newMeshVtxToOldMeshVtx,
											 std::vector<int> & newMeshVtxToOldMeshFace)
{		
	// defined by: AssignLabel, AssignSegmentID, AssignSegmentIDToFace(==AssignFaceLabels())
	ModelValueLabel * newLabels = new ModelValueLabel(m_pModel, m_sModelValueName, m_bPntCldOnly);
	for (int i=0; i<(int)m_mSegmentToLabelMap.size(); i++)
		newLabels->AssignLabel(i, GetSegmentLabel(i));
	
	for (int i=0; i<newMesh->NVertices(); i++)
	{
		assert(i < (int)newMeshVtxToOldMeshFace.size());
		//int segID = (int)m_pPerVertexValues->GetValueAtVertex(newMeshVtxToOldMeshVtx[i]);
		
		int segID = -1;
		if (newMeshVtxToOldMeshFace[i] < (int)m_FaceToSegmentID.size()
			&& newMeshVtxToOldMeshFace[i]>=0)
			segID = m_FaceToSegmentID[newMeshVtxToOldMeshFace[i]];
		newLabels->AssignSegmentID(ModelPoint(i, m_pModel), segID);
	}
	
	for (int i=0; i<newMesh->NFaces(); i++)
	{
		R3MeshFace * face = newMesh->Face(i);
		int newV1 = newMesh->VertexID(newMesh->VertexOnFace(face, 0));
		int newV2 = newMesh->VertexID(newMesh->VertexOnFace(face, 1));
		int newV3 = newMesh->VertexID(newMesh->VertexOnFace(face, 2));
		int seg1 = m_FaceToSegmentID[newMeshVtxToOldMeshFace[newV1]];
		int seg2 = m_FaceToSegmentID[newMeshVtxToOldMeshFace[newV2]];
		int seg3 = m_FaceToSegmentID[newMeshVtxToOldMeshFace[newV3]];
		if (seg1==seg2 || seg1==seg3)
			newLabels->AssignSegmentIDToFace(i, seg1);
		else if (seg2==seg3)
			newLabels->AssignSegmentIDToFace(i, seg2);
		else
			newLabels->AssignSegmentIDToFace(i, seg1);
	}
	
	//newLabels->AssignFaceLabels();
	return newLabels;
}

//void ModelValueLabel::AssignFacesFromPointSet(const VKString & pointSet, 
//											  const VKString & distanceMetric)
//{
//	//	ModelValuesInterpolatorDescriptor interpolator;
//	//	interpolator.m_sDistanceMetricName = distanceMetric;
//	//	interpolator.m_sInterpolateFromSet = pointSet;
//	//	interpolator.m_sInterpolateToSet = "AllVertices";
//	//	m_pPerVertexValues->Interpolate(interpolator);
//	m_pPerVertexValues->Interpolate("auto");
//	AssignFaceLabels();
//}

void ModelValueLabel::AssignFaceLabels()
{
	CollectionIdentityPointsets * facesInterpolator = m_pModel->FacesFromPointsInterpolator();
	assert(facesInterpolator!=NULL);
	PointSet * pntCloud = m_pModel->PointCloud();
	PointSet * pntsFaces = m_pModel->GetPointSet("AllFaces");
	assert(pntsFaces!=NULL);
	double * weights;
	int * corrs;
	int nCorrs;
	CollectionIdentityPointsets::PntSetType pntType;
	for (int i=0; i<pntsFaces->NumPoints(); i++)
	{
		facesInterpolator->QuickCorrsWithWeights(pntsFaces->GetPoint(i), &corrs, 
												 &weights, &nCorrs, &pntType);
		assert(nCorrs==1);
		AssignSegmentIDToFace(i, GetSegmentID(pntCloud->GetPoint(corrs[0])));
	}
//	for (int i=0; i<m_pModel->GetMesh()->NFaces(); i++)
//	{
//		R3MeshFace * face = m_pModel->GetMesh()->Face(i);
//		R3MeshVertex * v = m_pModel->GetMesh()->VertexOnFace(face);
//		int faceID = m_pModel->GetMesh()->FaceID(face);
//		int vID = m_pModel->GetMesh()->VertexID(v);
//		AssignSegmentIDToFace(faceID, GetSegmentID(ModelPoint(vID, m_pModel)));
//	}
}

int ModelValueLabel::GetSegmentID(const ModelPoint & pnt)
{
	return (int)Value(pnt);
}

VKString ModelValueLabel::GetSegmentLabel(int segID)
{
	//assert(segID>=0 && segID < (int)m_mSegmentToLabelMap.size());
	if (segID < 0 || segID >= (int)m_mSegmentToLabelMap.size())
		return "Unknown Segment";
	int labelID = m_mSegmentToLabelMap[segID];
	if (labelID==-1)
		return "Unknown Label";
	assert(labelID>=0 && labelID < m_Labels.count());
	return m_Labels[labelID];	
}

VKString ModelValueLabel::GetLabel(const ModelPoint & pnt)
{
	int segID = GetSegmentID(pnt);
	if (segID == -1)
		return "Unknown";
	return GetSegmentLabel(segID);	
}

int ModelValueLabel::NumSegments()
{
	return (int)m_mSegmentToLabelMap.size();
}

int ModelValueLabel::NumLabels()
{
	return (int)m_mLabelToSegmentMap.size();
}

VKString ModelValueLabel::GetModelValueType() const
{
	return "ModelValueLabel";
}

void ModelValueLabel::AssignLabel(int segmentID, const VKString & label)
{
//	std::cout<<"\nSeg "<<segmentID<<" label="<<label.c_str()<<std::endl;
	VKString locLabel = label;
	if (locLabel=="")
		locLabel = "UNKNOWN_LABEL";
	int labelID = m_Labels.containsGetID(locLabel);
	if (labelID==-1)
	{
		labelID = m_Labels.count();
		m_Labels.push_back(locLabel);
	}
	
	while (segmentID >= (int)m_mSegmentToLabelMap.size())
		m_mSegmentToLabelMap.push_back(-1);
		
	m_mSegmentToLabelMap[segmentID] = labelID;
	
	while (labelID >= (int)m_mLabelToSegmentMap.size())
		m_mLabelToSegmentMap.push_back(std::vector<int>());
	
	m_mLabelToSegmentMap[labelID].push_back(segmentID);
}

void ModelValueLabel::AssignSegmentID(const ModelPoint & pnt, int segmentID)
{
	m_pPerVertexValues->SetVertexValue(pnt, segmentID);
}

void ModelValueLabel::AssignSegmentIDToFace(int faceID, int segmentID)
{
	assert(faceID>=0 && faceID < (int)m_FaceToSegmentID.size());
	m_FaceToSegmentID[faceID] = segmentID;
}

void ModelValueLabel::DrawAsMesh(VisWindow * window)
{
	std::vector<R3Point> segmentCentroids;
	std::vector<int> segmentCentroidsNorm;
	
	glShadeModel(GL_FLAT);
	const R3Mesh * mesh = m_pModel->GetMesh();
	glBegin(GL_TRIANGLES);
	for (int i=0; i<(int)m_FaceToSegmentID.size(); i++)
	{
		assert(i < mesh->NFaces());
		R3MeshFace * face = mesh->Face(i);
		R3Point p1 = mesh->VertexPosition(mesh->VertexOnFace(face, 0));
		R3Point p2 = mesh->VertexPosition(mesh->VertexOnFace(face, 1));
		R3Point p3 = mesh->VertexPosition(mesh->VertexOnFace(face, 2));

		int segmentID = m_FaceToSegmentID[i];
		
		double r = 1, g = 0, b = 0;
		if (segmentID!=-1)
		{
			if (NumSegments() < 10)
				window->MapIntToRadicalColor(segmentID, r, g, b);
			else
				window->MapIntToColor(segmentID, r, g, b);
		}
		glColor3d(r, g, b);
		glVertex3d(p1.X(), p1.Y(), p1.Z());
		glVertex3d(p2.X(), p2.Y(), p2.Z());
		glVertex3d(p3.X(), p3.Y(), p3.Z());

		while (segmentID >= (int)segmentCentroids.size())
		{
			segmentCentroids.push_back(R3Point(0,0,0));
			segmentCentroidsNorm.push_back(0);
		}
		if (segmentID!=-1)
		{
			assert(segmentID!=-1);
			segmentCentroids[segmentID] += p1+p2+p3;
			segmentCentroidsNorm[segmentID] += 3;
		}
	}
	glEnd();
	
	for (int i=0; i<(int)segmentCentroids.size(); i++)
	{
		if (segmentCentroidsNorm[i] != 0)	// there are points in segment
		{
			VKString label = GetSegmentLabel(i);
			R3Point p = segmentCentroids[i] / segmentCentroidsNorm[i];
			window->GetRenderer()->RenderText(p, label, 
											  1., 0, 0, VisWindowRenderer::TEXT_SIZE_LARGE);
		}
	}

	// TODO: draw edges between bounding regions
//	glBegin(GL_LINES);
//	glLineWidth(2);
//	glColor3d(.8, .8, .8);
//	for (int i=0; i<mesh->NEdges(); i++)
//	{
//		if ()
//	}
//	glEnd();
	
	glShadeModel(GL_SMOOTH);
}

void ModelValueLabel::DrawAsPointCloud(const VKString & pntSet)
{
	// TODO
}

void ModelValueLabel::WriteModelValueChild(std::ofstream & textStream)
{
	textStream<<"Labels "<<m_Labels.count()<<"\n";
	
	for (int i=0; i<m_Labels.count(); i++)
		textStream<<m_Labels[i].c_str()<<"\n";
	
	textStream<<"Segs "<<NumSegments()<<"\n";
	for (int i=0; i<NumSegments(); i++)
		textStream<<m_mSegmentToLabelMap[i]<<" ";
	textStream<<"\n";
	
	textStream<<"NFaces "<<m_FaceToSegmentID.size()<<"\n";
	for (int i=0; i<(int)m_FaceToSegmentID.size(); i++)
		textStream<<m_FaceToSegmentID[i]<<" ";
	textStream<<"\n";
	
	textStream<<"LabelDepth "<<m_iLabelingDepth<<"\n";
	textStream<<"OtherLabelings "<<m_AllLabelings.count()<<"\n";
	for (int i=0; i<m_AllLabelings.count(); i++)
		textStream<<m_AllLabelings[i].c_str()<<" ";
}

void ModelValueLabel::ReadModelValueChild(std::ifstream & textStream)
{
	VKString text;
	text.readToken(textStream);
	assert(text=="Labels");
	int NLabels;
	textStream>>NLabels;
	text.readLine(textStream);
	for (int i=0; i<NLabels; i++)
	{
		m_Labels.push_back(VKString());
		m_Labels[i].readLine(textStream);
		m_mLabelToSegmentMap.push_back(std::vector<int>());
	}
	
	text.readToken(textStream);
	assert(text=="Segs");
	
	int NSegs;
	textStream>>NSegs;
	
	for (int i=0; i<NSegs; i++)
	{
		int labelID;
		textStream>>labelID;
		m_mSegmentToLabelMap.push_back(labelID);
		//assert(labelID >= 0 && labelID<(int)m_mLabelToSegmentMap.size());
		if (labelID>=0)
		{
			assert(labelID<(int)m_mLabelToSegmentMap.size());
			m_mLabelToSegmentMap[labelID].push_back(i);
		}
	}
	
	text.readToken(textStream);
	assert(text=="NFaces");

	int NFaces;
	textStream>>NFaces;
	if (NFaces!=m_pModel->GetMesh()->NFaces())
	{
		std::cout<<"[ERROR] Values defined for: "<<NFaces<<", but mesh has ";
		std::cout<<m_pModel->GetMesh()->NFaces()<<std::endl;
		assert(NFaces==m_pModel->GetMesh()->NFaces());
	}
	for (int i=0; i<NFaces; i++)
		textStream>>m_FaceToSegmentID[i];
	
	text.readToken(textStream);
	assert(text=="LabelDepth");
	textStream>>m_iLabelingDepth;
	
	text.readToken(textStream);
	assert(text=="OtherLabelings");
	
	int NOtherLabelings;
	textStream>>NOtherLabelings;
	for (int i=0; i<NOtherLabelings; i++)
	{
		m_AllLabelings.push_back("");
		m_AllLabelings[i].readToken(textStream);
	}
}

double ModelValueLabel::CalculateValue(const ModelPoint & sample) const
{
	return -1.;	// unknown segment, unknown label
}

int ModelValueLabel::LabelingDepth()
{
	return m_iLabelingDepth;
}

VKStringList ModelValueLabel::GetAllLabelings()
{
	return m_AllLabelings;
}

void ModelValueLabel::AddAllLabelings(int myDepth, VKStringList & allLabelings)
{
	m_iLabelingDepth = myDepth;
	m_AllLabelings = allLabelings;
}


