/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __ModelValueLabel_H
#define __ModelValueLabel_H

#include "ModelValue.h"

class ModelValueLabel : public ModelValue
{
	public:
		ModelValueLabel(Model3D * model, const VKString & name, bool pointCloudOnly = false);
		virtual ~ModelValueLabel();
	
		ModelValue * CopyModelValue(R3Mesh * oldMesh, R3Mesh * newMesh, 
									std::vector<int> & newMeshVtxToOldMeshVtx,
									std::vector<int> & newMeshVtxToOldMeshFace);
	
		virtual void AssignLabel(int segmentID, const VKString & label);
		virtual void AssignSegmentID(const ModelPoint & pnt, int segmentID);
		virtual void AssignSegmentIDToFace(int faceID, int segmentID);
//		virtual void AssignFacesFromPointSet(const VKString & pointSet, 
//											 const VKString & distanceMetric);
	
		virtual void DrawAsMesh(VisWindow * window);
		virtual void DrawAsPointCloud(const VKString & pntSet);
	
		virtual int GetSegmentID(const ModelPoint & pnt);
		virtual VKString GetSegmentLabel(int segmentID);
		virtual VKString GetLabel(const ModelPoint & pnt);
		virtual int NumSegments();
		virtual int NumLabels();
		
		virtual VKString GetModelValueType() const;
	
		virtual void WriteModelValueChild(std::ofstream & textStream);
		virtual void ReadModelValueChild(std::ifstream & textStream);
	
		// For labelings parsed from a scene graph
		virtual int LabelingDepth();
		virtual VKStringList GetAllLabelings();
		virtual void AddAllLabelings(int myDepth, VKStringList & allLabelings);
		virtual void AssignFaceLabels();
	
	protected:
		
		virtual double CalculateValue(const ModelPoint & sample) const;
		
		int m_iLabelingDepth;
		VKStringList m_AllLabelings;
	
		VKStringList m_Labels;
		std::vector<int> m_mSegmentToLabelMap;
		std::vector<std::vector<int> > m_mLabelToSegmentMap;
			
		std::vector<int> m_FaceToSegmentID;
		
};

#endif

