/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#include "PipelineStepApplication.h"
#include "AppLabeling.h"
#include "AppPartSelection.h"
#include "AppCreateDB.h"
#include "AppBenchmark.h"

PipelineStepApplication::PipelineStepApplication(AnalysisPipeline & pipeline, 
												 const VKString & moduleName)
: m_Pipeline(pipeline)
{
	m_sModuleName = moduleName;
	m_sAppName = m_Pipeline.m_Params.GetStrValue(moduleName, "MEMORY_AppName");
	//m_pDB = m_Pipeline.GetDB();
}

bool PipelineStepApplication::CaptureKeyPressed(DatabaseVisWindow * window,
												const char key, int mouseX, int mouseY)
{
	// can implement in child (if true is returned signal is not sent anywhere)
	return false;
}

bool PipelineStepApplication::CaptureMouseButtonEvent(DatabaseVisWindow * window, int x, int y, 
													  VisWindow::MouseButton mouseButtonID, 
													  VisWindow::MouseButtonState newState)
{
	// can implement in child (if true is returned signal is not sent anywhere)
	return false;
}

bool PipelineStepApplication::CaptureMouseMotionEvent(DatabaseVisWindow * window,
													  int x, int y, int dx, int dy)
{
	// can implement in child (if true is returned signal is not sent anywhere)
	return false;
}


void PipelineStepApplication::Draw(DatabaseVisWindow * window)
{
	// can implement in child
}

void PipelineStepApplication::InteractAppEnabled(DatabaseVisWindow * window)
{
	// can save drawing flags - and re-set them to something else
}

void PipelineStepApplication::InteractAppDisabled(DatabaseVisWindow * window)
{
	// can save drawing flags - and re-set them to something else
}

void PipelineStepApplication::WriteResults()
{
	VKString writeFile = m_Pipeline.GetMainDB()->GetAppFile(m_sAppName);
	m_Pipeline.GetMainDB()->CreateDirsOnPathToFile(writeFile);
	
	std::ofstream textStream(writeFile.c_str());
	assert(textStream.is_open());
	textStream<<"AppType "<<GetAppType().c_str()<<"\n";
	WriteResultsChild(textStream);
	textStream<<"AppAlign "<<"\n";
}


bool PipelineStepApplication::ReadResults(AnalysisPipeline & pipeline, 
										  const VKString & moduleName)
{
	VKString appName = pipeline.m_Params.GetStrValue(moduleName, "MEMORY_AppName");
	VKString readFile = pipeline.GetDB()->GetAppFile(appName);
	std::ifstream textStream(readFile.c_str());
	if (!textStream.is_open())
		return false;
	VKString text;
	text.readToken(textStream);
	assert(text=="AppType");
	text.readToken(textStream);
	PipelineStepApplication * app = NULL;
	if (text=="AppLabeling")
		app = new AppLabeling(pipeline, moduleName);
	else if (text=="AppPartSelection")
		app = new AppPartSelection(pipeline, moduleName);
    else if (text=="AppCreateDB")
        app = new AppCreateDB(pipeline, moduleName);
    else if (text=="AppBenchmark")
        app = new AppBenchmark(pipeline, moduleName);
	assert(app!=NULL);
	app->ReadResultsChild(textStream);
	text.readToken(textStream);
	assert(text=="AppAlign");
	pipeline.m_Apps[appName] = app;
	return true;
}

void PipelineStepApplication::WriteResultsChild(std::ofstream & textStream)
{
	// implement in a child
}

bool PipelineStepApplication::ReadResultsChild(std::ifstream & textStream)
{
	// implement in a child
	return true;
}






