/***************************************************************************
 *   Copyright (C) 2008 by Vladimir Kim									   *
 *   Author: Vladimir Kim (vk@princeton.edu)							   *
 ***************************************************************************/

#ifndef __PointDescriptor_H
#define __PointDescriptor_H

#include "Sortable.h"
#include "gaps.h"

#include "PointSet.h"
#include "ModelPoint.h"

class PointDescriptor;
class DatabaseVisWindow;
class ModelDatabase;

class PointDescriptorsContainer
{
	public:	
		PointDescriptorsContainer(int nDescriptors);
		virtual double PointSimilarity(int p1, int p2);	
		virtual void AddDescriptor(int id, PointDescriptor * descriptor);
		virtual PointDescriptor * GetDescriptor(int id);
		
	protected:
		int m_iNDescriptors;
		PointDescriptor ** m_aPointDescriptors;
};

class PointDescriptor
{
	public:
		PointDescriptor();
		virtual ~PointDescriptor();
		
		virtual void Draw(const ModelPoint & pnt, DatabaseVisWindow * window, Model3D * model);
		virtual double MeasureSimilarity(const PointDescriptor & compareToMe) = 0;
		virtual double MeasureSimilarityGivenTransform(const PointDescriptor & p2CompareToMe, 
													   const R3Affine & alignP2ToMe);
		
		
		static void ConstructGaussSplat2D(double dx, double dy, double sigma1, double sigma2,
										  double ** gaussSplat, int * splatNX, int * splatNY);
		static int GetID(int i, int j, int n1, int n2);

		int m_iClassID;	
		
};

class PointDescriptorFeatureVector : public PointDescriptor
{
	public:
		PointDescriptorFeatureVector();
		virtual ~PointDescriptorFeatureVector();
		double MeasureSimilarity(const PointDescriptor & compareToMe);

		static void QuickBatchSphericalHarmonics(ModelDatabase * db, 
												 const VKString & pntsName,
												 bool forceRecompute,
						     std::map<Model3D*, std::vector<PointDescriptor*> > & output);
	
	protected:
		LinAlgVectorReal * m_pDescriptorVals;
		
		
};


#endif


